/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#ifndef STRUCTGEO_H
#define STRUCTGEO_H

struct GEO
{
  /* GENERAL INFORMATION *************************************************** */
  int VerQty;			/* Geometry quantities of vertex, face, edge */
  int FaceQty;			/* and polyhedron. */
  int EdgeQty;
  int PolyQty;

  int DummyVerQty;
  int DummyEdgeQty;
  double **DomainSize;

  /* VERTEX INFORMATION **************************************************** */
  /* For vertex i (i=1...VerQty): */
  double **VerCoo;		/* VerCoo[i][j] (j=0...2) are the 3 coordinates. */
  int *VerEdgeQty;		/* VerEdgeQty[i] is the quantity of parent edges of */
  /* vertex i */
  int **VerEdgeNb;		/* VerEdge[i][j] (j=0...) are the numbers of the parent */
  /* edges */
  int **VerBound;		/* VerBound[i][j] (j=0...3) are respectively the qty */
  /* of boundaries of which vertex i belongs (0 to 3) */
  /* and their numbers. */
  int *VerState;		/* VerState[i] is the state of vertex i: */
  /* 0: unmodified, -1: deleted, */
  /* >0: modified, = vertex weight (i.e. qty of vertices */
  /*     it represents). */
  double *VerLength;		/* VerLength[i] is the characteristic mesh length of */
  /* vertex i. */

  /* EDGE INFORMATION ****************************************************** */
  /* For edge i (i=1...EdgeQty): */
  int **EdgeVerNb;		/* VerNb[i][j] (j=0,1) are the numbers of the two */
  /* vertices on which edge is based. */
  int *EdgeFaceQty;		/* EdgeFaceQty[i] is the quantity of parent faces of */
  /* vertex i */
  int **EdgeFaceNb;		/* EdgeFaceNb[i][j] (j=0...) are the numbers of the */
  /* parent faces */
  double *EdgeLength;		/* EdgeLength[i] is the length of edge i */
  int *EdgeState;		/* EdgeState[i]=0 means edge i still exists (=1 else) */
  int *EdgeDel;			/* EdgeDel[i]=0 means edge i can be deleted; */
  /* -1 -> to large */

  int *EdgeNodeQty;		/* EdgeNodeQty[i] is the qty of intermediate nodes. */
  /* EdgeNodeQty[i]!=0 means edge i is discretized. */

  double **EdgeNodes;		/* Sets of edge nodes: normalized coordinates */
  double **EdgeNodeCls;		/* Cls on nodes */
  int *EdgeNode1Id;		/* EdgeSeg1Id[i] is the id of the 1st segment of  */
  int *EdgeSeg1Id;		/* EdgeSeg1Id[i] is the id of the 1st segment of  */
  /* edge i. */

  /* FACE INFORMATION ****************************************************** */
  /* For face i (i=1...FaceQty): */
  int **FaceGerm;		/* FaceGerm[i][j] (j=0,1) are the numbers of the two */
  /* parent germs (or polyhedra). */
  int *FaceVerQty;		/* FaceVerQty[i] is the quantity of face vertices */
  int **FaceVerNb;		/* FaceVerNb[i][j] (j=1...FaceVerQty[i]) are the */
  /* numbers of the vertices. */
  int **FaceEdgeNb;		/* FaceEdgeNb[i][j] (j=1...FaceVerQty[i]) are the */
  /* numbers of the edges. */
  int **FaceEdgeOri;		/* FaceEdgeOri[i][j] (j=1...FaceVerQty[i]) are the */
  /* orientations of the edges. */
  double **FaceEq;		/* FaceEq[i][j] (j=0...3) are the four equation */
  /* parameters: */
  /* FaceEq[i][1]*X1+FaceEq[i][2]*X2+FaceEq[i][3]*X3=FaceEq[i][0] */
  int *FaceState;		/* FaceState[i]=0 means face i is unmodified */
  /* FaceState[i]>0 means face i is modified */
  /* FaceState[i]==-1 means face i is useless/deleted */
  int *FacePt;			/* point for the interpolation of the face */
  /* if facept>0, facept=nb of the face ver */
  /* if facept=0, the barycentetr is considered */
  double **FacePtCoo;		/* if FacePt=0, coo of the pt (=bary) */
  double *FaceFF;		/* ff of the face */
  int *FaceBound;		/* boundary number of the face (none --> 0). */

  int *FaceDis;			/* FaceDis[i]=1 means face i is discretized */
  struct NODES *FaceNodes;	/* Stuff for the non-planar faces */
  struct MESH *FaceM0D;
  struct MESH *FaceM1D;
  struct MESH *FaceM2D;

  /* POLYHEDRON INFORMATION ************************************************ */
  /* For polyhedron i (i=1...PolyQty): */
  int *PolyFaceQty;		/* PolyFaceQty[i] is the quantity of faces. */
  int **PolyFaceNb;		/* PolyFaceNb[i][j] (j=1...PolyFaceQty[i]) are the */
  /* numbers of the faces. */
  int **PolyFaceOri;		/* PolyFaceNb[i][j] (j=1...PolyFaceQty[i]) are the */
  /* orientations of the faces. */

};
typedef struct GEO GEO;

#endif /* STRUCTGEO_H */
