/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#ifndef STRUCTTESS_H
#define STRUCTTESS_H

struct TESS
{
  /* GENERAL INFORMATION *************************************************** */
  int VerQty;			/* Tessellation quantities of vertex, face, edge */
  int FaceQty;			/* and polyhedron. */
  int EdgeQty;
  int PolyQty;

  /* VERTEX INFORMATION **************************************************** */
  /* For vertex i (i=1...VerQty): */
  int **VerGerm;		/* VerGerm[i][j] (j=0...3) are the numbers of the four */
  /* parent germs */
  int **VerEdge;		/* VerEdge[i][j] (j=0...3) are the numbers of the four */
  /* parent edges */
  double **VerCoo;		/* VerCoo[i][j] (j=0...2) are the 3 coordinates. */
  int *VerOfDomain;		/* VerOfDomain[0...7] are the number of the vertices */
  /* which are vertices of the cubic domain: they have */
  /* only 3 parent edges. */

  /* EDGE INFORMATION ****************************************************** */
  /* For edge i (i=1...EdgeQty): */
  int **EdgeVerNb;		/* VerNb[i][j] (j=0,1) are the numbers of the two */
  /* vertices on which edge is based. */
  double *EdgeLength;		/* EdgeLength[i] is the length of edge i */

  /* FACE INFORMATION ****************************************************** */
  /* For face i (i=1...FaceQty): */
  int **FaceGerm;		/* FaceGerm[i][j] (j=0,1) are the numbers of the two */
  /* parent germs. */
  double **FaceEq;		/* FaceEq[i][j] (j=0...3) are the four equation */
  /* parameters: */
  /* FaceEq[i][1]*X1+FaceEq[i][2]*X2+FaceEq[i][3]*X3=FaceEq[i][0] */
  int *FaceVerQty;		/* FaceVerQty[i] is the quantity of face vertices */
  int **FaceVerNb;		/* FaceVerNb[i][j] (j=1...FaceVerQty[i]) are the */
  /* numbers of the vertices. */
  int **FaceEdgeNb;		/* FaceEdgeNb[i][j] (j=1...FaceVerQty[i]) are the */
  /* numbers of the edges. */
  int **FaceEdgeOri;		/* FaceEdgeOri[i][j] (j=1...FaceVerQty[i]) are the */
  /* orientations of the edges. */

  /* POLYHEDRON INFORMATION ************************************************ */
  /* For polyhedron i (i=1...PolyQty): */
  double **CenterCoo;		/* CenterCoo[i][j] (j=0...2) are the 3 coordinates of */
  /* the center (or "parent germ"). */
  int *PolyVerQty;		/* PolyVerQty[i] is the quantity of vertices. */
  int **PolyVerNb;		/* PolyVerNb[i][j] (j=1...PolyVerQty[i]) are the */
  /* numbers of the vertices. */
  int *PolyEdgeQty;		/* PolyEdgeQty[i] is the quantity of edges. */
  int **PolyEdgeNb;		/* PolyEdgeNb[i][j] (j=1...PolyEdgeQty[i]) are the */
  /* numbers of the edges. */
  int *PolyFaceQty;		/* PolyFaceQty[i] is the quantity of faces. */
  int **PolyFaceNb;		/* PolyFaceNb[i][j] (j=1...PolyFaceQty[i]) are the */
  int **PolyFaceOri;		/* PolyFaceOri[i][j] (j=1...PolyFaceQty[i]) are the */
  /* orientations of the faces: 1 if the face normal */
  /* goes out of the polyhedron, -1 else. */

  /*struct POLY Domain;         Domain is the tesselated polyhedron. */
};
typedef struct TESS TESS;

#endif /* STRUCTTESS_H */
