/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "OperUtils.h"

int
oneDIntMin (int *board, int beg, int end)
{
  int min;
  int i;

  min = board[beg];
  for (i = beg + 1; i <= end; i++)
    if (board[i] < min)
      min = board[i];

  return min;
}

int
oneDIntMax (int *board, int beg, int end)
{
  int max;
  int i;

  max = board[beg];
  for (i = beg + 1; i <= end; i++)
    if (board[i] > max)
      max = board[i];

  return max;
}

int
oneDIntSum (int *board, int beg, int end)
{
  int sum;
  int i;

  sum = 0;
  for (i = beg; i <= end; i++)
    sum += board[i];

  return sum;
}

int
oneDIntNbOfThisVal (int *board, int beg, int end, int val)
{
  int qty = 0;
  int i;

  for (i = beg; i <= end; i++)
    if (board[i] == val)
      qty++;

  return qty;
}

float
oneDFloatMin (float *board, int beg, int end)
{
  float min;
  int i;

  min = board[beg];
  for (i = beg + 1; i <= end; i++)
    if (board[i] < min)
      min = board[i];

  return min;
}

float
oneDFloatMax (float *board, int beg, int end)
{
  float max;
  int i;

  max = board[beg];
  for (i = beg + 1; i <= end; i++)
    if (board[i] > max)
      max = board[i];

  return max;
}

double
oneDDoubleMin (double *board, int beg, int end)
{
  double min;
  int i;

  min = board[beg];
  for (i = beg + 1; i <= end; i++)
    if (board[i] < min)
      min = board[i];

  return min;
}

double
oneDDoubleMax (double *board, int beg, int end)
{
  double max;
  int i;

  max = board[beg];
  for (i = beg + 1; i <= end; i++)
    if (board[i] > max)
      max = board[i];

  return max;
}

double
oneDDoubleSum (double *board, int beg, int end)
{
  double sum;
  int i;

  sum = 0;
  for (i = beg; i <= end; i++)
    sum += board[i];

  return sum;
}

int
oneDDoubleMaxPos (double *board, int beg, int end)
{
  double max;
  int pos;
  int i;

  pos = beg;
  max = board[beg];
  for (i = beg + 1; i <= end; i++)
    if (board[i] > max)
    {
      max = board[i];
      pos = i;
    }

  return pos;
}


int
oneDDoubleAMaxPos (double *board, int beg, int end)
{
  double max;
  int pos;
  int i;

  pos = beg;
  max = fabs (board[beg]);
  for (i = beg + 1; i <= end; i++)
    if (fabs (board[i]) > max)
    {
      max = fabs (board[i]);
      pos = i;
    }

  return pos;
}

void
oneDDoubleScale (double *board, int beg, int end, double factor)
{
  int i;

  for (i = beg; i <= end; i++)
    board[i] *= factor;

  return;
}

void
oneDIntRevElts (int *board, int n1, int n2)
{
  int tmp;

  tmp = board[n1];
  board[n1] = board[n2];
  board[n2] = tmp;

  return;
}

void
oneDDoubleRevElts (double *board, int n1, int n2)
{
  double tmp;

  tmp = board[n1];
  board[n1] = board[n2];
  board[n2] = tmp;

  return;
}

/* oneDRotPos supplies the position situated at near position of pos. */
int
oneDRotPos (int beg, int end, int pos, int near)
{
  pos += near;

  while (pos > end)
    pos -= (end - beg + 1);

  while (pos < beg)
    pos += (end - beg + 1);

  return pos;
}

void
oneDIntRotElts (int *board, int beg, int end, int num)
{
  int i;
  int *copy;

  copy = (int *) oneDDoubleDAlloc_int (end + 1);

  for (i = beg; i <= end; i++)
    copy[i] = board[i];

  for (i = beg; i <= end; i++)
    board[i] = copy[oneDRotPos (beg, end, i, num)];

  return;
}

/* oneDIntEltPos searches value from beg to end in board. It records
 * its position in board and add the value near. If the position such
 * obtained is out of the board (lower than beg or higher than end),
 * it is modified: if position becomes higher than end, its value goes
 * again from beg: end+1<--beg. That is the same for position<beg.
 * 3 4 5 6 7 8 9 10 11 12 13 14 15 (beg=3, end=15)
 * 3--15 19 --> 19-(15-3+1)
 * 3--15 30 --> 30-(15-3+1)-(15-3+1)
 * 3--15  1 --> 1 +(15-3+1)
 * 3--15 -1 --> -1+(15-3+1)
 * 3--15 -10--> -10+(15-3+1)
 */
int
oneDIntEltPos (int *board, int beg, int end, int value, int near)
{
  int i;
  int pos = -1;

  for (i = beg; i <= end; i++)
    if (board[i] == value)
    {
      pos = i;
      break;
    }

  if (pos != -1 && near == 0)
    return pos;

  if (pos != -1)
    return oneDRotPos (beg, end, pos, near);
  else
    return -1;
}

/* The oneDIntSort subroutine sorts the board from element beg */
/* to element end, using the bubble sorting algorithm. */
void
oneDIntSort (int *board, int beg, int end)
{
  int i, j;

  for (i = 1; i <= end - beg; i++)
    for (j = end; j >= beg + i; j--)
      if (board[j - 1] > board[j])
	oneDIntRevElts (board, j - 1, j);

  return;
}

/* The oneDIntCompress subroutine compress the board from element beg */
/* to element end. A previous oneDIntSort step is required. */
int
oneDIntCompress (int *board, int beg, int end)
{
  int i;
  int end2;
  int *board2;

  board2 = oneDDoubleDAlloc_int (end + 1);

  end2 = beg - 1;
  for (i = beg; i <= end - 1; i++)
    if (board[i] != board[i + 1])
    {
      end2++;
      board2[end2] = i;
    }
  end2++;
  board2[end2] = end;

  for (i = beg; i <= end2; i++)
    board[i] = board[board2[i]];
  for (i = end2 + 1; i <= end; i++)
    board[i] = 0;

  oneDIntFree (board2);

  return end2;
}

int
oneDIntSortNCompress (int *board, int beg, int end)
{
  int end2;

  oneDIntSort (board, beg, end);
  end2 = oneDIntCompress (board, beg, end);

  return end2;
}

/* oneDIntDeleteNCompress deletes element value from board and compresses */
/* the resulting board, loop times. */
int
oneDIntDeleteNCompress (int *board, int beg, int end, int value, int loop)
{
  int l;
  int i;
  int qty = 0;
  int pos;

  for (l = 1; l <= loop; l++)
  {
    pos = oneDIntEltPos (board, beg, end, value, 0);
    if (pos != -1)
    {
      for (i = pos; i <= end - 1; i++)
	board[i] = board[i + 1];
      qty++;
    }
    else
      break;
  }

  return qty;
}

int *
oneDIntCat (int *a, int bega, int enda, int *b, int begb, int endb,
	    int begres)
{
  int i;
  int elt;
  int *res;

  res = oneDDoubleDAlloc_int (enda - bega + 1 + endb - begb + 1 + begres);

  elt = begres;
  for (i = bega; i <= enda; i++)
  {
    res[elt] = a[i];
    elt++;
  }
  for (i = begb; i <= endb; i++)
  {
    res[elt] = b[i];
    elt++;
  }

  return res;
}


void
oneDIntCpy (int *a, int bega, int enda, int *b, int begb, int alloc)
{
  int i;
  int qty;

  qty = enda - bega + 1;

  if (alloc == 1)		/* test this! */
    b = oneDDoubleDAlloc_int (begb + qty);

  for (i = 0; i <= qty - 1; i++)
    b[begb + i] = a[bega + i];

  return;
}

int
oneDIntFindNReplace (int *board, int beg, int end, int old, int nnew, int loop)
{
  int i;
  int pos;
  int qty = 0;

  for (i = 1; i <= loop; i++)
  {
    pos = oneDIntEltPos (board, beg, end, old, 0);
    if (pos != -1)
    {
      qty++;
      board[pos] = nnew;
    }
    else
      break;
  }

  return qty;
}

int
oneDIntCommonElts (int *board1, int beg1, int qty1,
		   int *board2, int beg2, int qty2, int **pres, int begres)
{
  int i;
  int tmpqty;
  int *tmp = oneDDoubleDAlloc_int (qty1);

  tmpqty = 0;
  for (i = beg1; i <= qty1 + beg1 - 1; i++)	/* for every board1 elt, checking if it belongs */
  {				/* to board2. */

    /*printf("board1[%d]=%d\n",i,board1[i]); */

    if (oneDIntEltPos (board2, beg2, qty2 + beg2 - 1, board1[i], 0) != -1)
    {
      tmp[tmpqty] = board1[i];
      tmpqty++;
      /*printf("recorded\n"); */
    }
  }

  /*
     printf("tmpqty=%d\n",tmpqty);
     for(i=0;i<=tmpqty-1;i++)
     printf("tmp%d: %d\n",i,tmp[i]);
   */

  (*pres) = oneDDoubleDAlloc_int (begres + tmpqty);
  oneDIntCpy (tmp, 0, tmpqty - 1, (*pres), begres, 0);

  oneDIntFree (tmp);

  return tmpqty;
}

void
oneDDoubleCpy (double *a, int bega, int enda, double *b, int begb, int alloc)
{
  int i;
  int qty;

  qty = enda - bega + 1;

  if (alloc == 1)		/* test this! */
    b = oneDDoubleDAlloc (begb + qty);

  for (i = 0; i <= qty - 1; i++)
    b[begb + i] = a[bega + i];

  return;
}
