/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"PrintGeoAsX.h"

void
PrintGeoAsAsy (int print, int *faces, struct GEO Geo, char *filename)
{
  int i, edge, ver;
  FILE *mast;
  int *FaceList = oneDDoubleDAlloc_int (Geo.FaceQty + 1);
  int *EdgeList = oneDDoubleDAlloc_int (Geo.EdgeQty + 1);
  int *VerList = oneDDoubleDAlloc_int (Geo.VerQty + 1);

  if (print == 0)
    return;

  if (print == 1)
  {
    PrintGeoFaceToPlot (Geo, faces, FaceList);
    PrintGeoEdgeToPlot (Geo, FaceList, EdgeList);
    PrintGeoVerToPlot (Geo, FaceList, VerList);
  }
  else
  {
    for (i = 1; i <= Geo.EdgeQty; i++)
      if (Geo.EdgeState[i] == 0)
      {
	EdgeList[0]++;
	EdgeList[EdgeList[0]] = i;
      }
    for (i = 1; i <= Geo.VerQty; i++)
      if (Geo.VerState[i] >= 0)
      {
	VerList[0]++;
	VerList[VerList[0]] = i;
      }
  }

  /* printing geometry */
  mast = OpenFile (filename, "w");
  fprintf (mast, "import three;\n");
  fprintf (mast, "currentprojection=obliqueX;\n");
  fprintf (mast, "unitsize(10cm);\n");

  for (i = 1; i <= VerList[0]; i++)
  {
    ver = VerList[i];
    fprintf (mast, "triple p%d =(%f, %f, %f);\n", VerList[i],
	     Geo.VerCoo[ver][0], Geo.VerCoo[ver][1], Geo.VerCoo[ver][2]);
  }
  for (i = 1; i <= EdgeList[0]; i++)
  {
    edge = EdgeList[i];
    fprintf (mast, "draw (p%d--p%d);\n", Geo.EdgeVerNb[edge][0],
	     Geo.EdgeVerNb[edge][1]);
  }

  CloseFile (mast, filename, "w");

  oneDIntFree (VerList);
  oneDIntFree (EdgeList);
  oneDIntFree (FaceList);

  return;
}

void
PrintGeoAsMast (int print, int *faces, struct GEO Geo, char *filename)
{
  int i, j, edge, ver;
  FILE *mast;
  int *FaceList = oneDDoubleDAlloc_int (Geo.FaceQty + 1);
  int *EdgeList = oneDDoubleDAlloc_int (Geo.EdgeQty + 1);
  int *VerList = oneDDoubleDAlloc_int (Geo.VerQty + 1);

  if (print == 0)
    return;

  if (print == 1)
  {
    PrintGeoFaceToPlot (Geo, faces, FaceList);
    PrintGeoEdgeToPlot (Geo, FaceList, EdgeList);
    PrintGeoVerToPlot (Geo, FaceList, VerList);
  }
  else
  {
    for (i = 1; i <= Geo.EdgeQty; i++)
      if (Geo.EdgeState[i] == 0)
      {
	EdgeList[0]++;
	EdgeList[EdgeList[0]] = i;
      }
    for (i = 1; i <= Geo.VerQty; i++)
      if (Geo.VerState[i] >= 0)
      {
	VerList[0]++;
	VerList[VerList[0]] = i;
      }
  }

  /* printing geometry */
  mast = OpenFile (filename, "w");
  fprintf (mast, "****master\n");
  fprintf (mast, " ***geometry\n");
  for (i = 1; i <= VerList[0]; i++)
  {
    ver = VerList[i];
    fprintf (mast, "  **point p%d\n", VerList[i]);
    fprintf (mast, "   *position ");
    for (j = 0; j < 3; j++)
      fprintf (mast, " %.12f", Geo.VerCoo[ver][j]);
    fprintf (mast, "\n");
  }
  for (i = 1; i <= EdgeList[0]; i++)
  {
    edge = EdgeList[i];
    fprintf (mast, "  **line l%d\n", edge);
    fprintf (mast, "   *p1 p%d\n", Geo.EdgeVerNb[edge][0]);
    fprintf (mast, "   *p2 p%d\n", Geo.EdgeVerNb[edge][1]);
  }

  fprintf (mast, " ***plotting\n");
  fprintf (mast, "  **default_options 1 1 1\n");
  fprintf (mast, " ***lines3d\n");
  fprintf (mast, "  **enable 0\n");
  fprintf (mast, "****return\n");

  CloseFile (mast, filename, "w");

  oneDIntFree (VerList);
  oneDIntFree (EdgeList);
  oneDIntFree (FaceList);

  return;
}
