/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"Tess2Geo.h"

/* Tess2Geotializing the properties of the vertices ******************* */

void
Tess2GeoVerQty (struct TESS Tess, struct GEO *pGeo)
{
  (*pGeo).VerQty = Tess.VerQty;

  return;
}

void
Tess2GeoVerCoo (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;

  for (i = 1; i <= (*pGeo).VerQty; i++)
    for (j = 0; j <= 2; j++)
      (*pGeo).VerCoo[i][j] = Tess.VerCoo[i][j];

  return;
}

void
Tess2GeoVerEdge (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;

  for (i = 1; i <= (*pGeo).VerQty; i++)
  {
    (*pGeo).VerEdgeQty[i] = 4;
    for (j = 0; j <= 3; j++)
      (*pGeo).VerEdgeNb[i][j] = Tess.VerEdge[i][j];
  }
  for (i = 0; i <= 7; i++)
    (*pGeo).VerEdgeQty[Tess.VerOfDomain[i]] = 3;

  return;
}

void
Tess2GeoVerBound (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;

  for (i = 1; i <= (*pGeo).VerQty; i++)
    for (j = 0; j <= 3; j++)
      if (Tess.VerGerm[i][j] < 0)
      {
	(*pGeo).VerBound[i][0]++;
	(*pGeo).VerBound[i][(*pGeo).VerBound[i][0]] = Tess.VerGerm[i][j];
      }

  return;
}

/*
void Tess2GeoVerState(struct TESS Tess,struct GEO* pGeo)
{

    return;
}
*/

/* Tess2Geotializing the properties of the edges ******************* */

void
Tess2GeoEdgeQty (struct TESS Tess, struct GEO *pGeo)
{
  (*pGeo).EdgeQty = Tess.EdgeQty;

  return;
}

void
Tess2GeoEdgeVerNb (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;

  for (i = 1; i <= (*pGeo).EdgeQty; i++)
    for (j = 0; j <= 1; j++)
      (*pGeo).EdgeVerNb[i][j] = Tess.EdgeVerNb[i][j];

  return;
}

void
Tess2GeoEdgeFace (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;
  int tmpedge;

  for (i = 1; i <= Tess.FaceQty; i++)
    for (j = 1; j <= Tess.FaceVerQty[i]; j++)
    {
      tmpedge = Tess.FaceEdgeNb[i][j];
      (*pGeo).EdgeFaceNb[tmpedge][(*pGeo).EdgeFaceQty[tmpedge]] = i;
      (*pGeo).EdgeFaceQty[tmpedge]++;
    }

  return;
}

void
Tess2GeoEdgeLength (struct GEO *pGeo)
{
  int i;

  for (i = 1; i <= (*pGeo).EdgeQty; i++)
  {
    /*LineHeader(-1); printf("  edge %d/%d\n",i,(*pGeo).EdgeQty); */
    CalcEdgeLength (pGeo, i);
  }

  return;
}

/*
void Tess2GeoEdgeDel(struct TESS Tess,struct GEO* pGeo)
{
    int i;
    double bound;
    
    double mean=0.4313/pow(Tess.PolyQty,0.33333333333333333333);

    bound=0.5*mean;
    for(i=1;i<=(*pGeo).EdgeQty;i++)
	if((*pGeo).EdgeLength[i]>bound)
	    (*pGeo).EdgeDel[i]=-1;

    return;
}
*/


/* Tess2Geotializing the properties of the faces ******************* */

void
Tess2GeoFaceQty (struct TESS Tess, struct GEO *pGeo)
{
  (*pGeo).FaceQty = Tess.FaceQty;

  return;
}

void
Tess2GeoFaceGerm (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;

  for (i = 1; i <= (*pGeo).FaceQty; i++)
    for (j = 0; j <= 1; j++)
      (*pGeo).FaceGerm[i][j] = Tess.FaceGerm[i][j];

  return;
}

void
Tess2GeoFaceVer (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;

  for (i = 1; i <= (*pGeo).FaceQty; i++)
  {
    (*pGeo).FaceVerQty[i] = Tess.FaceVerQty[i];
    (*pGeo).FaceVerNb[i] = oneDDoubleDAlloc_int ((*pGeo).FaceVerQty[i] + 1);

    for (j = 1; j <= (*pGeo).FaceVerQty[i]; j++)
      (*pGeo).FaceVerNb[i][j] = Tess.FaceVerNb[i][j];
  }

  return;
}

/* a preliminar Tess2GeoFaceVer is needed! */
void
Tess2GeoFaceEdge (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;

  for (i = 1; i <= (*pGeo).FaceQty; i++)
  {
    (*pGeo).FaceEdgeNb[i] = oneDDoubleDAlloc_int ((*pGeo).FaceVerQty[i] + 1);
    (*pGeo).FaceEdgeOri[i] = oneDDoubleDAlloc_int ((*pGeo).FaceVerQty[i] + 1);

    for (j = 1; j <= (*pGeo).FaceVerQty[i]; j++)
    {
      (*pGeo).FaceEdgeNb[i][j] = Tess.FaceEdgeNb[i][j];
      (*pGeo).FaceEdgeOri[i][j] = Tess.FaceEdgeOri[i][j];
    }
  }

  return;
}

/* a preliminar Tess2GeoFaceVer is needed! */
/*
void Tess2GeoFaceFF(struct TESS Tess,struct GEO* pGeo)
{
    int i;

    for(i=1;i<=(*pGeo).FaceQty;i++)
	(*pGeo).FaceFF[i]=oneDDoubleDAlloc((*pGeo).FaceVerQty[i]+1);

    return;
}
*/

void
Tess2GeoFaceEq (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;

  for (i = 1; i <= (*pGeo).FaceQty; i++)
    for (j = 0; j <= 3; j++)
      (*pGeo).FaceEq[i][j] = Tess.FaceEq[i][j];

  return;
}

void
Tess2GeoFaceBound (struct GEO *pGeo)
{
  int i;

  for (i = 1; i <= (*pGeo).FaceQty; i++)
    (*pGeo).FaceBound[i] = ThisFaceBound (*pGeo, i);

  return;
}

/* Tess2Geotializing the properties of the faces ******************* */

void
Tess2GeoPolyQty (struct TESS Tess, struct GEO *pGeo)
{
  (*pGeo).PolyQty = Tess.PolyQty;

  return;
}

void
Tess2GeoPolyFace (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;

  for (i = 1; i <= (*pGeo).PolyQty; i++)
  {
    /*sprintf(message,"  poly %d: ",i); */
    /*ut_print_message(-1,message); */
    (*pGeo).PolyFaceQty[i] = Tess.PolyFaceQty[i];
    /*sprintf(message,"%d faces\n",Tess.PolyFaceQty[i]); */
    /*ut_print_message(-1,message); */
    (*pGeo).PolyFaceNb[i] = oneDDoubleDAlloc_int ((*pGeo).PolyFaceQty[i] + 1);
    (*pGeo).PolyFaceOri[i] =
      oneDDoubleDAlloc_int ((*pGeo).PolyFaceQty[i] + 1);

    for (j = 1; j <= (*pGeo).PolyFaceQty[i]; j++)
    {
      /*LineHeader(-1); printf("   face %d/%d\n",j,(*pGeo).PolyFaceQty[i]); */
      (*pGeo).PolyFaceNb[i][j] = Tess.PolyFaceNb[i][j];
      (*pGeo).PolyFaceOri[i][j] = Tess.PolyFaceOri[i][j];
    }
  }

  return;
}
