/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "MeshUtils.h"

void
WriteNSets (int nset, int **FoDNodes, FILE * file)
{
  int i;
  char NsetNames[100][100];
  int **FoDBNodes;
  int **EoDNodes;
  int **EoDBNodes;
  int **CoDNodes;

  if (nset <= 0)
    return;

  if (nset >= 1)
    {
      /* printing faces */
      sprintf (NsetNames[1], "x0");
      sprintf (NsetNames[2], "x1");
      sprintf (NsetNames[3], "y0");
      sprintf (NsetNames[4], "y1");
      sprintf (NsetNames[5], "z0");
      sprintf (NsetNames[6], "z1");

      for (i = 1; i <= 6; i++)
	WriteNSet (NsetNames[i], FoDNodes[i], file);
    }

  if (nset >= 2)
    {
      FoD2EoDNodes (FoDNodes, &EoDNodes);
      EoD2CoDNodes (EoDNodes, &CoDNodes);

      /* printing vertices */
      sprintf (NsetNames[1], "x0y0z0");
      sprintf (NsetNames[2], "x1y0z0");
      sprintf (NsetNames[3], "x0y1z0");
      sprintf (NsetNames[4], "x1y1z0");
      sprintf (NsetNames[5], "x0y0z1");
      sprintf (NsetNames[6], "x1y0z1");
      sprintf (NsetNames[7], "x0y1z1");
      sprintf (NsetNames[8], "x1y1z1");

      for (i = 1; i <= 8; i++)
	WriteNSet (NsetNames[i], CoDNodes[i], file);

      /* printing edges */
      sprintf (NsetNames[1], "y0z0");
      sprintf (NsetNames[2], "y1z0");
      sprintf (NsetNames[3], "y0z1");
      sprintf (NsetNames[4], "y1z1");
      sprintf (NsetNames[5], "x0z0");
      sprintf (NsetNames[6], "x0z1");
      sprintf (NsetNames[7], "x1z0");
      sprintf (NsetNames[8], "x1z1");
      sprintf (NsetNames[9], "x0y0");
      sprintf (NsetNames[10], "x1y0");
      sprintf (NsetNames[11], "x0y1");
      sprintf (NsetNames[12], "x1y1");

      for (i = 1; i <= 12; i++)
	WriteNSet (NsetNames[i], EoDNodes[i], file);
    }

  if (nset >= 3)
    {
      FoDBNodes = oneDpIntDAlloc (7);
      FoD2FoDBNodes (FoDNodes, EoDNodes, &FoDBNodes);
      EoDBNodes = oneDpIntDAlloc (13);
      EoD2EoDBNodes (EoDNodes, CoDNodes, &EoDBNodes);

      /* printing faces body */
      sprintf (NsetNames[1], "x0body");
      sprintf (NsetNames[2], "x1body");
      sprintf (NsetNames[3], "y0body");
      sprintf (NsetNames[4], "y1body");
      sprintf (NsetNames[5], "z0body");
      sprintf (NsetNames[6], "z1body");

      for (i = 1; i <= 6; i++)
	WriteNSet (NsetNames[i], FoDBNodes[i], file);

      /* printing edges body */
      sprintf (NsetNames[1], "y0z0body");
      sprintf (NsetNames[2], "y1z0body");
      sprintf (NsetNames[3], "y0z1body");
      sprintf (NsetNames[4], "y1z1body");
      sprintf (NsetNames[5], "x0z0body");
      sprintf (NsetNames[6], "x0z1body");
      sprintf (NsetNames[7], "x1z0body");
      sprintf (NsetNames[8], "x1z1body");
      sprintf (NsetNames[9], "x0y0body");
      sprintf (NsetNames[10], "x1y0body");
      sprintf (NsetNames[11], "x0y1body");
      sprintf (NsetNames[12], "x1y1body");

      for (i = 1; i <= 12; i++)
	WriteNSet (NsetNames[i], EoDBNodes[i], file);
    }

  /* 
     ZZZ
     Free nsets
   */

  return;
}

void
WriteBC (int bc, struct NODES Nodes, int **FoDNodes, char *filename)
{
  FILE *file = OpenFile (filename, "w");
  int **FoDBNodes;
  int **EoDNodes;
  int **EoDBNodes;
  int **CoDNodes;

  FoD2EoDNodes (FoDNodes, &EoDNodes);
  FoD2FoDBNodes (FoDNodes, EoDNodes, &FoDBNodes);
  EoD2CoDNodes (EoDNodes, &CoDNodes);
  EoD2EoDBNodes (EoDNodes, CoDNodes, &EoDBNodes);

  if (bc == 1) /* Prescribing uniform strain inside the surfaces */
    {
      /* for every face body nodes, writing conditions on the normal disps. */
      WriteBCFaceBody (1, CoDNodes[7][1], Nodes, FoDBNodes, 2, 3, file);	/* x0 */
      WriteBCFaceBody (2, CoDNodes[8][1], Nodes, FoDBNodes, 2, 3, file);	/* x1 */
      WriteBCFaceBody (3, CoDNodes[6][1], Nodes, FoDBNodes, 1, 3, file);	/* y0 */
      WriteBCFaceBody (4, CoDNodes[8][1], Nodes, FoDBNodes, 1, 3, file);	/* y1 */
      WriteBCFaceBody (5, CoDNodes[4][1], Nodes, FoDBNodes, 1, 2, file);	/* z0 */
      WriteBCFaceBody (6, CoDNodes[8][1], Nodes, FoDBNodes, 1, 2, file);	/* z1 */

      WriteBCEdgeBody (1, CoDNodes[2][1], Nodes, EoDBNodes, 1, file);
      WriteBCEdgeBody (2, CoDNodes[4][1], Nodes, EoDBNodes, 1, file);
      WriteBCEdgeBody (3, CoDNodes[6][1], Nodes, EoDBNodes, 1, file);
      WriteBCEdgeBody (4, CoDNodes[8][1], Nodes, EoDBNodes, 1, file);
      WriteBCEdgeBody (5, CoDNodes[3][1], Nodes, EoDBNodes, 2, file);
      WriteBCEdgeBody (6, CoDNodes[7][1], Nodes, EoDBNodes, 2, file);
      WriteBCEdgeBody (7, CoDNodes[4][1], Nodes, EoDBNodes, 2, file);
      WriteBCEdgeBody (8, CoDNodes[8][1], Nodes, EoDBNodes, 2, file);
      WriteBCEdgeBody (9, CoDNodes[5][1], Nodes, EoDBNodes, 3, file);
      WriteBCEdgeBody (10, CoDNodes[6][1], Nodes, EoDBNodes, 3, file);
      WriteBCEdgeBody (11, CoDNodes[7][1], Nodes, EoDBNodes, 3, file);
      WriteBCEdgeBody (12, CoDNodes[8][1], Nodes, EoDBNodes, 3, file);

      /* One more constraint for the lateral deformations to be equal? */
    }


  CloseFile (file, filename, "w");

  return;
}

int
PtElt2d (double *Coo, struct NODES Nodes, struct MESH Mesh)
{
  int i, elt;

  elt = -1;
  for (i = 1; i <= Mesh.EltQty; i++)
    if (InElt2d (Coo, Nodes, Mesh, Mesh.EltNb[i]) == 1)
      {
	elt = Mesh.EltNb[i];
	break;
      }

  return elt;
}


int
InElt2d (double *Coo, struct NODES Nodes, struct MESH Mesh, int EltNb)
{
  int i, j, node, res, EltPos, nodepos;
  double *CentreCoo = oneDDoubleDAlloc (2);
  double **NodesCoo = twoDDoubleDAlloc (5, 2);
  double *SideEq = oneDDoubleDAlloc (3);

  EltPos = Mesh.EltNbInv[EltNb];

  /* Recording coo of the elt nodes and the elt centre */
  for (i = 1; i <= 4; i++)
    {
      node = Mesh.EltNodes[EltPos][i - 1];
      nodepos = Nodes.NodesNbInv[node];
      for (j = 0; j <= 1; j++)
	{
	  NodesCoo[i][j] = Nodes.NodesCoo[nodepos][j];
	  CentreCoo[j] += (NodesCoo[i][j] / 4);
	}
    }

  /* Calculating the equation of the elt segments & checking whether the
   * centre and the studied point are in the same side -- if not, the pt
   * does not belong to the element.
   */
  res = 1;
  for (i = 1; i <= 4; i++)
    {
      Points2Line (NodesCoo[i], NodesCoo[oneDRotPos (1, 4, i, 1)], SideEq);

      if (Side2d (SideEq, CentreCoo - 1) != Side2d (SideEq, Coo - 1))
	{
	  res = -1;
	  break;
	}
    }

  oneDDoubleFree (CentreCoo);
  twoDDoubleFree (NodesCoo, 5);
  oneDDoubleFree (SideEq);

  return res;
}
