/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"ReadMsh.h"

int
ReadNodesHead (FILE * msh)
{
  int nodeqty;

  FTrash (1, msh);

  fscanf (msh, "%d", &nodeqty);
  if (nodeqty == 0)
  {
    ut_print_message (2, "Mesh has not got nodes!\n");
    abort ();
  }

  return nodeqty;
}

void
ReadNodesFoot (FILE * msh)
{
  char foot[1000];

  fscanf (msh, "%s", foot);

  if (strcmp (foot, "$EndNodes") != 0)
    ut_print_message (2, "Reading msh file: error.\n");

  return;
}

void
ReadNodesProp (struct NODES *pNodes, FILE * msh)
{
  int i;

  /* allocating and setting NodesNb & NodesCoo */
  (*pNodes).NodesNb = oneDDoubleDAlloc_int ((*pNodes).NodeQty + 1);
  (*pNodes).NodesCoo = twoDDoubleDAlloc ((*pNodes).NodeQty + 1, 3);

  for (i = 1; i <= (*pNodes).NodeQty; i++)
    ReadThisNode (pNodes, i, msh);

  /* NodesNbMax */
  (*pNodes).NodeNbMax = oneDIntMax ((*pNodes).NodesNb, 1, (*pNodes).NodeQty);

  /* allocating and setting NodesNbInv */
  (*pNodes).NodesNbInv = oneDDoubleDAlloc_int ((*pNodes).NodeNbMax + 1);
  for (i = 1; i <= (*pNodes).NodeQty; i++)
    (*pNodes).NodesNbInv[(*pNodes).NodesNb[i]] = i;

  return;
}


int
ReadEltsHead (FILE * msh)
{
  int totaleltqty;

  FTrash (1, msh);

  fscanf (msh, "%d", &totaleltqty);

  return totaleltqty;
}

int
ReadEltsOfDim (struct MESH *pMesh, int Dimension, int MaxEltQty,
	       int *Residual, FILE * msh)
{
  int *EElsets = oneDDoubleDAlloc_int (MaxEltQty + 1);

  if (Dimension < 0 || Dimension > 3)
  {
    LineHeader (0);
    printf ("Wrong mesh dimension: %d!\n", Dimension);
    abort ();
  }

  (*pMesh).Dimension = Dimension;

  ReadEltsProp (pMesh, EElsets, MaxEltQty, Residual, msh);

  SetElsets (pMesh, EElsets);

  oneDIntFree (EElsets);

  return (*pMesh).EltQty;
}

void
ReadEltsFoot (FILE * msh)
{
  char foot[1000];

  fscanf (msh, "%s", foot);

  if (strcmp (foot, "$EndElements") != 0)
    ut_print_message (2, "Reading msh file: error.\n");

  return;
}
