/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "TessImport.h"

void
TIVerHead (FILE * file)
{
  if (CharScanNComp ("**vertex", file) != 0)
    abort ();

  return;
}

void
TIVerQty (struct TESS *pTess, FILE * file)
{
  fscanf (file, "%d", &(*pTess).VerQty);

  return;
}

void
TIVerProp (struct TESS *pTess, FILE * file)
{
  int i;
  int ver;

  (*pTess).VerGerm = twoDIntDAlloc ((*pTess).VerQty + 1, 4);
  (*pTess).VerEdge = twoDIntDAlloc ((*pTess).VerQty + 1, 4);
  (*pTess).VerCoo = twoDDoubleDAlloc ((*pTess).VerQty + 1, 3);

  for (i = 1; i <= (*pTess).VerQty; i++)
  {
    fscanf (file, "%d", &ver);
    TIVerParGerm (pTess, ver, file);
    TIVerParEdge (pTess, ver, file);
    TIVerCoo (pTess, ver, file);
  }

  return;
}

void
TIEdgeHead (FILE * file)
{
  if (CharScanNComp ("**edge", file) != 0)
    abort ();

  return;
}

void
TIEdgeQty (struct TESS *pTess, FILE * file)
{
  fscanf (file, "%d", &(*pTess).EdgeQty);

  return;
}

void
TIEdgeProp (struct TESS *pTess, FILE * file)
{
  int i;
  int edge;

  (*pTess).EdgeVerNb = twoDIntDAlloc ((*pTess).EdgeQty + 1, 2);

  for (i = 1; i <= (*pTess).EdgeQty; i++)
  {
    fscanf (file, "%d", &edge);
    TIEdgeVerNb (pTess, edge, file);
  }

  return;
}


void
TIFaceHead (FILE * file)
{
  if (CharScanNComp ("**face", file) != 0)
    abort ();

  return;
}


void
TIFaceQty (struct TESS *pTess, FILE * file)
{
  fscanf (file, "%d", &(*pTess).FaceQty);

  return;
}


void
TIFaceProp (struct TESS *pTess, FILE * file)
{
  int i;
  int face;

  (*pTess).FaceGerm = twoDIntDAlloc ((*pTess).FaceQty + 1, 2);
  (*pTess).FaceEq = twoDDoubleDAlloc ((*pTess).FaceQty + 1, 4);
  (*pTess).FaceVerQty = oneDDoubleDAlloc_int ((*pTess).FaceQty + 1);
  (*pTess).FaceVerNb = oneDpIntDAlloc ((*pTess).FaceQty + 1);
  (*pTess).FaceEdgeNb = oneDpIntDAlloc ((*pTess).FaceQty + 1);
  (*pTess).FaceEdgeOri = oneDpIntDAlloc ((*pTess).FaceQty + 1);

  for (i = 1; i <= (*pTess).FaceQty; i++)
  {
    fscanf (file, "%d", &face);
    TIFaceGerm (pTess, face, file);
    TIFaceEq (pTess, face, file);
    TIFaceVerNEdgeProp (pTess, face, file);
  }

  return;
}

void
TIPolyHead (FILE * file)
{
  if (CharScanNComp ("**polyhedron", file) != 0)
    abort ();

  return;
}


void
TIPolyQty (struct TESS *pTess, FILE * file)
{
  fscanf (file, "%d", &(*pTess).PolyQty);

  return;
}

void
TIPolyProp (struct TESS *pTess, FILE * file)
{
  int i;
  int poly;

  (*pTess).CenterCoo = twoDDoubleDAlloc ((*pTess).PolyQty + 1, 3);
  (*pTess).PolyVerQty = oneDDoubleDAlloc_int ((*pTess).PolyQty + 1);
  (*pTess).PolyVerNb = oneDpIntDAlloc ((*pTess).PolyQty + 1);
  (*pTess).PolyEdgeQty = oneDDoubleDAlloc_int ((*pTess).PolyQty + 1);
  (*pTess).PolyEdgeNb = oneDpIntDAlloc ((*pTess).PolyQty + 1);
  (*pTess).PolyFaceQty = oneDDoubleDAlloc_int ((*pTess).PolyQty + 1);
  (*pTess).PolyFaceNb = oneDpIntDAlloc ((*pTess).PolyQty + 1);
  (*pTess).PolyFaceOri = oneDpIntDAlloc ((*pTess).PolyQty + 1);

  for (i = 1; i <= (*pTess).PolyQty; i++)
  {
    fscanf (file, "%d", &poly);
    TICenterCoo (pTess, poly, file);
    TIPolyVerProp (pTess, poly, file);
    TIPolyEdgeProp (pTess, poly, file);
    TIPolyFaceProp (pTess, poly, file);
  }

  return;
}

void
TIDomainHead (FILE * file)
{
  if (CharScanNComp ("**domain", file) != 0)
    abort ();

  return;
}
