/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"mp.h"
#include"version.h"

void NeperHeader (void);
void NeperFoot (void);
void ProgPresentation (char*);

int
main (int argc, char **argv)
{
  int ArgQty, Res;
  int fargc;
  char **fargv = twoDCharDAlloc (100, 100);
  char **ArgList = twoDCharDAlloc (100, 100);
  char *Arg = oneDCharDAlloc (100);
  char *Version;
  char ***module;
  char* inifile = oneDCharDAlloc (1000);

  /* module definitions -------------------------------------------- */
  int ModuleQty = 5;

  int MajorVersion, MinorVersion;
  char PatchVersion[10];
  MajorVersion = MAJOR_VERSION;
  MinorVersion = MINOR_VERSION;
  sprintf (PatchVersion, "%s", PATCH_VERSION);

  Version = oneDCharDAlloc (10);
  sprintf (Version, "%d.%d.%s", MajorVersion, MinorVersion, PatchVersion);

  module = threeDCharDAlloc (ModuleQty + 1, 2, 10);

  sprintf (module[1][0], "-T");
  sprintf (module[2][0], "-MM");
  sprintf (module[3][0], "-FM");
  sprintf (module[4][0], "-O");
  sprintf (module[5][0], "-TR");
  /* --------------------------------------------------------------- */

  if (argc == 1)
    {
      ProgPresentation ("");

      NeperFoot ();
      return 1;
    }

  /* Searching module name (string completion stuff) */
  /* This is the possible argument list. */
  ArgQty = 8;
  sprintf (ArgList[1], "--help");
  sprintf (ArgList[2], "--version");
  sprintf (ArgList[3], "--license");
  sprintf (ArgList[4], "-MM");
  sprintf (ArgList[5], "-FM");
  sprintf (ArgList[6], "-T");
  sprintf (ArgList[7], "-O");
  sprintf (ArgList[8], "-TR");

  Res = StringComp (argv[1], ArgList, ArgQty, Arg);

  if (Res == 1)
    {
      LineHeader (2);
      printf ("Several possibilities for option `%s'.\n", argv[1]);
      BadArguments ();
    }
  else if (Res == -1)
    {
      LineHeader (2);
      printf ("Unknown option `%s'.\n", argv[1]);
      BadArguments ();
    }

  /* Checking whether it is help or version; else it is a module */
  if (argc == 2 && strcmp (Arg, "--help") == 0)
    {
      ProgPresentation ("");
      NeperFoot ();
      return 1;
    }

  if (argc == 2 && strcmp (Arg, "--version") == 0)
    {
      printf ("%s\n", Version);
      return EXIT_SUCCESS;
    }
  
  if (argc == 2 && strcmp (Arg, "--license") == 0)
    {
      ut_print_gplv3 (stdout);
      return EXIT_SUCCESS;
    }

  NeperHeader ();

  sprintf (inifile, "%s/.neperrc", getenv ("HOME"));
  ReadFOptions (inifile, &fargc, fargv, "neper", Arg);

  if (strcmp (Arg, "-T") == 0)
    neper_t (fargc, fargv, argc - 1, argv + 1);
  else if (strcmp (Arg, "-O") == 0)
    neper_o (fargc, fargv, argc - 1, argv + 1);
  else if (strcmp (Arg, "-FM") == 0)
    neper_fm (fargc, fargv, argc - 1, argv + 1);
  else if (strcmp (Arg, "-MM") == 0)
    neper_mm (fargc, fargv, argc - 1, argv + 1);
  else if (strcmp (Arg, "-TR") == 0)
  {
    ut_print_message (0, "Module -TR is not included in the distrib version.\n");
  }

  /* printing program foot */
  NeperFoot ();

  twoDCharFree (fargv, 100);
  twoDCharFree (ArgList, 100);
  oneDCharFree (Version);
  oneDCharFree (Arg);

  return 0;
}

void
NeperHeader ()
{
  printf
    ("\n========================    N   e   P   e   R    =======================\n");
  return;
}

void
NeperFoot ()
{
  printf
    ("========================================================================\n\n");

  return;
}

void
ProgPresentation (char* add)
{
  char message[1000];
  NeperHeader ();

  ut_print_message (0, "A software to build 3D random polycrystals for the FE method.\n");
  sprintf (message, "Version %d.%d.%s%s (%s)\n", MAJOR_VERSION,
	   MINOR_VERSION, PATCH_VERSION, add, PROG_DATE);
  ut_print_message (0, message);
  ut_print_message (0, "<http://neper.sourceforge.net>\n");
  ut_print_message (0, "Copyright (C) 2004-2009, Romain Quey.\n");

  ut_print_message (0, "This program comes with ABSOLUTELY NO WARRANTY; this is free\n");
  ut_print_message (0, "software, and you are welcome to redistribute it under certain\n");
  ut_print_message (0, "conditions; for details, run `neper --license'.\n");

  ut_print_message (0, "Type `info neper' to access the on-line reference manual.\n");

  ut_print_messagewnc (0, 72,
	      "Send comments, suggestions or bug reports to <rquey@users.sourceforge.net>.");

  return;
}
