/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"CreateGeo.h"

void
CreateGeo (struct TESS Tess, struct GEO *pGeo, struct GEOPARA GeoPara,
	   struct IN In)
{
  int i;
  int DelId = 0, DelQty;
  struct GEO GeoC, GeoCC;
  char message[1000];

  /* READING TESSELATION AND WRITING IT AS GEO *************************** */
  Tess2Geo (Tess, pGeo);

  if (In.ffalgo != 1)		/* Meshing 0D & 1D: only needed for ffalgo!=1 */
    {				/* for the evaluation of the generated ff     */
      GeoMeshing0D (pGeo, GeoPara);

      if (In.edgedis == 1)
	GeoMeshing1D (pGeo, GeoPara);
    }

  /* DELETING THE SMALL EDGES (if maxff>0) ******************************* */
  /* if maxff>0, Merging of the geo (mloop loops max) */
  if (GeoPara.maxff > 1e-15 && In.mloop >= 1)
    {
      Geo2Geo (*pGeo, &GeoC, 1);	/* this is for the memory allocation. This */
      Geo2Geo (*pGeo, &GeoCC, 1);	/* is done here for every geo for the member */

      sprintf (message, "Deleting small edges ... (length < %g)",
	       GeoPara.sel);
      MessageWNC (0, 72, message);
      for (i = 1; i <= In.mloop; i++)
	{
	  DelQty =
	    MergeGeo (pGeo, &GeoC, &GeoCC, GeoPara, In, &DelId, i);

	  /* if no edge has been deleted, the deletion process completes. */
	  if (DelQty <= 0)
	    break;
	}
      ut_print_message (0, "Deleted small edges.\n");
    }

  if (TestGeo (*pGeo, In.verbosity) != 0)
    {
      MessageWNC (2, 72,
		  "Sorry, the modified geometry does not match the requirements.  This is likely a bug.  Please send a bug report to <romain.quey@laposte.net>.");
      abort();
    }

  return;
}
