/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"DeleteEdge.h"

int
DeleteEdge (struct GEO *pGeo, struct GEOPARA GeoPara, struct IN In,
	    int edge, int *pver, double *pmaxff, int *pDelId)
{
  int delver, test;

  /* printf("beg of DeleteEdge\n"); */

  (*pDelId)++;

  if (In.verbosity >= 2)
    printf ("\ndeleting edge %5d (%5d) ------------------------------\n",
	    edge, *pDelId);

  UpdateEdgeState (pGeo, edge);
	      (*pGeo).EdgeDel[edge] = 1;

  SearchDelNNewVer (pGeo, edge, &delver, pver);

  /* UpdateFace updates the properties of the parent faces
   * of the edge and more generally of delver.
   */
  if (In.verbosity >= 2)
    ut_print_message (0, "Updating faces.\n");
  UpdateFaceVerNEdge (pGeo, edge, delver, *pver, In.verbosity);
  if (In.verbosity >= 2)
    ut_print_message (0, "Updated faces.\n");

  if (In.verbosity >= 2)
    ut_print_message (0, "Updating ver properties.\n");
  UpdateVer (pGeo, edge, delver, *pver, In.verbosity);
  if (In.verbosity >= 2)
    ut_print_message (0, "Updated ver properties.\n");

  if (In.verbosity >= 2)
    ut_print_message (0, "Updating edge properties.\n");
  UpdateEdgeVerNb (pGeo, delver, *pver, In.verbosity);
  if (In.verbosity >= 2)
    ut_print_message (0, "Updated edge properties.\n");

  if (In.verbosity >= 2)
    ut_print_message (0, "Updating new ver coordinates.\n");
  UpdateVerCoo (pGeo, delver, *pver, In.verbosity);
  if (In.verbosity >= 2)
    ut_print_message (0, "Updated new ver coordinates.\n");

  if (In.verbosity >= 2)
    ut_print_message (0, "Updating edge lengths.\n");
  UpdateEdgeNVerLength (pGeo, GeoPara, In.ffalgo, *pver, In.verbosity);
  if (In.verbosity >= 2)
    ut_print_message (0, "Updated edge lengths.\n");

  if (In.verbosity >= 2)
    ut_print_message (0, "Updating face bounds.\n");
  UpdateFaceBound (pGeo, *pver);
  if (In.verbosity >= 2)
    ut_print_message (0, "Updated face bounds.\n");

  if (In.ffalgo != 1)
    /* This may be replaced by the determination of VerLength
     * for the new vertex only (or a small neighbourhood of it).
     */
    GeoMeshing0D (pGeo, GeoPara);

  if (In.verbosity >= 2)
    ut_print_message (0, "Testing geometry.\n");

  test = TestGeoAroundVer2 (*pGeo, *pver, 0);
  if (test != 0)
    (*pmaxff) = 888.8;
  else
  {
    if (In.verbosity >= 2)
      ut_print_message (0, "Updating face flatness faults.\n");
    (*pmaxff) = UpdateFaceFF (pGeo, GeoPara, In, *pver);
    if (In.verbosity >= 2)
      ut_print_message (0, "Updated face flatness faults.\n");
  }

  if (In.verbosity >= 2)
    {
      LineHeader (0);
      printf ("maxff=%.2f\n", (*pmaxff));
    }
  

  return test;
}
