/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"FaceFF.h"

double
ThisVerFaceFF (struct GEO *pGeo, struct IN In, int ver)
{
  int i;
  double ff, res;
  int face;
  int * Faces = ut_alloc_1d_int (100);
  int *FaceNodes;

  VerFace (*pGeo, ver, Faces);

  res = 0;
  for (i = 1; i <= Faces[0]; i++)
    {
      /*if(verbosity>=3) printf("face %d/%d\n",i,Faces[0]); */
      face = Faces[i];
      if ((*pGeo).FaceVerQty[face] > 3)
	{
	  if (verbosity >= 3)
	    printf ("performing discretization\n");
	  DiscretizeThisFace (pGeo, In.gmsh, face);
	  FaceNodes = GeoFaceNodes (*pGeo, face);
	  InterpolateFaceBody (pGeo, &((*pGeo).FaceNodes[i]), FaceNodes,
			       face);
	  oneDIntFree (FaceNodes);
	  if (verbosity >= 3)
	    printf ("performed discretization\n");

	  if (verbosity >= 3)
	    printf ("performing ThisFaceFF\n");
	  ff =
	    ThisFaceFF ((*pGeo).FaceNodes[face], (*pGeo).FaceM2D[face], face,
			1);
	  if (verbosity >= 3)
	    printf ("performed ThisFaceFF\n");

	  res = DoubleMax (res, ff);
	}
    }

  oneDIntFree (Faces);

  return res;
}

double
ThisVerFaceFF2 (struct GEO *pGeo, struct GEOPARA GeoPara,
		struct IN In, int ver, int interpolation)
{
  int i;
  double res = 0, ff;
  int face, FaceList[100];
  int *FaceNodes;
  FILE *file;
  struct NODES Nodes;
  struct MESH Mesh2D;
  struct MESH Trash0D, Trash1D, Trash3D;
  char *Args = oneDCharDAlloc (1000);

  VerFace (*pGeo, ver, FaceList);

  FaceFFMeshGeo (pGeo, GeoPara, FaceList);

  sprintf (Args, "-2 -v 0 -algo tri tmp.geo");
  CallGmsh (In.gmsh, Args);

  file = OpenFile ("tmp.msh", "R");

  if (verbosity >= 3)
    printf ("Reading Head\n");
  ReadMshHead (file);
  if (verbosity >= 3)
    printf ("Read Head\n");
  if (verbosity >= 3)
    printf ("Reading Nodes\n");
  ReadNodes (&Nodes, file);
  if (verbosity >= 3)
    printf ("Read Nodes\n");
  if (verbosity >= 3)
    printf ("Reading 2DElts\n");
  ReadElts (&Trash0D, &Trash1D, &Mesh2D, &Trash3D, file);
  if (verbosity >= 3)
    printf ("Read 2DElts\n");
  CloseFile (file, "tmp.msh", "R");

  for (i = 1; i <= FaceList[0]; i++)
    {
      face = FaceList[i];

      if (interpolation == 1)
	{
	  FaceNodes = ThisFaceNodes (Mesh2D, face);
	  InterpolateFaceBody (pGeo, &Nodes, FaceNodes, face);
	  oneDIntFree (FaceNodes);
	}

      ff = ThisFaceFF (Nodes, Mesh2D, face, 1);
      (*pGeo).FaceFF[face] = ff;

      res = DoubleMax (res, ff);
    }

  FreeNodes (&Nodes);
  FreeMesh (&Mesh2D);		/* the others mesh structs are empty. */
  /*FreeMesh (&Trash3D); */
  oneDCharFree (Args);

  return res;
}
