/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"FaceFF.h"

double
FFBaryFace (struct GEO *pGeo, int face)
{
  int i, j, k;
  int ver1, ver2, ver3;
  double ff = 180;
  double minff = 180;
  double thisff = 180;
  double **n;
  double *angle;

  n = ut_alloc_2d ((*pGeo).FaceVerQty[face] + 1, 3);
  angle = ut_alloc_1d ((*pGeo).FaceVerQty[face] + 1);

  /* calculation of the barycenter of the face. */
  for (k = 0; k <= 2; k++)
    {
      for (i = 1; i <= (*pGeo).FaceVerQty[face]; i++)
	{
	  ver1 = (*pGeo).FaceVerNb[face][i];
	  (*pGeo).FacePtCoo[face][k] += (*pGeo).VerCoo[ver1][k];
	}
      (*pGeo).FacePtCoo[face][k] /= (*pGeo).FaceVerQty[face];
    }

  /* for face point on each ver of the face, the fff is calculated:
   * fff = maximum angle between 2 triangular parts of the face (see
   * the method of interpolation of the face).
   */
  for (i = 1; i <= (*pGeo).FaceVerQty[face]; i++)
    {
      thisff = 0;

      /* ver i is the face point. */
      ver1 = (*pGeo).FaceVerNb[face][i];

      for (j = 1; j <= (*pGeo).FaceVerQty[face] - 2; j++)
	{
	  ver2 =
	    (*pGeo).
	    FaceVerNb[face][oneDRotPos (1, (*pGeo).FaceVerQty[face], i, j)];
	  ver3 =
	    (*pGeo).
	    FaceVerNb[face][oneDRotPos
			    (1, (*pGeo).FaceVerQty[face], i, j + 1)];

	  TriangleN ((*pGeo).VerCoo[ver1], (*pGeo).VerCoo[ver2],
		     (*pGeo).VerCoo[ver3], n[j], 0);
	}
      for (j = 1; j <= (*pGeo).FaceVerQty[face] - 2; j++)
	for (k = j + 1; k <= (*pGeo).FaceVerQty[face] - 2; k++)
	  {
	    ff = VectorAngle (n[j], n[k], 0, 1);
	    thisff = DoubleMax (thisff, ff);
	  }

      if (thisff < minff)
	{
	  minff = thisff;
	  (*pGeo).FacePt[face] = i;
	}
    }

  thisff = 0;
  for (j = 1; j <= (*pGeo).FaceVerQty[face]; j++)
    {
      ver2 =
	(*pGeo).
	FaceVerNb[face][oneDRotPos (1, (*pGeo).FaceVerQty[face], i, j)];
      ver3 =
	(*pGeo).
	FaceVerNb[face][oneDRotPos (1, (*pGeo).FaceVerQty[face], i, j + 1)];

      TriangleN ((*pGeo).FacePtCoo[face], (*pGeo).VerCoo[ver2],
		 (*pGeo).VerCoo[ver3], n[j], 0);
    }
  for (j = 1; j <= (*pGeo).FaceVerQty[face]; j++)
    for (k = j + 1; k <= (*pGeo).FaceVerQty[face] - 2; k++)
      {
	ff = VectorAngle (n[j], n[k], 0, 1);
	thisff = DoubleMax (thisff, ff);
      }

  if (thisff < minff)
    {
      (*pGeo).FacePt[face] = 0;
      minff = thisff;
    }

  (*pGeo).FaceFF[face] = minff;

  twoDDoubleFree (n, (*pGeo).FaceVerQty[face] + 1);
  oneDDoubleFree (angle);

  return minff;
}
