/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"MergeGeo.h"

int
MergeGeo (struct GEO *pGeo, struct GEO *pGeoC, struct GEO *pGeoCC,
	  struct GEOPARA GeoPara, struct IN In, int *pDelId, int loop)
{
  int i, trash, testgeo;
  int edge, edgeb, newver;
  double length, lengthratio;
  double tmp;
  double maxff, minmaxff = 0;
  int minid = 0;
  int ThisDelQty, DelQty = 0;

  /* printf("beg of MergeGeo\n"); */

  edge = NextEdgeToDel (*pGeo, GeoPara, &lengthratio);
  length = (*pGeo).EdgeLength[edge];

  while (edge != -1)
    {
      ThisDelQty = 0;

      Geo2Geo (*pGeo, pGeoC, 0);

      testgeo =
	DeleteEdge (pGeoC, GeoPara, In, edge, &newver, &maxff, pDelId);

      /* if (testgeo == 0 and) maxff < max, the edge is deleted */
      /* else (maxff > max), we try with another edge */
      /* even if testgeo != 0, the deletion continues:  we let the
       * program a chance to find a suitable solution, and generally it
       * does.
       */

      /* if the max ff is acceptable, we record the modif into Geo
       * else, we re-initialize it.
       */
      if (maxff < GeoPara.maxff) /* also means that testgeo == 0 */
	{
	  ThisDelQty = 1;
	  Geo2Geo (*pGeoC, pGeo, 0);
	}
      else
	{
	  if (In.verbosity >= 2)
	    {
	      printf ("Deleting neighbouring edges, in turn: %d: ",
		      (*pGeoC).VerEdgeQty[newver]);
	      for (i = 0; i <= (*pGeoC).VerEdgeQty[newver] - 1; i++)
		printf ("%d ", (*pGeoC).VerEdgeNb[newver][i]);
	      printf ("\n");
	    }

	  minmaxff = 180;
	  minid = 0;
	  for (i = 0; i < (*pGeoC).VerEdgeQty[newver]; i++)
	    {
	      edgeb = (*pGeoC).VerEdgeNb[newver][i];

	      /* if((*pGeoC).EdgeDel[edgeb]!=-1) */
	      if ((*pGeoC).EdgeLength[edgeb] < 2 * GeoPara.sel - length)
		{
		  Geo2Geo (*pGeoC, pGeoCC, 0);
		  testgeo =
		    DeleteEdge (pGeoCC, GeoPara, In, edgeb, &trash, &tmp,
				pDelId);
		  /* 
		   * if (TestDelStop (In, (*pDelId), *pGeoCC, pGeo) == 1)
		    return -1;
		    */

		  if (testgeo == 0 && tmp < minmaxff)
		    {
		      minmaxff = tmp;
		      minid = edgeb;
		    }
		}
	      else if (In.verbosity >= 3)
		{
		  LineHeader (0);
		  printf ("edgedel %d=-1 --> skipping\n", edgeb);
		}
	    }

	  if (minmaxff < GeoPara.maxff)
	    {
	      ThisDelQty = 2;

	      DeleteEdge (pGeo, GeoPara, In, edge, &newver, &tmp, pDelId);
	      DeleteEdge (pGeo, GeoPara, In, minid, &newver, &tmp, pDelId);
      
	      /* this node has been deleted: edgedel <- 1 */
	      (*pGeo).EdgeDel[minid] = 1;
	    }
	}
      
      /* in all cases, edgedel <- 1 */
      (*pGeo).EdgeDel[edge] = 1;

      DelQty += ThisDelQty;

      if (In.verbosity >= 1)
	PrintDeletion (loop, lengthratio, ThisDelQty, edge, minid, maxff, 
		       minmaxff);

      /* Searching next edge to delete */
      if (In.verbosity >= 2)
	ut_print_message (0, "Searching smallest edge\n");
      edge = NextEdgeToDel (*pGeo, GeoPara, &lengthratio);
      if (In.verbosity >= 2)
	ut_print_message (0, "Searched smallest edge\n");
    }

  /* ForceMerge deletes the edges specified manually 
   * ForceMerge(pGeo,GeoPara);
   */

  /* at the end,  edgedel is reset to 0. */
  for (i = 1; i <= (*pGeo).EdgeQty; i++)
    (*pGeo).EdgeDel[i] = 0;

  /* printf("end of MergeGeo\n"); */

  return DelQty;
}
