/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"TestFace.h"

int
TestFaceReciprocityEdge (struct GEO Geo, int i)
{
  int j;
  int edge;

  if (Geo.FaceVerQty[i] < 3)
    {
      /*
         LineHeader(2);
         printf("Face %d has less than 3 = %d vers/edges!\n",i,Geo.FaceVerQty[i]);
       */
      return 2;
    }

  for (j = 1; j <= Geo.FaceVerQty[i]; j++)
    {
      edge = Geo.FaceEdgeNb[i][j];
      if (oneDIntEltPos
	  (Geo.EdgeFaceNb[edge], 0, Geo.EdgeFaceQty[edge] - 1, i, 0) == -1)
	{
	  /*
	     LineHeader(2);
	     printf("edge %d of face %d = edge %d has not face %d!\n",j,i,edge,i);
	   */
	  return 2;
	}
    }

  return 0;
}

int
TestFaceReciprocityVer (struct GEO Geo, int i)
{
  int j;
  int ver;
  int * faces = ut_alloc_1d_int (100);

  for (j = 1; j <= Geo.FaceVerQty[i]; j++)
    {
      ver = Geo.FaceVerNb[i][j];
      VerFace (Geo, ver, faces);
      if (oneDIntEltPos (faces, 1, faces[0], i, 0) == -1)
	{
	  /*
	     LineHeader(2);
	     printf("ver %d of face %d = ver %d has not face %d!\n",j,i,ver,i);
	   */
	  return 2;
	}
    }

  oneDIntFree (faces);

  return 0;
}

int
TestFaceReciprocityPoly (struct GEO Geo, int i)
{
  int j;
  int poly;

  if (Geo.FaceGerm[i][0] < 0 && Geo.FaceGerm[i][0] < 0)
    {
      /*
         LineHeader(2);
         printf("face %d has 2 non-real poly!\n",i);
       */
      return 2;
    }

  for (j = 0; j <= 1; j++)
    {
      poly = Geo.FaceGerm[i][j];
      if (poly > 0)
	if (oneDIntEltPos
	    (Geo.PolyFaceNb[poly], 1, Geo.PolyFaceQty[poly], i, 0) == -1)
	  {
	    /*
	       LineHeader(2);
	       printf("poly %d of face %d = poly %d has not face %d!\n",j,i,poly,i);
	     */
	    return 2;
	  }
    }

  return 0;
}

/* TestFaceState checks FaceState. If the face is set as modified, at
 * least one of its vertices must be modified. If the face is set as
 * unmodified, all its vertices must be unmodified.
 */
int
TestFaceState (struct GEO Geo, int i)
{
  int j, ver;

  /* if the face has state=0, it is not modified.
   * every of its vertices must be unmodified (state!=0).
   */
  if (Geo.FaceState[i] == 0)
    {
      for (j = 1; j <= Geo.FaceVerQty[i]; j++)
	{
	  ver = Geo.FaceVerNb[i][j];
	  if (Geo.VerState[ver] != 0)
	    return 1;		/* modified --> pb */
	}
      return 0;			/* unmodified --> ok */
    }
  /* if the face has state>0, it is modified.
   * at least one of its vertices must be modified (state!=0).
   */
  else if (Geo.FaceState[i] > 0)
    {
      for (j = 1; j <= Geo.FaceVerQty[i]; j++)
	{
	  ver = Geo.FaceVerNb[i][j];
	  if (Geo.VerState[ver] != 0)
	    return 0;		/* one ver modified --> ok */
	}
      return 1;			/* no ver modified --> pb */
    }
  else
    return 0;			/* del face --> ok (maybe this is useless). */
}

/* TestFaceBound tests whether the face has 3 aligned vertices (on a domain edge)
 * or not. The case where three of the vertices of a face have 2 common bounds is
 * considered as critical.
 */
int
TestFaceBound (struct GEO Geo, int i)
{
  int j, k, l, ver;
  int qty = 0;

  int * Bounds = ut_alloc_1d_int (100);
  int * BB = ut_alloc_1d_int (2);

  Bounds[0] = 0;

  for (j = 1; j <= Geo.FaceVerQty[i]; j++)
    {
      ver = Geo.FaceVerNb[i][j];
      for (k = 1; k <= Geo.VerBound[ver][0]; k++)
	{
	  Bounds[0]++;
	  Bounds[Bounds[0]] = Geo.VerBound[ver][k];
	}
    }
  gsl_sort_int (Bounds + 1, 1, Bounds[0]);
  Bounds[0] = oneDIntCompress (Bounds, 1, Bounds[0]);

  for (j = 1; j <= Bounds[0]; j++)
    for (k = j + 1; k <= Bounds[0]; k++)
      {
	BB[0] = Bounds[j];
	BB[1] = Bounds[k];
	qty = 0;
	for (l = 1; l <= Geo.FaceVerQty[i]; l++)
	  {
	    ver = Geo.FaceVerNb[i][l];
	    if (oneDIntEltPos
		(Geo.VerBound[ver], 1, Geo.VerBound[ver][0], BB[0], 0) != -1
		&& oneDIntEltPos (Geo.VerBound[ver], 1, Geo.VerBound[ver][0],
				  BB[1], 0) != -1)
	      qty++;
	  }

	if (qty >= 3)
	  {
	    for (l = 1; l <= Geo.FaceVerQty[i]; l++)
	      {
		ver = Geo.FaceVerNb[i][l];
		if (oneDIntEltPos
		    (Geo.VerBound[ver], 1, Geo.VerBound[ver][0], BB[0],
		     0) == -1
		    || oneDIntEltPos (Geo.VerBound[ver], 1,
				      Geo.VerBound[ver][0], BB[1], 0) == -1)
		  return 0;
	      }

	    return 2;
	  }
      }

  oneDIntFree (Bounds);
  oneDIntFree (BB);

  return 0;
}
