/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"TestPoly.h"

int
TestPolyReciprocityFace (struct GEO Geo, int i)
{
  int j;
  int face;

  if (Geo.PolyFaceQty[i] < 4)
    {
      MessageWNC (2, 72,
		  "A polyhedron has a wrong amount of faces.  Classically, this is due to a (too) high value of critical length for the small edges.  You should decrease rcl and/or increase pcl (or decrease sel if you are using it).\n");

      abort ();
      return 3;
    }

  for (j = 1; j <= Geo.PolyFaceQty[i]; j++)
    {
      face = Geo.PolyFaceNb[i][j];
      if (oneDIntEltPos (Geo.FaceGerm[face], 0, 1, i, 0) == -1)
	{
	  /*
	     LineHeader(2);
	     printf("face %d of poly %d = face %d has not poly %d!\n",j,i,face,i);
	   */
	  return 3;
	}
    }

  return 0;
}

/* this routines tests if several faces belongs to the same domain
 * boundary, which means that there are coplanar. */

int
TestPolyCoplaneityFace (struct GEO Geo, int i)
{
  int j, face, res = 0;
  int *tmp = ut_alloc_1d_int (Geo.PolyFaceQty[i] + 1);

  tmp[0] = Geo.PolyFaceQty[i];
  for (j = 1; j <= Geo.PolyFaceQty[i]; j++)
    {
      face = Geo.PolyFaceNb[i][j];
      tmp[j] = Geo.FaceBound[face];
    }

  for (j = 1; j <= 6; j++)
    if (oneDIntNbOfThisVal (tmp, 1, tmp[0], -j) > 1)
      {
	res = 3;
	break;
      }

  oneDIntFree (tmp);

  return res;
}
