/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "GeoStuff.h"

void
WriteFFF (struct IN In, struct GEO Geo)
{
  int i;
  FILE *file = OpenFile (In.ff, "w");

  for (i = 1; i <= Geo.FaceQty; i++)
    if (Geo.FaceState[i] > 0)
      fprintf (file, "%d\t%10.6f\n", i, Geo.FaceFF[i]);

  CloseFile (file, In.ff, "w");

  return;
}

/*
   Writing of the modified faces
void WriteMod(struct GEO Geo,char* filename)
{
    int   i,col,modqty=0;
    FILE* out;

    for(i=1;i<=Geo.FaceQty;i++)
	if(Geo.FaceState[i]>0)
	    modqty++;

    out=OpenFile(filename,"w");
    
    fprintf(out,"%d\n",modqty);
    
    col=0;
    for(i=1;i<=Geo.FaceQty;i++)
	if(Geo.FaceState[i]>0)
	    FPrintfWNC(out,i,&col,72);
    fprintf(out,"\n");

    CloseFile(out);

    return;
}
*/

/* Writing of the faces of domain */
void
WriteFoD (struct GEO Geo, char *filename)
{
  int i, j, k, bound, col;
  FILE *out;

  int **fod = ut_alloc_2d_int (7, Geo.FaceQty + 1);

  /* searching */
  for (i = 1; i <= Geo.FaceQty; i++)
    if (Geo.FaceState[i] >= 0 && Geo.FaceBound[i] != 0)
      {
	bound = Geo.FaceBound[i];
	fod[-bound][0]++;
	fod[-bound][fod[-bound][0]] = i;
      }

  /* file writing */
  out = OpenFile (filename, "w");
  for (j = 1; j <= 6; j++)
    {
      fprintf (out, "%d %d\n", j, fod[j][0]);
      col = 0;
      for (k = 1; k <= fod[j][0]; k++)
	FPrintfWNC (out, fod[j][k], &col, 72);
      fprintf (out, "\n");
    }
  CloseFile (out, filename, "w");

  twoDIntFree (fod, 7);

  return;
}

void
WriteEdgel (struct GEO Geo, struct IN In)
{
  int i;
  FILE *out = OpenFile (In.edgel, "w");

  for (i = 1; i <= Geo.EdgeQty; i++)
    if (Geo.EdgeState[i] == 0)
      fprintf (out, "%d %.10f\n", i, Geo.EdgeLength[i]);

  CloseFile (out, In.edgel, "w");

  return;
}

void
WriteAngle (struct GEO Geo, struct IN In)
{
  int i, j;
  int ver;
  double *vect1 = ut_alloc_1d (3);
  double *vect2 = ut_alloc_1d (3);
  double angle;
  FILE *out = OpenFile (In.angle, "w");

  for (i = 1; i <= Geo.FaceQty; i++)
    if (Geo.FaceState[i] != -1)
      for (j = 1; j <= Geo.FaceVerQty[i]; j++)
	{
	  ver = Geo.FaceVerNb[i][j];
	  AngleAtVer (Geo, i, ver, &angle);

	  fprintf (out, "%d-%d %.10f\n", i, j, angle);
	}

  CloseFile (out, In.angle, "w");

  oneDDoubleFree (vect1);
  oneDDoubleFree (vect2);

  return;
}
