/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"WriteGeo.h"

void
WriteThisVer (struct GEO Geo, int ver, FILE * file, int physical)
{
  fprintf (file, "Point(%d) = {%.12f, %.12f, %.12f, %.12f};\n", ver,
	   Geo.VerCoo[ver][0], Geo.VerCoo[ver][1], Geo.VerCoo[ver][2],
	   Geo.VerLength[ver]);

  if (physical == 1)
    fprintf (file, "Physical Point(%d) = {%d};\n", ver, ver);

  return;
}

void
WriteThisVerWMCL (struct GEO Geo, struct GEOPARA GeoPara,
		  int ver, FILE * file, int physical)
{
  double eps = 1e-12;

  fprintf (file, "Point(%d) = {%.12f, %.12f, %.12f,", ver,
	   Geo.VerCoo[ver][0], Geo.VerCoo[ver][1], Geo.VerCoo[ver][2]);

  if (Geo.VerLength[ver] < (1 - eps) * GeoPara.cl)
    fprintf (file, " %.12f};\n", Geo.VerLength[ver]);
  else
    fprintf (file, " MeanCL};\n");

  if (physical == 1)
    fprintf (file, "Physical Point(%d) = {%d};\n", ver, ver);

  return;
}

void
WriteThisEdge (struct GEO Geo, int edge, FILE * file, int physical)
{
  if (physical == 0)
    fprintf (file, "Line(%d) = {%d,%d} ;\n",
	     edge, Geo.EdgeVerNb[edge][0], Geo.EdgeVerNb[edge][1]);
  else
    fprintf (file, "Line(%d) = {%d,%d} ; Physical Line(%d) = {%d};\n",
	     edge, Geo.EdgeVerNb[edge][0], Geo.EdgeVerNb[edge][1], edge,
	     edge);

  return;
}

void
WriteThisDiscreteEdge (struct GEO Geo, int edge, FILE * file)
{
  int i, j;
  int ver1, ver2;
  double * coo = ut_alloc_1d (3);

  ver1 = Geo.EdgeVerNb[edge][0];
  ver2 = Geo.EdgeVerNb[edge][1];

  fprintf (file, "\n/* Tessellation edge %d (%d vers, %d segs): */\n", edge,
	   Geo.EdgeNodeQty[edge], Geo.EdgeNodeQty[edge] + 1);
  /* Adding dummy vertices */
  for (i = 1; i <= Geo.EdgeNodeQty[edge]; i++)
    {
      for (j = 0; j <= 2; j++)
	coo[j] = (1 - Geo.EdgeNodes[edge][i]) * Geo.VerCoo[ver1][j]
	  + Geo.EdgeNodes[edge][i] * Geo.VerCoo[ver2][j];

      fprintf (file, "Point(%d) = {%.12f, %.12f, %.12f, %.12f};\n",
	       Geo.EdgeNode1Id[edge] + i - 1, coo[0], coo[1], coo[2],
	       Geo.EdgeNodeCls[edge][i]);
    }

  /* Writing the dummy segments instead of the whole edge.
   * writing 1st segment:
   */
  fprintf (file, "Line(%d) = {%d,%d} ;\n",
	   Geo.EdgeSeg1Id[edge], Geo.EdgeVerNb[edge][0],
	   Geo.EdgeNode1Id[edge]);

  /* writing intermediate segments
   */
  for (i = 2; i <= Geo.EdgeNodeQty[edge]; i++)
    {
      fprintf (file, "Line(%d) = {%d,%d} ;\n",
	       Geo.EdgeSeg1Id[edge] + i - 1, Geo.EdgeNode1Id[edge] + i - 2,
	       Geo.EdgeNode1Id[edge] + i - 1);

    }

  /* writing last segment */
  fprintf (file, "Line(%d) = {%d,%d} ;\n",
	   Geo.EdgeSeg1Id[edge] + Geo.EdgeNodeQty[edge],
	   Geo.EdgeNode1Id[edge] + Geo.EdgeNodeQty[edge] - 1,
	   Geo.EdgeVerNb[edge][1]);

  oneDDoubleFree (coo);

  return;
}

void
WriteThisFace (struct GEO Geo, int face, FILE * file, int physical)
{
  int i, j;			/* mute variables */
  int EdgeQty;
  int edge, ori;

  EdgeQty = Geo.FaceVerQty[face];
  fprintf (file, "Line Loop(%d) = {", face);

  for (i = 1; i <= EdgeQty - 1; i++)
    {
      ori = Geo.FaceEdgeOri[face][i];
      edge = Geo.FaceEdgeNb[face][i];

      if (Geo.EdgeNodeQty[edge] == 0)
	fprintf (file, "%d,", ori * edge);

      else if (ori == 1)
	for (j = 0; j <= Geo.EdgeNodeQty[edge]; j++)
	  fprintf (file, "%d,", Geo.EdgeSeg1Id[edge] + j);

      else
	for (j = Geo.EdgeNodeQty[edge]; j >= 0; j--)
	  fprintf (file, "-%d,", Geo.EdgeSeg1Id[edge] + j);
    }

  ori = Geo.FaceEdgeOri[face][EdgeQty];
  edge = Geo.FaceEdgeNb[face][EdgeQty];

  if (Geo.EdgeNodeQty[edge] == 0)
    fprintf (file, "%d};\n", ori * edge);
  else if (ori == 1)
    {
      for (j = 0; j <= Geo.EdgeNodeQty[edge] - 1; j++)
	fprintf (file, "%d,", Geo.EdgeSeg1Id[edge] + j);

      fprintf (file, "%d};\n", Geo.EdgeSeg1Id[edge] + Geo.EdgeNodeQty[edge]);
    }
  else
    {
      for (j = Geo.EdgeNodeQty[edge]; j >= 1; j--)
	fprintf (file, "-%d,", Geo.EdgeSeg1Id[edge] + j);

      fprintf (file, "-%d};\n", Geo.EdgeSeg1Id[edge]);
    }


  fprintf (file, "Plane Surface(%d) = {%d}; ", face, face);

  if (physical == 1)
    fprintf (file, "Physical Surface(%d) = {%d};\n", face, face);

  return;
}

void
WriteThisDiscreteFace (struct GEO Geo, int face, FILE * file)
{
  int i, j, node, nodenbinv;

  fprintf (file, "Discrete Surface(%d) = {%d,%d}\n", face,
	   Geo.FaceNodes[face].NodeQty, Geo.FaceM2D[face].EltQty);

  /* list of nodes */
  fprintf (file, "{ ");
  for (i = 1; i <= Geo.FaceNodes[face].NodeQty - 1; i++)
    {
      for (j = 0; j <= 2; j++)
	fprintf (file, "%.15f, ", Geo.FaceNodes[face].NodesCoo[i][j]);
      fprintf (file, "1,0,0,\n");
    }
  i = Geo.FaceNodes[face].NodeQty;
  for (j = 0; j <= 2; j++)
    fprintf (file, "%.15f, ", Geo.FaceNodes[face].NodesCoo[i][j]);
  fprintf (file, "1,0,0 }\n");

  /* list of elements */
  fprintf (file, "{ ");
  for (i = 1; i <= Geo.FaceM2D[face].EltQty - 1; i++)
    {
      fprintf (file, "3, ");
      for (j = 0; j <= 2; j++)
	{
	  node = Geo.FaceM2D[face].EltNodes[i][j];
	  nodenbinv = Geo.FaceNodes[face].NodesNbInv[node];
	  fprintf (file, "%d,", nodenbinv - 1);
	}
      fprintf (file, "\n");
    }
  fprintf (file, "3, ");
  i = Geo.FaceM2D[face].EltQty;
  for (j = 0; j <= 1; j++)
    {
      node = Geo.FaceM2D[face].EltNodes[i][j];
      nodenbinv = Geo.FaceNodes[face].NodesNbInv[node];
      fprintf (file, "%d,", nodenbinv - 1);
    }
  j = 2;
  node = Geo.FaceM2D[face].EltNodes[i][j];
  nodenbinv = Geo.FaceNodes[face].NodesNbInv[node];
  fprintf (file, "%d };\n", nodenbinv - 1);

  fprintf (file, "Physical Surface(%d) = {%d};\n", face, face);

  return;
}

void
WriteThisPoly (struct GEO Geo, int poly, FILE * file)
{
  int i;			/* mute variables */
  int FaceQty;

  FaceQty = Geo.PolyFaceQty[poly];
  fprintf (file, "Surface Loop(%d) = {", poly);
  for (i = 1; i <= FaceQty - 1; i++)
    fprintf (file, "%d,", Geo.PolyFaceOri[poly][i] * Geo.PolyFaceNb[poly][i]);
  fprintf (file, "%d};\n",
	   Geo.PolyFaceOri[poly][i] * Geo.PolyFaceNb[poly][FaceQty]);

  fprintf (file, "Volume(%d) = {%d}; ", poly, poly);
  fprintf (file, "Physical Volume(%d) = {%d};\n", poly, poly);

  return;
}
