/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_fm.h"

void
TreatArg_fm (int fargc, char **fargv, int argc, char **argv,
	     struct IN *pIn, struct GEOPARA *pGeoPara)
{
  char *body;
  int inibody = 0;
  int initess = 0;		/* =1 if tess is initialized. */

  /* Setting default options */
  SetDefaultOptions_fm (pIn, pGeoPara);

  /* Reading options. */
  SetOptions_fm (pIn, pGeoPara, &body, &initess, &inibody, fargc, fargv);
  SetOptions_fm (pIn, pGeoPara, &body, &initess, &inibody, argc, argv);

  /* Testing options */
  if ((*pIn).mesh == 0 && (*pIn).conv == 0 && initess == 0)
    MissingFile ();
  else if (((*pIn).mesh == 1 && (*pIn).conv == 1)
	   || (initess == 1 && (*pIn).mesh == 1)
	   || (initess == 1 && (*pIn).conv == 1))
    {
      BadArguments ();
    }
  else if (((*pIn).mesh == 1 || (*pIn).conv == 1)
           && (*pIn).printmsh == 0 && (*pIn).printgeof == 0)
    {
      BadArguments ();
    }

  if ((*pIn).edgedis == 0 && (*pGeoPara).maxff > 1e-15)
    {
      ut_print_message (2,
	       "When using `-bwcy-edgedis 0', \"-maxff >0\" is disabled.\n");
      BadArguments ();
    }

  /* Writing file names */

  /* Writing body */
  if (inibody == 0)
    {
      body = oneDCharDAlloc (1000);

      if (initess == 1)
	body = GiveBody ((*pIn).tess);
      else if ((*pIn).mesh == 1)
	body = GiveBody ((*pIn).geo);
      else if ((*pIn).conv == 1)
	body = GiveBody ((*pIn).msh);
      else
	BadArguments ();
    }

  /* Writing all file names */
  if (initess == 0)
    (*pIn).tess = AddExtension (body, ".tess");
  if ((*pIn).mesh == 0)
    (*pIn).geo = AddExtension (body, ".geo");
  if ((*pIn).conv == 0)
    (*pIn).msh = AddExtension (body, ".msh");

  (*pIn).geob = AddExtension (body, ".geob");
  (*pIn).ff = AddExtension (body, ".ff");
  (*pIn).edgel = AddExtension (body, ".edgel");
  (*pIn).angle = AddExtension (body, ".angle");
  (*pIn).msh = AddExtension (body, ".msh");
  (*pIn).geof = AddExtension (body, ".geof");
  (*pIn).fod = AddExtension (body, ".fod");
  (*pIn).estat = AddExtension (body, ".estat");
  (*pIn).eangle = AddExtension (body, ".eangle");
  (*pIn).nstat = AddExtension (body, ".nstat");
  (*pIn).oin = AddExtension (body, ".oin");
  (*pIn).mast = AddExtension (body, "-print.mast");
  (*pIn).asy = AddExtension (body, "-print.asy");

  if ((*pIn).mesh == 1)		/* checking if the input files exist. */
    {
      if (FileExists ((*pIn).geo) == 0)
	OpenFile ((*pIn).geo, "r");
      if (FileExists ((*pIn).fod) == 0)
	OpenFile ((*pIn).fod, "r");
    }
  if ((*pIn).conv == 1)		/* checking if the input files exist. */
    {
      if (FileExists ((*pIn).msh) == 0)
	OpenFile ((*pIn).msh, "r");
    }

  if (inibody == 0)
    oneDCharFree (body);

  return;
}

/* rcl2cl sets cl from rcl */
extern void
rcl2cl (struct TESS Tess, struct GEOPARA *pGeoPara)
{
  int i;
  double mindist;
  double **size = ut_alloc_2d (3, 2);

  (*pGeoPara).cl = (*pGeoPara).rcl * MeanEdgeLength (Tess);

  BoundingBox (Tess, size);

  /* Calculating the minimum distance: */
  mindist = 1e32;
  for (i = 0; i <= 2; i++)
    mindist = DoubleMin (mindist, size[i][1] - size[i][0]);

  if ((*pGeoPara).cl > mindist)
    {
      LineHeader (1);
      printf ("Changing cl from %f to %f.\n", (*pGeoPara).cl, 0.99 * mindist);
      (*pGeoPara).cl = 0.99 * mindist;
    }

  twoDDoubleFree (size, 3);

  return;
}

void
BoundingBox (struct TESS Tess, double **size)
{
  int i, j, ver;

  /* Searching the tessellation bounding box */
  size[0][0] = size[1][0] = size[2][0] = 1e32;
  size[0][1] = size[1][1] = size[2][1] = -1e32;

  for (i = 0; i <= 7; i++)
    {
      ver = Tess.VerOfDomain[i];
      for (j = 0; j <= 2; j++)
	{
	  size[j][0] = DoubleMin (size[j][0], Tess.VerCoo[ver][j]);
	  size[j][1] = DoubleMax (size[j][1], Tess.VerCoo[ver][j]);
	}
    }

  return;
}

double
MeanEdgeLength (struct TESS Tess)
{
  double **size = ut_alloc_2d (3, 2);
  double vol;

  BoundingBox (Tess, size);

  /* Calculating the volume (assuming that the tessellation is a
   * parallelepiped).
   */
  vol =
    (size[0][1] - size[0][0]) * (size[1][1] - size[1][0]) * (size[2][1] -
							     size[2][0]);

  /* Formulae taken from Kumar & Kurtz, Materials characterization, 1995,
     and more accurately from Statistics of Three Dimensional Random Voronoi
     Tessellations, A. Brakke, March 2005:
     mean edge length (for a unit volume polyhedron) = 0.430857994283959
   */

  twoDDoubleFree (size, 3);

  return 0.430857994283959 / pow (Tess.PolyQty / vol, 0.3333333333333333);
}
