/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_fm.h"

/* SetDefaultOptions set the options to their default values */
void
SetDefaultOptions_fm (struct IN *pIn, struct GEOPARA *pGeoPara)
{
  /* General options --------------------------------------- */
  (*pIn).verbosity = 0;
  (*pIn).nset = 1;
  (*pIn).gmsh = "gmsh";

  (*pIn).format = oneDCharDAlloc (100);
  sprintf ((*pIn).format, "geof");
  (*pIn).printgeo = 0;
  (*pIn).geo = NULL;
  (*pIn).printmsh = 0;
  (*pIn).msh = NULL;
  (*pIn).printgeof = 1;
  (*pIn).geof = NULL;
  (*pIn).printfod = 1;
  (*pIn).fod = NULL;
  (*pIn).printestat = 0;
  (*pIn).estat = NULL;
  (*pIn).printnstat = 0;
  (*pIn).nstat = NULL;
  (*pIn).printedgel = 0;
  (*pIn).edgel = NULL;
  (*pIn).printangle = 0;
  (*pIn).angle = NULL;
  (*pIn).print = 0;
  (*pIn).liset = 0;

  /* Options for the geometry modification ----------------- */
  (*pGeoPara).maxff = 0;
  (*pIn).newver = 1;
  (*pIn).mloop = 2;
  (*pIn).ffalgo = 1;
  (*pIn).edgedis = 1;
  (*pIn).geo2geob = 0;
  (*pIn).morder = 1;
  (*pIn).mesh = 0;
  (*pIn).conv = 0;

  /* Options for the meshing ------------------------------- */
  (*pGeoPara).cl = -1;		/* if unmodified directly, set from rcl. */
  (*pGeoPara).rcl = 1;
  (*pGeoPara).clmin = 0;
  (*pGeoPara).pcl = 2;
  (*pGeoPara).sel = -1;
  (*pGeoPara).rsel = -1;	/* relatively to the default value */
  (*pGeoPara).sa = 0;
  (*pGeoPara).sacl = -1;
  (*pGeoPara).sarcl = -1;
  (*pIn).meshalgorithm = 3;

  /* Other options ----------------------------------------- */
  /* Options for debugging --------------------------------- */
  (*pIn).meshimprovement2d = 1;
  (*pIn).delstop = 1e7;

  /* Useless or obsolete options ---------------------------
     (*pGeoPara).forcemergeqty = 0;
     (*pGeoPara).forbidmergeqty= 0;
   */
  
  (*pIn).printfaces = ut_alloc_1d_int (7);
  (*pIn).printfaces[0] = 3;
  (*pIn).printfaces[1] = 2;
  (*pIn).printfaces[2] = 4;
  (*pIn).printfaces[3] = 6;

  return;
}

void
SetOptions_fm (struct IN *pIn, struct GEOPARA *pGeoPara, char **pbody,
	       int *pinitess, int *pinibody, int argc, char **argv)
{
  int i, j, qty, ArgQty, Res;
  char **ArgList = twoDCharDAlloc (101, 101);
  char *Arg = oneDCharDAlloc (101);
  char * arg;

  /* This is the possible argument list. */
  ArgQty = 32;
  sprintf (ArgList[1], "-o");
  sprintf (ArgList[2], "-v");
  sprintf (ArgList[3], "-gmsh");
  sprintf (ArgList[4], "-bwcy-newver");
  sprintf (ArgList[5], "-maxff");
  sprintf (ArgList[6], "-mloop");
  sprintf (ArgList[7], "-bwcy-edgedis");
  sprintf (ArgList[8], "-cl");
  sprintf (ArgList[9], "-rcl");
  sprintf (ArgList[10], "-pcl");
  sprintf (ArgList[11], "-bwcy-clmin");
  sprintf (ArgList[12], "-ffalgo");
  sprintf (ArgList[13], "-morder");
  sprintf (ArgList[14], "-edgel");
  sprintf (ArgList[15], "-nset");
  sprintf (ArgList[16], "-estat");
  sprintf (ArgList[17], "-nstat");
  sprintf (ArgList[18], "-geo2geob");
  sprintf (ArgList[19], "-format");
  sprintf (ArgList[20], "-sel");
  sprintf (ArgList[21], "-rsel");
  sprintf (ArgList[22], "-angle");
  sprintf (ArgList[23], "-bwcy-meshimprovement2d");
  sprintf (ArgList[24], "-mesh");
  sprintf (ArgList[25], "-convert");
  sprintf (ArgList[26], "-meshalgorithm");
  sprintf (ArgList[27], "-sa");
  sprintf (ArgList[28], "-sacl");
  sprintf (ArgList[29], "-sarcl");
  sprintf (ArgList[30], "-print");
  sprintf (ArgList[31], "-printfaces");
  sprintf (ArgList[32], "-liset");

  /* Reading arguments ----------------------------------- */
  for (i = 1; i < argc; i++)
    {
      /* Input file name */
      if (argv[i][0] != '-')
	{
	  if ((*pinitess) == 0)
	    {
	      (*pIn).tess = oneDCharDAlloc (strlen (argv[i]));
	      (*pIn).tess = argv[i];
	      (*pinitess) = 1;
	    }
	  else
	    BadArguments ();
	}
      else
	{
	  /* Searching option name (string completion stuff) */
	  Res = StringComp (argv[i], ArgList, ArgQty, Arg);
	  if (Res == 1)
	    {
	      LineHeader (2);
	      printf ("Several possibilities for option `%s'.\n", argv[i]);
	      BadArguments ();
	    }
	  else if (Res == -1)
	    {
	      LineHeader (2);
	      printf ("Unknown option `%s'.\n", argv[i]);
	      BadArguments ();
	    }

	  /* General options ------------------------------------------------ */
	  if (strcmp (Arg, "-o") == 0 && i < argc - 1)
	    {
	      (*pbody) = NextArgAsChar (argv, &i, Arg);
	      (*pbody) = GiveBody (*pbody);
	      (*pinibody) = 1;
	    }
	  else if (strcmp (Arg, "-v") == 0 && i < argc - 1)
	    (*pIn).verbosity = NextArgAsInt (argv, &i, Arg, 0, 1);

	  else if (strcmp (Arg, "-gmsh") == 0 && i < argc - 1)
	    (*pIn).gmsh = NextArgAsChar (argv, &i, Arg);

	  /* General options to build the geometry */
	  else if (strcmp (Arg, "-bwcy-newver") == 0 && i < argc - 1)
	    (*pIn).newver = NextArgAsInt (argv, &i, Arg, 0, 1);

	  /* Options for the geometry modification ---------------------------- */
	  else if (strcmp (Arg, "-maxff") == 0 && i < argc - 1)
	    (*pGeoPara).maxff = NextArgAsReal (argv, &i, Arg, 0., 90.);
	  else if (strcmp (Arg, "-mloop") == 0 && i < argc - 1)
	    (*pIn).mloop = NextArgAsInt (argv, &i, Arg, 0, 1e7);

	  /* Options for the meshing ----------------------------------------- */
	  else if (strcmp (Arg, "-bwcy-edgedis") == 0 && i < argc - 1)
	    (*pIn).edgedis = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-cl") == 0 && i < argc - 1)
	    (*pGeoPara).cl = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
	  else if (strcmp (Arg, "-rcl") == 0 && i < argc - 1)
	    (*pGeoPara).rcl = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
	  else if (strcmp (Arg, "-sa") == 0 && i < argc - 1)
	    (*pGeoPara).sa = NextArgAsReal (argv, &i, Arg, 0., 180.);
	  else if (strcmp (Arg, "-sacl") == 0 && i < argc - 1)
	    (*pGeoPara).sacl = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
	  else if (strcmp (Arg, "-sarcl") == 0 && i < argc - 1)
	    (*pGeoPara).sarcl = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
	  else if (strcmp (Arg, "-bwcy-clmin") == 0 && i < argc - 1)
	    (*pGeoPara).clmin = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
	  else if (strcmp (Arg, "-pcl") == 0 && i < argc - 1)
	    (*pGeoPara).pcl = NextArgAsReal (argv, &i, Arg, 1., 1.e30);
	  else if (strcmp (Arg, "-sel") == 0 && i < argc - 1)
	    (*pGeoPara).sel = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
	  else if (strcmp (Arg, "-rsel") == 0 && i < argc - 1)
	    (*pGeoPara).rsel = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
	  else if (strcmp (Arg, "-meshalgorithm") == 0 && i < argc - 1)
	    (*pIn).meshalgorithm = NextArgAsInt (argv, &i, Arg, 1, 3);
	  else if (strcmp (Arg, "-bwcy-meshimprovement2d") == 0
		   && i < argc - 1)
	    (*pIn).meshimprovement2d = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-ffalgo") == 0 && i < argc - 1)
	    (*pIn).ffalgo = NextArgAsInt (argv, &i, Arg, 0, 2);
	  /*
	     else if(strcmp(Arg,"-ffmeas")==0 && i<argc-1)
	     (*pIn).ffmeas=NextArgAsInt(argv,&i,Arg,0,1);
	   */
	  else if (strcmp (Arg, "-mesh") == 0 && i < argc - 1)
	    {
	      (*pIn).geo = NextArgAsChar (argv, &i, Arg);
	      (*pIn).mesh = 1;
	    }
	  else if (strcmp (Arg, "-convert") == 0 && i < argc - 1)
	    {
	      (*pIn).msh = NextArgAsChar (argv, &i, Arg);
	      (*pIn).conv = 1;
	    }
	  else if (strcmp (Arg, "-morder") == 0 && i < argc - 1)
	    (*pIn).morder = NextArgAsInt (argv, &i, Arg, 1, 2);
	  else if (strcmp (Arg, "-edgel") == 0 && i < argc - 1)
	    (*pIn).printedgel = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-angle") == 0 && i < argc - 1)
	    (*pIn).printangle = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-nset") == 0 && i < argc - 1)
	    (*pIn).nset = NextArgAsInt (argv, &i, Arg, 0, 3);
	  else if (strcmp (Arg, "-estat") == 0 && i < argc - 1)
	    (*pIn).printestat = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-nstat") == 0 && i < argc - 1)
	    (*pIn).printnstat = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-geo2geob") == 0 && i < argc - 1)
	    (*pIn).geo2geob = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-print") == 0 && i < argc - 1)
	    (*pIn).print = NextArgAsInt (argv, &i, Arg, 0, 2);
	  else if (strcmp (Arg, "-liset") == 0 && i < argc - 1)
	    (*pIn).liset = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-printfaces") == 0 && i < argc - 1)
	  {
	    (*pIn).printfaces[0] = NextArgAsInt (argv, &i, Arg, 1, 6);
	    for (j = 1; j <=(*pIn).printfaces[0]; j++)
	      (*pIn).printfaces[j] = NextArgAsInt (argv, &i, Arg, 1, 6);
	  }

	  /* debug */
	  else if (strcmp (Arg, "-delstop") == 0 && i < argc - 1)
	    (*pIn).delstop = NextArgAsInt (argv, &i, Arg, 0, 1e7);
	  else if (strcmp (Arg, "-format") == 0 && i < argc - 1)
	    {
	      (*pIn).format = NextArgAsChar (argv, &i, Arg);
	      qty = ut_arg_listqty ((*pIn).format);
	      (*pIn).printgeo = 0;
	      (*pIn).printmsh = 0;
	      (*pIn).printgeof = 0;
	      for (j = 1; j <= qty; j++)
	      {
		arg = ut_arg_listmember ((*pIn).format, qty, j);
		
		if (strcmp (arg, "geo") == 0)
		  (*pIn).printgeo = 1;
		else if (strcmp (arg, "msh") == 0)
		  (*pIn).printmsh = 1;
		else if (strcmp (arg, "geof") == 0)
		  (*pIn).printgeof = 1;
		else
		  BadArguments ();

		free (arg);
	      }
	    }
	  else
	    {
	      if (i == 1)
		BadArguments ();
	      else
		ArgError (Arg);
	    }
	  /*
	     else if(strcmp(Arg,"-forcemerge")==0)
	     {
	     i++;
	     if(isdigit(Arg[0])!=0 || argv[i][0]=='.')
	     sscanf(Arg,"%d",&(*pGeoPara).forcemergeqty);
	     else
	     BadArguments(pname);

	     if((*pGeoPara).forcemergeqty<=argc-i-1)
	     {
	     (*pGeoPara).forcemerge=ut_alloc_1d_int((*pGeoPara).forcemergeqty+1);
	     (*pGeoPara).forcemerge[0]=(*pGeoPara).forcemergeqty;
	     for(j=1;j<=(*pGeoPara).forcemergeqty;j++)
	     {
	     i++;
	     if(isdigit(Arg[0])!=0 || argv[i][0]=='.')
	     sscanf(Arg,"%d",&(*pGeoPara).forcemerge[j]);
	     else
	     BadArguments(pname);
	     }
	     inimaxff=1;
	     }
	     else
	     BadArguments(pname);
	     }
	     else if(strcmp(Arg,"-forbidmerge")==0)
	     {
	     i++;
	     sscanf(Arg,"%d",&(*pGeoPara).forbidmergeqty);
	     if((*pGeoPara).forbidmergeqty<=argc-i-1)
	     {
	     (*pGeoPara).forbidmerge=ut_alloc_1d_int((*pGeoPara).forbidmergeqty+1);
	     (*pGeoPara).forbidmerge[0]=(*pGeoPara).forbidmergeqty;
	     for(j=1;j<=(*pGeoPara).forbidmergeqty;j++)
	     {
	     i++;
	     if(isdigit(Arg[0])!=0 || argv[i][0]=='.')
	     sscanf(Arg,"%d",&(*pGeoPara).forbidmerge[j]);
	     else
	     BadArguments(pname);
	     }
	     }
	     else
	     BadArguments(pname);
	     }
	   */
	}
    }

  twoDCharFree (ArgList, 101);
  oneDCharFree (Arg);

  return;
}
