/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"Meshing0D.h"

/* VerMeshing sets VerLength */
void
VerMeshing (struct GEO *pGeo, int ver, struct GEOPARA GeoPara)
{
  int i;
  int * Faces = ut_alloc_1d_int(100);
  int edge;
  double *dist;
  double *angle;
  double mindist;
  double minangle;
  double cl = GeoPara.cl;
  double pcl = GeoPara.pcl;
  double clmin = GeoPara.clmin;
  int pos;

  /* printf("beg of VerMeshing\n"); */

  /* Calculating VerLength from cl, pcl & clmin (& mindist) */
  dist = ut_alloc_1d ((*pGeo).VerEdgeQty[ver]);

  for (i = 0; i <= (*pGeo).VerEdgeQty[ver] - 1; i++)
    {
      edge = (*pGeo).VerEdgeNb[ver][i];
      dist[i] = (*pGeo).EdgeLength[edge];
    }
  mindist = oneDDoubleMin (dist, 0, (*pGeo).VerEdgeQty[ver] - 1);

  if (mindist * (1 + pcl) > cl)
    (*pGeo).VerLength[ver] = cl;
  else
    (*pGeo).VerLength[ver] = mindist * pcl;

  (*pGeo).VerLength[ver] = DoubleMax ((*pGeo).VerLength[ver], clmin);

  oneDDoubleFree (dist);

  /* Calculating the value due to the ver angles */
  if (GeoPara.sa > 1e-6)
    {
      VerFace (*pGeo, ver, Faces);
      angle = ut_alloc_1d (Faces[0] + 1);
      for (i = 1; i <= Faces[0]; i++)
	{
	  pos =
	    oneDIntEltPos ((*pGeo).FaceVerNb[Faces[i]], 1,
			   (*pGeo).FaceVerQty[Faces[i]], ver, 0);
	  if (pos == -1)	/* This means that the edge will be skipped */
	    {
	      minangle = 180;
	      break;
	    }

	  AngleAtVer ((*pGeo), Faces[i], ver, &angle[i]);
	}

      minangle = oneDDoubleMin (angle, 1, Faces[0]);

      if (minangle < GeoPara.sa)
	(*pGeo).VerLength[ver] =
	  DoubleMin ((*pGeo).VerLength[ver], GeoPara.sacl);

      oneDDoubleFree (angle);
    }

  /* printf("end of VerMeshing\n"); */
  oneDIntFree (Faces);

  return;
}
