/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"Meshing1D.h"

void
EdgeMeshing (struct GEO *pGeo, int edge, double cl, double pcl)
{
  int i;
  int seg, segqty, ptqty;
  int seg1ptqty = -1;
  double nextcoo, nextcooseg1;
  int alloc;

  double ** Seg = ut_alloc_2d (4, 3);
  double x, x1, x2, X;
  double *Coo, *Cl;

  double nearone = 0.95;

  double l = (*pGeo).EdgeLength[edge];
  double cl1 = (*pGeo).VerLength[(*pGeo).EdgeVerNb[edge][0]];
  double cl2 = (*pGeo).VerLength[(*pGeo).EdgeVerNb[edge][1]];

  /* pcl is the factor between 2 successive segments of an edge; */
  /* PCL is the maximum absolute slope of the curve cl=f(x).     */
  /* PCL = pcl-1 > 0                                             */
  double PCL = DoubleMax (0.001, pcl - 1);

  cl1 /= l;
  cl2 /= l;
  cl /= l;

  alloc = 10 + (int) (1 / (DoubleMin (cl1, cl2)));	/* 10 for security */
  Coo = ut_alloc_1d (alloc);
  Cl = ut_alloc_1d (alloc);

  if (cl1 > cl || cl2 > cl || PCL < 0)
    {
      ut_print_message (2, "EdgeMeshing: Bad values of cl1, cl2 & cl\n");
      abort ();
    }

  if (cl1 / cl > nearone && cl2 / cl > nearone)
    {
      segqty = 1;
      Seg[1][0] = 1;
      Seg[1][1] = 0;
      Seg[1][2] = cl;
    }
  else if (cl1 / cl > nearone)
    {
      x = 1 - (cl - cl2) / PCL;

      if (x > 0)
	{
	  segqty = 2;
	  Seg[1][0] = x;
	  Seg[1][1] = 0;
	  Seg[1][2] = cl;
	  Seg[2][0] = 1;
	  Seg[2][1] = -PCL;
	  Seg[2][2] = cl2 + PCL;
	}
      else
	{
	  segqty = 1;
	  Seg[1][0] = 1;
	  Seg[1][1] = cl2 - cl;
	  Seg[1][2] = cl;
	}
    }
  else if (cl2 / cl > nearone)
    {
      x = (cl - cl1) / PCL;

      if (x < 1)
	{
	  segqty = 2;
	  Seg[1][0] = x;
	  Seg[1][1] = PCL;
	  Seg[1][2] = cl1;
	  Seg[2][0] = 1;
	  Seg[2][1] = 0;
	  Seg[2][2] = cl;
	}
      else
	{
	  segqty = 1;
	  Seg[1][0] = 1;
	  Seg[1][1] = cl - cl1;
	  Seg[1][2] = cl1;
	}
    }
  else				/* cl1 & cl2 are different from cl: 1, 2 or 3 parts? */
    {
      x = 0.5 + (cl2 - cl1) / (2 * PCL);

      /* printf("x=%lf\n",x);
       * printf("cl(x)=%lf\n",Seg[1][1]*x+Seg[1][2]);
       */

      if (x < 0 || x > 1)
	{
	  /* PCL=cl2-cl1 is used to go from cl1 to cl2 */
	  segqty = 1;

	  Seg[1][0] = 1;
	  Seg[1][1] = cl2 - cl1;
	  Seg[1][2] = cl1;
	}
      else if (PCL * x + cl1 < cl)
	{
	  segqty = 2;

	  /* cl = PCL*x + cl1 */
	  Seg[1][0] = x;
	  Seg[1][1] = PCL;
	  Seg[1][2] = cl1;
	  /* cl = PCL*(1-x) + cl2 = cl2+PCL -PCL*x */
	  Seg[2][0] = 1;
	  Seg[2][1] = -PCL;
	  Seg[2][2] = cl2 + PCL;
	}
      else			/* 3 parts */
	{
	  segqty = 3;

	  x1 = (cl - cl1) / PCL;
	  x2 = (cl2 + PCL - cl) / PCL;

	  if (x1 > x || x2 < x)
	    {
	      printf ("Bad x1, x, x2\n");
	      abort ();
	    }

	  /* cl = PCL*x + cl1 */
	  Seg[1][0] = x1;
	  Seg[1][1] = PCL;
	  Seg[1][2] = cl1;
	  /* cl = cl */
	  Seg[2][0] = x2;
	  Seg[2][1] = 0;
	  Seg[2][2] = cl;
	  /* cl = PCL*(1-x) + cl2 = cl2+PCL -PCL*x */
	  Seg[3][0] = 1;
	  Seg[3][1] = -PCL;
	  Seg[3][2] = cl2 + PCL;
	}
    }


  /* Calculation of Coo */
  Coo[0] = 0;
  ptqty = 0;
  for (i = 1; i <= segqty; i++)
    {
      nextcoo = NextCoo (Coo[ptqty], Seg[i][1], Seg[i][2]);

      if (ptqty == seg1ptqty)
	{
	  nextcooseg1 = NextCoo (Coo[ptqty], Seg[1][1], Seg[1][2]);
	  if (nextcooseg1 < nextcoo)
	    nextcoo = nextcooseg1;
	}

      while (nextcoo < Seg[i][0])
	{
	  ptqty++;
	  Coo[ptqty] = nextcoo;

	  nextcoo = NextCoo (Coo[ptqty], Seg[i][1], Seg[i][2]);
	}

      if (i == 1)
	seg1ptqty = ptqty;
    }

  if (ptqty == seg1ptqty && Seg[1][1] < 0)
    X = Coo[ptqty] + Seg[1][1] * Coo[ptqty] + Seg[1][2];
  else
    X = Coo[ptqty] + cl2;

  for (i = 1; i <= ptqty; i++)
    Coo[i] /= X;

  /* Calculation of Cl */
  seg = 1;
  for (i = 1; i <= ptqty; i++)
    {
      if (Coo[i] < Seg[seg][0])
	Cl[i] = Seg[seg][1] * Coo[i] + Seg[seg][2];
      else
	{
	  seg++;
	  i--;
	}
    }

  if ((*pGeo).EdgeNodeQty[edge] != 0)
    {
      (*pGeo).EdgeNodeQty[edge] = 0;

      oneDDoubleFree ((*pGeo).EdgeNodes[edge]);
      oneDDoubleFree ((*pGeo).EdgeNodeCls[edge]);
    }

  (*pGeo).EdgeNodeQty[edge] = ptqty;

  (*pGeo).EdgeNodes[edge] = ut_alloc_1d (ptqty + 1);
  (*pGeo).EdgeNodeCls[edge] = ut_alloc_1d (ptqty + 1);

  for (i = 1; i <= ptqty; i++)
    {
      (*pGeo).EdgeNodes[edge][i] = Coo[i];
      (*pGeo).EdgeNodeCls[edge][i] = Cl[i] * l;
    }

  oneDDoubleFree (Coo);
  oneDDoubleFree (Cl);

  /* printf("DummyVerQty=%d\n",(*pGeo).DummyVerQty);   */
  /* printf("DummyEdgeQty=%d\n",(*pGeo).DummyEdgeQty); */

  /* !!!! WARNING !!!! DummyVerQty & DummyEdgeQty are set to 0 here!!! */
  /* Seems to be ok ...  don't know ...  10 dec 2006                   */

  (*pGeo).DummyVerQty = 0;
  (*pGeo).DummyEdgeQty = 0;

  for (i = 1; i <= (*pGeo).EdgeQty; i++)
    if ((*pGeo).EdgeNodeQty[i] != 0)
      {
	(*pGeo).EdgeNode1Id[i] = (*pGeo).VerQty + (*pGeo).DummyVerQty + 1;
	(*pGeo).EdgeSeg1Id[i] = (*pGeo).EdgeQty + (*pGeo).DummyEdgeQty + 1;

	(*pGeo).DummyVerQty += (*pGeo).EdgeNodeQty[i];
	(*pGeo).DummyEdgeQty += ((*pGeo).EdgeNodeQty[i] + 1);
      }
  
  twoDDoubleFree (Seg, 4);

  return;
}
