/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"Meshing2D.h"

/* FaceTest4Dis sets FaceDis of face to 1 if is has to be discretized
 * (non-negligeable flatness fault).
 */
void
FaceTest4Dis (struct GEO *pGeo, int face)
{
  if ((*pGeo).FaceState[face] > 0	/* modified face        */
      && (*pGeo).FaceBound[face] == 0	/* non-boundary face    */
      && (*pGeo).FaceVerQty[face] > 3)	/* non-triangular face  */
    /*&& (*pGeo).FaceFF    [face]>1e-4 )                       */
    (*pGeo).FaceDis[face] = 1;

  return;
}

void
DiscretizeThisFace (struct GEO *pGeo, char *gmsh, int face)
{
  struct MESH TrashMesh3;
  FILE *file;

  /* Writing the face as a geo file ------------------------ */
  if (verbosity > 1)
    printf ("performing WriteThisFaceWAllEntities\n");
  file = OpenFile ("tmp.geo", "W");
  WriteMeshingOptions (file);
  WriteThisFaceWAllEntities (pGeo, face, file);
  CloseFile (file, "tmp.geo", "W");
  if (verbosity > 1)
    printf ("performed  WriteThisFaceWAllEntities\n");

  /* meshing geo ------------------------------------------- */
  if (verbosity > 1)
    printf ("performing Mesh2DGeo\n");
  Mesh2DGeo (gmsh, "tmp.geo");
  if (verbosity > 1)
    printf ("performed Mesh2DGeo\n");

  /* reading mesh ------------------------------------------ */
  if (verbosity > 1)
    printf ("performing ReadMsh\n");
  file = OpenFile ("tmp.msh", "R");
  ReadMsh (&((*pGeo).FaceNodes[face]), &((*pGeo).FaceM0D[face]),
	   &((*pGeo).FaceM1D[face]), &((*pGeo).FaceM2D[face]),
	   &TrashMesh3, file);
  CloseFile (file, "tmp.msh", "R");
  if (verbosity > 1)
    printf ("performed ReadMsh\n");

  remove ("tmp.geo");
  remove ("tmp.msh");

  return;
}

/* returns 1 if the mesh is of good quality
 * -1 else
 */
int
Test2DMesh (struct GEO Geo, int face)
{
  int i, j;
  int tmp, res;
  int nb, qty, edge;
  int *ninv = ut_alloc_1d_int (3);
  double per, minper;		/* minimum perimeter for a 2-D element */
  double *eltlength;
  double *length;
  int maxedgenodeqty;

  qty = 0;
  maxedgenodeqty = 0;
  for (i = 1; i <= Geo.FaceVerQty[face]; i++)
    {
      edge = Geo.FaceEdgeNb[face][i];
      qty += (Geo.EdgeNodeQty[edge] + 1);
      maxedgenodeqty = DoubleMax (maxedgenodeqty, Geo.EdgeNodeQty[edge]);
    }

  length = ut_alloc_1d (qty + 2);
  eltlength = ut_alloc_1d (maxedgenodeqty + 2);

  nb = 0;
  for (i = 1; i <= Geo.FaceVerQty[face]; i++)
    {
      edge = Geo.FaceEdgeNb[face][i];

      if (Geo.EdgeNodeQty[edge] == 0)
	{
	  nb++;
	  length[nb] = Geo.EdgeLength[edge];
	}
      else
	{
	  /* first elt */
	  eltlength[1] = Geo.EdgeLength[edge] * Geo.EdgeNodes[edge][1];

	  /* intermediate elt */
	  for (j = 2; j <= Geo.EdgeNodeQty[edge]; j++)
	    eltlength[j] = Geo.EdgeLength[edge] *
	      (Geo.EdgeNodes[edge][j] - Geo.EdgeNodes[edge][j - 1]);

	  /* last elt */
	  eltlength[Geo.EdgeNodeQty[edge] + 1] =
	    Geo.EdgeLength[edge] * (1 -
				    Geo.EdgeNodes[edge][Geo.
							EdgeNodeQty[edge]]);

	  if (Geo.FaceEdgeOri[face][i] == 1)
	    for (j = 1; j <= Geo.EdgeNodeQty[edge] + 1; j++)
	      {
		nb++;
		length[nb] = eltlength[j];
	      }
	  else
	    for (j = Geo.EdgeNodeQty[edge] + 1; j >= 1; j--)
	      {
		nb++;
		length[nb] = eltlength[j];
	      }
	}
    }

  /* Calculating the minimum allowed perimeter */
  minper = 1e32;
  for (i = 1; i <= qty - 1; i++)
    minper = DoubleMin (minper, length[i] + length[i + 1]);
  minper = DoubleMin (minper, length[qty] + length[1]);

  res = 1;

  for (i = 1; i <= (Geo.FaceM2D[face]).EltQty; i++)
    {
      for (j = 0; j <= 2; j++)
	{
	  tmp = (Geo.FaceM2D[face]).EltNodes[i][j];
	  ninv[j] = (Geo.FaceNodes[face]).NodesNbInv[tmp];
	}

      per = threeDDoubleDist ((Geo.FaceNodes[face]).NodesCoo[ninv[0]],
			      (Geo.FaceNodes[face]).NodesCoo[ninv[1]], 0)
	+ threeDDoubleDist ((Geo.FaceNodes[face]).NodesCoo[ninv[1]],
			    (Geo.FaceNodes[face]).NodesCoo[ninv[2]], 0)
	+ threeDDoubleDist ((Geo.FaceNodes[face]).NodesCoo[ninv[2]],
			    (Geo.FaceNodes[face]).NodesCoo[ninv[0]], 0);

      if (per < minper)
	{
	  res = -1;
	  break;
	}
    }

/*
    if(res==-1)
    {
	printf("face=%d: minper=%lf per=%lf\n",face,minper,per);
	abort();
    }
    */

  oneDDoubleFree (eltlength);
  oneDDoubleFree (length);
  oneDIntFree (ninv);

  return res;
}


int *
GeoFaceNodes (struct GEO Geo, int face)
{
  int j;

  int *FaceNodes = ut_alloc_1d_int (Geo.FaceNodes[face].NodeQty + 1);

  FaceNodes[0] = Geo.FaceNodes[face].NodeQty;
  for (j = 1; j <= FaceNodes[0]; j++)
    FaceNodes[j] = Geo.FaceNodes[face].NodesNb[j];

  return FaceNodes;
}

void
InterpolateFaceBody (struct GEO *pGeo, struct NODES *pn,
		     int *FaceNodes, int face)
{
  int i, j, pos;
  int TriangleQty;
  double ***FaceTriangle;

  /* FaceTriangle is allocated with the maximum triangle quantity. */
  FaceTriangle = threeDDoubleDAlloc ((*pGeo).FaceVerQty[face] + 1, 4, 4);
  TriangleQty = CalcFaceTriangle (pGeo, face, FaceTriangle);

  /* for(i=1;i<=(*pn).NodeQty;i++) */
  for (i = 1; i <= FaceNodes[0]; i++)
    {
      pos = (*pn).NodesNbInv[FaceNodes[i]];

      for (j = 1; j <= TriangleQty; j++)
	if (   Side (FaceTriangle[j][1], (*pn).NodesCoo[pos] - 1) == -1
	    && Side (FaceTriangle[j][2], (*pn).NodesCoo[pos] - 1) == -1
	    && Side (FaceTriangle[j][3], (*pn).NodesCoo[pos] - 1) == -1)
	  {
	    ProjectPointAlongOnto ((*pn).NodesCoo[pos],
				   (*pGeo).FaceEq[face] + 1,
				   FaceTriangle[j][0]);
	    break;
	  }
    }

  /* FaceTriangle freeing */
  for (i = (*pGeo).FaceVerQty[face]; i <= 0; i--)
    twoDDoubleFree (FaceTriangle[i], 4);
  free (FaceTriangle);

  return;
}
