/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"Meshing2D.h"

void
Mesh2DGeo (char *gmsh, char *filename)
{
  char *Args = oneDCharDAlloc (1000);

  sprintf (Args, "-2 -v 0 -algo tri %s", filename);

  CallGmsh (gmsh, Args);

  oneDCharFree (Args);

  return;
}

int
CalcFaceTriangle (struct GEO *pGeo, int face, double ***FaceTriangle)
{
  int i, j, id, ver, ver1, ver2;
  int verqty = (*pGeo).FaceVerQty[face];

  double * p1 = ut_alloc_1d (3);
  double * p2 = ut_alloc_1d (3);
  double * p3 = ut_alloc_1d (3);
  double * bary = ut_alloc_1d (3);
  double * n = ut_alloc_1d (3);

  bary[0] = bary[1] = bary[2] = 0;
  for (i = 1; i <= verqty; i++)
    {
      ver = (*pGeo).FaceVerNb[face][i];
      for (j = 0; j <= 2; j++)
	bary[j] += (*pGeo).VerCoo[ver][j];
    }
  for (j = 0; j <= 2; j++)
    bary[j] /= verqty;

  if ((*pGeo).FacePt[face] == 0)	/* p3 is the face barycenter. */
      for (j = 0; j <= 2; j++)
	p3[j] = bary[j];
  else				/* p3 is a face vertex. */
    {
      ver = (*pGeo).FaceVerNb[face][(*pGeo).FacePt[face]];
      for (j = 0; j <= 2; j++)
	p3[j] = (*pGeo).VerCoo[ver][j];
    }

  id = 0;
  for (i = 1; i <= verqty; i++)
    {
      /* if the ver is the one just before p3 or p3 itself, it is skipped
       * (this may occur only if p3nb!=0, so only if p3 is a face vertex).
       */
      if (i == (*pGeo).FacePt[face]
	  || oneDRotPos (1, verqty, i, 1) == (*pGeo).FacePt[face])
	continue;

      /* 1st and 2nd pts */
      ver1 = (*pGeo).FaceVerNb[face][i];
      ver2 = (*pGeo).FaceVerNb[face][oneDRotPos (1, verqty, i, 1)];

      for (j = 0; j <= 2; j++)
	p1[j] = (*pGeo).VerCoo[ver1][j];
      for (j = 0; j <= 2; j++)
	p2[j] = (*pGeo).VerCoo[ver2][j];

      /* if the 3 points are aligned (on an edge of the domain), skipping. */
      if (PointAligned (p1, p2, p3, 1e-6))	/* use an angle-criterion instead.  */
	continue;

      id++;			/* identifier of the treated triangular part. */

      /*
         printf("vers: ");
         printf("%d(%.3lf %.3lf %.3lf) ",(*pGeo).FaceVerNb[face][i],\
         p1[0],p1[1],p1[2]);
         printf("%d(%.3lf %.3lf %.3lf) ",(*pGeo).FaceVerNb[face]\
         [oneDRotPos(1,verqty,i,1)],\
         p2[0],p2[1],p2[2]);
         printf("%d(%.3lf %.3lf %.3lf) ",(*pGeo).FaceVerNb[face]\
         [(*pGeo).FacePt[face]],\
         p3[0],p3[1],p3[2]);
         printf("\n");
       */

      Points2Plane (p1, p2, p3, FaceTriangle[id][0]);

      /* below, "...Eq[face]+1" makes it possible to get the 
       * normal parameters from [0] instead of [1] ...
       */

      n[0] = (*pGeo).FaceEq[face][1];
      n[1] = (*pGeo).FaceEq[face][2];
      n[2] = (*pGeo).FaceEq[face][3];

      PointsNInVect2Plane (p1, p2, n, FaceTriangle[id][1]);

      if (Side (FaceTriangle[id][1], p3 - 1) == 1)	/* that is, if the face normal goes */
	oneDDoubleScale (FaceTriangle[id][1], 0, 3, -1);	/* into the triangle.   */

      PointsNInVect2Plane (p2, p3, n, FaceTriangle[id][2]);
      if (Side (FaceTriangle[id][2], p1 - 1) == 1)
	oneDDoubleScale (FaceTriangle[id][2], 0, 3, -1);

      PointsNInVect2Plane (p3, p1, n, FaceTriangle[id][3]);
      if (Side (FaceTriangle[id][3], p2 - 1) == 1)
	oneDDoubleScale (FaceTriangle[id][3], 0, 3, -1);

         /*
	  * printf("triangle%d(%d): %fx+%fy+%fz-%f=0\n",id,i,\
         FaceTriangle[id][0][1],FaceTriangle[id][0][2],\
         FaceTriangle[id][0][3],FaceTriangle[id][0][0]);
         printf("bound 1: %fx+%fy+%fz-%f=0\n",\
         FaceTriangle[id][1][1],FaceTriangle[id][1][2],\
         FaceTriangle[id][1][3],FaceTriangle[id][1][0]);
         printf("bound 2: %fx+%fy+%fz-%f=0\n",\
         FaceTriangle[id][2][1],FaceTriangle[id][2][2],\
         FaceTriangle[id][2][3],FaceTriangle[id][2][0]);
         printf("bound 3: %fx+%fy+%fz-%f=0\n",\
         FaceTriangle[id][3][1],FaceTriangle[id][3][2],\
         FaceTriangle[id][3][3],FaceTriangle[id][3][0]);
	 */
    }

  oneDDoubleFree(p1);
  oneDDoubleFree(p2);
  oneDDoubleFree(p3);
  oneDDoubleFree(bary);
  oneDDoubleFree(n);

  return id;
}

/*
void RecordFaceVerCoo(struct GEO Geo,int face,double** VerCoo)
{
    int i,j,ver;
    
    for(i=1;i<=Geo.FaceVerQty[face];i++)
    {
	ver=Geo.FaceVerNb[face][i];
	for(j=0;j<=2;j++)
	    VerCoo[i][j]=Geo.VerCoo[ver][j];
    }

    return;
}
void CorrectFaceVerCoo(struct GEO* pGeo,int face,double** VerCoo)
{
    int i,j,ver;

    for(i=1;i<=(*pGeo).FaceVerQty[face];i++)
    {
	ver=(*pGeo).FaceVerNb[face][i];
	for(j=0;j<=2;j++)
	    (*pGeo).VerCoo[ver][j]=VerCoo[i][j];
    }

    return;
}

void InterpolateFaceBound(struct GEO* pGeo,int face)
{
    printf("Performed InterpolateFaceVer\n");
    InterpolateFaceEdge(pGeo,face);
    printf("Performing InterpolateFaceVer\n");
    InterpolateFaceVer(pGeo,face);

    return;
}
*/
