/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"WriteGeof.h"
/* Searching the tessellation faces of each face of the domain */
int **
ReadFoD (char *fodname)
{
  int i, j, tmp, size;
  int **fod = oneDpIntDAlloc (7);

  FILE *file = OpenFile (fodname, "r");

  for (i = 1; i <= 6; i++)
    {
      fscanf (file, "%d%d", &tmp, &size);
      fod[tmp] = ut_alloc_1d_int (size + 1);
      fod[tmp][0] = size;
      for (j = 1; j <= fod[tmp][0]; j++)
	fscanf (file, "%d", &fod[tmp][j]);
    }

  CloseFile (file, fodname, "r");

  return fod;
}

int **
SearchFoDNodes (struct MESH Mesh2D, int **FoD)
{
  int i, j, nbnodes;
  int **data = oneDpIntDAlloc (7);
  int **FaceNodes;

  for (i = 1; i <= 6; i++)
    {
      FaceNodes = oneDpIntDAlloc (FoD[i][0] + 1);

      nbnodes = 0;
      for (j = 1; j <= FoD[i][0]; j++)
	{
	  FaceNodes[j] = ThisFaceNodes (Mesh2D, FoD[i][j]);
	  nbnodes += FaceNodes[j][0];
	}

      data[i] = ut_alloc_1d_int (nbnodes + 1);

      for (j = 1; j <= FoD[i][0]; j++)
	{
	  oneDIntCpy (FaceNodes[j], 1, FaceNodes[j][0], data[i],
		      data[i][0] + 1, 0);
	  data[i][0] += FaceNodes[j][0];
	}

      oneDIntSort (data[i], 1, data[i][0]);
      data[i][0] = oneDIntCompress (data[i], 1, data[i][0]);

      twoDIntFree (FaceNodes, FoD[i][0] + 1);
    }

  return data;
}

void
WriteNodes (struct NODES Nodes, FILE * geof)
{
  int i, j;

  fprintf (geof, "***geometry\n");
  fprintf (geof, "**node\n");
  fprintf (geof, "%d 3\n", Nodes.NodeQty);
  for (i = 1; i <= Nodes.NodeQty; i++)
    {
      fprintf (geof, "%d", Nodes.NodesNb[i]);
      for (j = 0; j <= 2; j++)
	fprintf (geof, "  %.12f", Nodes.NodesCoo[i][j]);
      fprintf (geof, "\n");
    }

  return;
}

void
WriteElts (struct MESH Mesh3D, FILE * geof)
{
  int i, j;

  fprintf (geof, "**element\n");
  fprintf (geof, "%d\n", Mesh3D.EltQty);

  if (Mesh3D.EltOrder == 1)
    for (i = 1; i <= Mesh3D.EltQty; i++)
      {
	fprintf (geof, "%d c3d4", Mesh3D.EltNb[i]);
	for (j = 0; j <= 3; j++)
	  fprintf (geof, " %d", Mesh3D.EltNodes[i][j]);
	fprintf (geof, "\n");
      }
  else if (Mesh3D.EltOrder == 2)
    for (i = 1; i <= Mesh3D.EltQty; i++)
      {
	fprintf (geof, "%d c3d10", Mesh3D.EltNb[i]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][0]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][1]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][2]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][4]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][5]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][6]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][7]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][9]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][8]);
	fprintf (geof, " %d", Mesh3D.EltNodes[i][3]);
	fprintf (geof, "\n");
      }

  return;
}

/*
void WriteNsets(int nset,int** FoDNodes,FILE* geof)
{
    int i,node;
    char* NsetNames[7];

    if(nset>=1)
    {
	NsetNames[1]="x=0"; NsetNames[2]="x=1";
	NsetNames[3]="y=0"; NsetNames[4]="y=1";
	NsetNames[5]="z=0"; NsetNames[6]="z=1";

	for(i=1;i<=6;i++)
	    WriteNSet(NsetNames[i],FoDNodes[i],geof);
    }
    
    if(nset>=2)
    {
	node=Search3FacesCommonNode(FoDNodes,1,3,5);
	fprintf(geof,"**nset x0y0z0\n%d\n",node);

	node=Search3FacesCommonNode(FoDNodes,2,3,5);
	fprintf(geof,"**nset x1y0z0\n%d\n",node);

	node=Search3FacesCommonNode(FoDNodes,1,4,5);
	fprintf(geof,"**nset x0y1z0\n%d\n",node);

	node=Search3FacesCommonNode(FoDNodes,2,4,5);
	fprintf(geof,"**nset x1y1z0\n%d\n",node);
	
	node=Search3FacesCommonNode(FoDNodes,1,3,6);
	fprintf(geof,"**nset x0y0z1\n%d\n",node);

	node=Search3FacesCommonNode(FoDNodes,2,3,6);
	fprintf(geof,"**nset x1y0z1\n%d\n",node);

	node=Search3FacesCommonNode(FoDNodes,1,4,6);
	fprintf(geof,"**nset x0y1z1\n%d\n",node);

	node=Search3FacesCommonNode(FoDNodes,2,4,6);
	fprintf(geof,"**nset x1y1z1\n%d\n",node);
    }

    return;
}
*/
    
void
WriteFoDLiset (struct GEO Geo, int** FoD, struct NODES Nodes,
               struct MESH Mesh2D, FILE* geof)
{
  int i, j, k, qty, edge, face, idinv, idinv2;
  int face1, face2, face1nodeqty, face2nodeqty;
  int* edge_list;
  int* face1nodes;
  int* face2nodes;
  int* node_list;
  int nodeqty;
  char** LisetNames = ut_alloc_2d_char (7, 100);
  double* ref = ut_alloc_1d (3);
  double* uvect = ut_alloc_1d (3);
  double* vect  = ut_alloc_1d (3);
  double* coo;
  int* index;

  sprintf (LisetNames[1], "x0");
  sprintf (LisetNames[2], "x1");
  sprintf (LisetNames[3], "y0");
  sprintf (LisetNames[4], "y1");
  sprintf (LisetNames[5], "z0");
  sprintf (LisetNames[6], "z1");
  
  for (i = 1; i <= 6; i++) /* for every face of the domain */
  {
    fprintf (geof, "**liset %s\n", LisetNames[i]);

    /* all edges of the faces must be plotted; recording them */
    qty = 0;
    for (j = 1; j <= FoD[i][0]; j++)
      qty += Geo.FaceVerQty[FoD[i][j]];

    edge_list = ut_alloc_1d_int (qty);

    for (j = 1; j <= FoD[i][0]; j++)
    {
      face = FoD[i][j];

      for (k = 1; k <= Geo.FaceVerQty[face]; k++)
      {
	edge = Geo.FaceEdgeNb[face][k];

	if (ut_board_1d_int_eltpos (edge_list + 1, edge_list[0], edge) == -1)
	  edge_list[++edge_list[0]] = edge;
      }
    }

    /* for every edge, plotting liset */
    for (j = 1; j <= edge_list[0]; j++)
    {
      edge = edge_list[j];
      face1 = Geo.EdgeFaceNb[edge][0];
      face2 = Geo.EdgeFaceNb[edge][1];

      /* building the list of nodes */
      MeshElsetNodes (Mesh2D, face1, &face1nodes, &face1nodeqty);
      MeshElsetNodes (Mesh2D, face2, &face2nodes, &face2nodeqty);

      ut_board_1d_inter (face1nodes, face1nodeqty, \
	                 face2nodes, face2nodeqty, \
			 &node_list, &nodeqty);
      ut_free_1d_int (face1nodes);
      ut_free_1d_int (face2nodes);

      idinv = Nodes.NodesNbInv[node_list[0]];
      ut_board_1d_memcpy (ref, 3, Nodes.NodesCoo[idinv]);
      idinv2 = Nodes.NodesNbInv[node_list[1]];
      ut_board_1d_sub (ref, Nodes.NodesCoo[idinv2], 3, uvect);

      coo = ut_alloc_1d (nodeqty);
      for (k = 0; k < nodeqty; k++)
      {
	idinv2 = Nodes.NodesNbInv[node_list[k]];
	ut_board_1d_sub (ref, Nodes.NodesCoo[idinv2], 3, vect);
	coo[k] = ut_vector_scalprod (uvect, vect);
      }

      index = ut_alloc_1d_int (nodeqty);
      ut_board_1d_sort_index (coo, nodeqty, index);
      ut_board_1d_permutation_int (node_list, nodeqty, index);

      if (Mesh2D.EltOrder == 1)
	for (k = 0; k < nodeqty - 1; k++)
	  fprintf (geof, "line %d %d\n", node_list[k], node_list[k + 1]);
      else if (Mesh2D.EltOrder == 2)
	for (k = 0; k < nodeqty - 2; k+=2)
	  fprintf (geof, "quad %d %d %d\n", node_list[k], node_list[k + 1], node_list[k + 2]);
      else
	abort();
      
      ut_free_1d (coo);
      ut_free_1d_int (index);
      ut_free_1d_int (node_list);
    }

    ut_free_1d_int (edge_list);
  }
  
  ut_free_2d_char (LisetNames, 7);
  ut_free_1d (uvect);
  ut_free_1d (vect);
  
  return;
}

void
WriteElsets (struct MESH Mesh3D, FILE * geof)
{
  int i, j, col;

  for (i = 1; i <= Mesh3D.ElsetQty; i++)
    {
      fprintf (geof, "**elset poly%d\n", i);
      col = 0;
      for (j = 1; j <= Mesh3D.Elsets[i][0]; j++)
	FPrintfWNC (geof, Mesh3D.Elsets[i][j], &col, 72);

      fprintf (geof, "\n");
    }

  return;
}
