/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"neper_fm.h"

int
neper_fm (int fargc, char **fargv, int argc, char **argv)
{
  struct IN In;
  struct TESS Tess;
  struct GEO Geo;
  struct GEOPARA GeoPara;
  struct NODES Nodes;
  struct MESH TrashMesh0, TrashMesh1;
  struct MESH Mesh2D;
  struct MESH Mesh3D;
  FILE *msh;

  char *mname = "-FM";

  /* Printing program header */
  ModuleHeader (mname, fargc, fargv, argc, argv);

  /* Reading input data */
  ut_print_message (0, "Reading input data ...\n");
  InputData_fm (&In, &Tess, &GeoPara, fargc, fargv, argc, argv);
  ut_print_message (0, "Read input data.\n");

  GmshRC ("bak");

  /* Creating geometry ----------------------------------------------- */
  if (In.mesh == 0 && In.conv == 0)
    {
      /* Creating geometry */
      ut_print_message (0, "Creating geometry ...\n");
      CreateGeo (Tess, &Geo, GeoPara, In);
      ut_print_message (0, "Created geometry.\n");

      /* Meshing vertices (already done if ffalgo!=1) */
      if (In.ffalgo == 1)
	GeoMeshing0D (&Geo, GeoPara);

      if (In.edgedis == 1)	/* Meshing edges */
	{
	  ut_print_message (0, "Creating 1-D mesh ...\n");
	  GeoMeshing1D (&Geo, GeoPara);
	  ut_print_message (0, "Created 1-D mesh.\n");
	}

      if ((GeoPara.maxff > 1e-9 && In.mloop >= 1)
	  || In.meshimprovement2d == 1)
	{
	  /* Meshing non-planar faces (recorded as meshes into geo). */
	  if (In.meshimprovement2d == 0)
	    ut_print_message (0, "Creating 2-D mesh (non-plane faces) ...\n");
	  else
	    ut_print_message (0, "Creating 2-D mesh ...\n");

	  GeoMeshing2D (&Geo, In);

	  if (In.meshimprovement2d == 0)
	    ut_print_message (0, "Created 2-D mesh (non-plane faces).\n");
	  else
	    ut_print_message (0, "Created 2-D mesh.\n");
	}

      /* writing of: geo, fod, fff, mod, edgel  */
      WriteGeoStuff (In, Geo, GeoPara, argc, argv, "unknown");
    }

  /* Meshing geometry ---------------------------------------------------- */
  Mesh3D.EltQty = 0;
  if ((In.mesh == 0 && In.conv == 0 && (In.printmsh == 1 || In.printgeof == 1)) || (In.mesh == 1))
    {
      if (In.meshimprovement2d == 0)
	ut_print_message (0, "Creating 2-D & 3-D mesh ...\n");
      else
	ut_print_message (0, "Creating 3-D mesh ...\n");

      GeoMeshing3D (In, &Nodes, &Mesh2D, &Mesh3D);	/* writing .msh */

      if (In.meshimprovement2d == 0)
	ut_print_message (0, "Created 2-D & 3-D mesh.\n");
      else
	ut_print_message (0, "Created 3-D mesh.\n");
    }

  if ((In.mesh == 0 && In.conv == 0 && In.printgeof == 1)
      || (In.mesh == 1 && In.printgeof == 1) || (In.conv == 1))
    {
      /* Write geof if needed -------------------------------------------- */
      msh = OpenFile (In.msh, "R");
      ReadMsh (&Nodes, &TrashMesh0, &TrashMesh1, &Mesh2D, &Mesh3D, msh);
      CloseFile (msh, In.msh, "R");
      WriteGeof (In, Geo, Nodes, Mesh2D, Mesh3D);

      if (In.conv == 0 && In.printmsh == 0)
	remove (In.msh);
    }

  GmshRC ("unbak");

  if (In.print > 0)
  {
    PrintGeoAsMast (In.print, In.printfaces, Geo, In.mast);
    PrintGeoAsAsy (In.print, In.printfaces, Geo, In.asy);
  }

  /* Print statistics ---------------------------------------------------- */
  if (In.printestat == 1 || In.printnstat == 1)
    if (Mesh3D.EltQty > 0)
      MStat (Nodes, Mesh2D, Mesh3D, In);

  /* Removing some files if needed */
  if (In.mesh == 0 && In.conv == 0 && In.printgeo == 0)
    {
      remove (In.geo);
      remove (In.geob);
      remove (In.fod);
      remove (In.ff);
    }


  return 0;
}
