/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_mm.h"

/* SetDefaultOptions set the options to their default values */
void
SetDefaultOptions_mm (struct IN *pIn, struct MMDEF *pMMDef,
		      struct DNEIGH *pDNeigh, struct MAPMESH *pMapMesh)
{
  (*pMMDef).relset = NULL;
  (*pMMDef).geof = NULL;
  (*pMMDef).loadmesh = NULL;

  (*pIn).verbosity = 0;
  (*pIn).input = -1;
  (*pIn).ps2d = 0;		/* like in module -T */
  (*pIn).nset = 1;
  (*pIn).faset = 0;

  (*pIn).nsite = 0;
  (*pIn).nsitesize = -1;

  (*pDNeigh).ttype = 0;
  (*pDNeigh).NDensity = 100;
  (*pDNeigh).nN = 0;
  (*pDNeigh).Id = -1;

  (*pMapMesh).MeshSize = ut_alloc_1d_int (3);
  (*pMapMesh).MeshSize[0] = 20;
  (*pMapMesh).MeshSize[1] = 20;
  (*pMapMesh).MeshSize[2] = 20;
  (*pMapMesh).MeshOrder = 1;
  (*pMMDef).mesh = 1;

  return;
}

void
SetOptions_mm (struct IN *pIn, struct MMDEF *pMMDef, struct DNEIGH *pDNeigh,
	       struct MAPMESH *pMapMesh, int *piniN, int *pinitess,
	       char **pbody, int *pinibody, int argc, char **argv)
{
  int i, ArgQty, Res;
  char **ArgList = twoDCharDAlloc (101, 101);
  char *Arg = oneDCharDAlloc (101);

  /* This is the possible argument list. */
  ArgQty = 15;
  sprintf (ArgList[1], "-o");
  sprintf (ArgList[2], "-n");
  sprintf (ArgList[3], "-id");
  sprintf (ArgList[4], "-ttype");
  sprintf (ArgList[5], "-nset");
  sprintf (ArgList[6], "-ps2d");
  sprintf (ArgList[7], "-msize");
  sprintf (ArgList[8], "-morder");
  sprintf (ArgList[9], "-mesh");
  sprintf (ArgList[10], "-lmesh");
  sprintf (ArgList[11], "-v");
  sprintf (ArgList[12], "-nsite");
  sprintf (ArgList[13], "-nsitewidth");
  sprintf (ArgList[14], "-faset");
  sprintf (ArgList[15], "-msize3");

  for (i = 1; i <= argc - 1; i++)
    {
      if (argv[i][0] != '-')
	{
	  if ((*pinitess) == 0)
	    {
	      (*pIn).tess = oneDCharDAlloc (strlen (argv[i]));
	      (*pIn).tess = argv[i];
	      (*pinitess) = 1;
	    }
	  else
	    BadArguments ();
	}
      else
	{
	  /* Searching option name (string completion stuff) */
	  Res = StringComp (argv[i], ArgList, ArgQty, Arg);
	  if (Res == 1)
	    {
	      LineHeader (2);
	      printf ("Several possibilities for option `%s'.\n", argv[i]);
	      BadArguments ();
	    }
	  else if (Res == -1)
	    {
	      LineHeader (2);
	      printf ("Unknown option `%s'.\n", argv[i]);
	      BadArguments ();
	    }

	  if (strcmp (Arg, "-o") == 0 && i < argc - 1)
	    {
	      (*pbody) = NextArgAsChar (argv, &i, Arg);
	      (*pbody) = GiveBody (*pbody);
	      (*pinibody) = 1;
	    }
	  else if (strcmp (Arg, "-n") == 0 && i < argc - 1)
	    {
	      (*pDNeigh).NDensity = NextArgAsInt (argv, &i, Arg, 0, 1e7);
	      (*piniN) = 1;
	    }
	  else if (strcmp (Arg, "-id") == 0 && i < argc - 1)
	    (*pDNeigh).Id = NextArgAsInt (argv, &i, Arg, 0, 1e7);
	  else if (strcmp (Arg, "-ttype") == 0 && i < argc - 1)
	    (*pDNeigh).ttype = NextArgAsInt (argv, &i, Arg, 0, 2);
	  else if (strcmp (Arg, "-nset") == 0 && i < argc - 1)
	    (*pIn).nset = NextArgAsInt (argv, &i, Arg, 0, 3);
	  else if (strcmp (Arg, "-faset") == 0 && i < argc - 1)
	    (*pIn).faset = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-nsite") == 0 && i < argc - 1)
	    (*pIn).nsite = NextArgAsInt (argv, &i, Arg, 0, 4);
	  else if (strcmp (Arg, "-nsitewidth") == 0 && i < argc - 1)
	    (*pIn).nsitesize = NextArgAsReal (argv, &i, Arg, 0, 1e32);
	  else if (strcmp (Arg, "-ps2d") == 0 && i < argc - 1)
	    (*pIn).ps2d = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-msize") == 0 && i < argc - 1)
	  {
	    (*pMapMesh).MeshSize[0] = NextArgAsInt (argv, &i, Arg, 1, 1e7);
	    ut_board_1d_int_set ((*pMapMesh).MeshSize, 3, (*pMapMesh).MeshSize[0]);
	  }
	  else if (strcmp (Arg, "-msize3") == 0 && i < argc - 1)
	  {
	    (*pMapMesh).MeshSize[0] = NextArgAsInt (argv, &i, Arg, 1, 1e7);
	    (*pMapMesh).MeshSize[1] = NextArgAsInt (argv, &i, Arg, 1, 1e7);
	    (*pMapMesh).MeshSize[2] = NextArgAsInt (argv, &i, Arg, 1, 1e7);
	  }
	  else if (strcmp (Arg, "-morder") == 0 && i < argc - 1)
	    (*pMapMesh).MeshOrder = NextArgAsInt (argv, &i, Arg, 1, 2);
	  else if (strcmp (Arg, "-mesh") == 0)
	    (*pMMDef).mesh = NextArgAsInt (argv, &i, Arg, 0, 1);
	  else if (strcmp (Arg, "-lmesh") == 0)
	    {
	      (*pMMDef).loadmesh = NextArgAsChar (argv, &i, Arg);
	      (*pMMDef).mesh = 2;
	    }
	  else if (strcmp (Arg, "-v") == 0 && i < argc - 1)
	    (*pIn).verbosity = NextArgAsInt (argv, &i, Arg, 0, 2);
	  else
	    {
	      BadArguments ();
	      exit (0);
	    }
	}
    }

  twoDCharFree (ArgList, 101);
  oneDCharFree (Arg);


  return;
}
