/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "RMeshing.h"

void
RMeshing (struct NODES *pNodes, struct MESH *pMesh,
          int ***pFoDNodes, struct IN In, struct DNEIGH *pDNeigh,
	  struct MMDEF MMDef, struct MAPMESH *pMap)
{
  int i, j, k, elt;
  int EltQty;
  struct GEO Geo;
  FILE* in;
  struct TESS Tess;

  /* Calculating element coordinates ------------------------------- */

  EltQty = (*pMap).MeshSize[0] * (*pMap).MeshSize[1] * (*pMap).MeshSize[2];

  (*pMap).EltCoo = gsl_matrix_float_calloc (EltQty + 1, 3);
  (*pMap).EltPoly = gsl_vector_int_calloc (EltQty + 1);
  (*pMap).EltId = (size_t *) calloc (EltQty + 1, sizeof (size_t));

  elt = 0;
  for (k = 1; k <= (*pMap).MeshSize[2]; k++)
    for (j = 1; j <= (*pMap).MeshSize[1]; j++)
      for (i = 1; i <= (*pMap).MeshSize[0]; i++)
	{
	  elt++;
	  gsl_matrix_float_set ((*pMap).EltCoo, elt, 0,
				((double) i -
				 0.5) / (double) (*pMap).MeshSize[0]);
	  gsl_matrix_float_set ((*pMap).EltCoo, elt, 1,
				((double) j -
				 0.5) / (double) (*pMap).MeshSize[1]);
	  gsl_matrix_float_set ((*pMap).EltCoo, elt, 2,
				((double) k -
				 0.5) / (double) (*pMap).MeshSize[2]);
	}

  /* Creating mapped mesh ------------------------------------------ */

  if (MMDef.mesh == 1)
    {
      ut_print_message (0, "Creating mapped mesh ...\n");
      BuildMMesh3D ((*pMap).MeshSize, (*pMap).MeshOrder, In.nset, pNodes, pMesh, pFoDNodes);

      ut_print_message (0, "Created mapped mesh.\n");
    }

  /* Searching elsets ---------------------------------------------- */

  ut_print_message (0, "Searching elsets ...\n");

  /* Calculation of EltPoly */
  if (In.input == 0)
  {
    in = ut_file_open (In.tess, "r");
    TessImport (&Tess, in);
    ut_file_close (in, In.tess, "r");

    Geo.DomainSize = ut_alloc_2d (3, 2);
    Tess2GeoDomainSize (Tess, &Geo);

    ut_nodes_scale (pNodes, Geo.DomainSize[0][1] - Geo.DomainSize[0][0],
			    Geo.DomainSize[1][1] - Geo.DomainSize[1][0],
			    Geo.DomainSize[2][1] - Geo.DomainSize[2][0]);
    ut_nodes_shift (pNodes, Geo.DomainSize[0][0],
			    Geo.DomainSize[1][0],
			    Geo.DomainSize[2][0]);
    ut_mmesh_scale (pMap,   Geo.DomainSize[0][1] - Geo.DomainSize[0][0],
			    Geo.DomainSize[1][1] - Geo.DomainSize[1][0],
			    Geo.DomainSize[2][1] - Geo.DomainSize[2][0]);
    ut_mmesh_shift (pMap,   Geo.DomainSize[0][0],
			    Geo.DomainSize[1][0],
			    Geo.DomainSize[2][0]);

    WriteGeofMesh ((*pMap).MeshSize, (*pMap).MeshOrder, In.nset, In.faset, *pNodes, *pMesh, *pFoDNodes, MMDef.geof);
      
    RMeshingFTess (Tess, pMap, pDNeigh, In.verbosity);
  }
  else
  {
    WriteGeofMesh ((*pMap).MeshSize, (*pMap).MeshOrder, In.nset, In.faset, *pNodes, *pMesh, *pFoDNodes, MMDef.geof);
    RMeshingFNId (In, pMap, pDNeigh);
  }

  /* Calculation of EltId */
  gsl_sort_int_index ((*pMap).EltId, (*pMap).EltPoly->data, 1, EltQty + 1);

  ut_print_message (0, "Searched elsets.\n");

  return;
}
