/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_t.h"

/* SetDefaultOptions set the options to their default values */
void
SetDefaultOptions_t (struct IN *pIn, struct GERMSET *pGermSet)
{
  (*pIn).size = ut_alloc_1d (7);	/* size of the domain to tesselate */

  (*pIn).size[1] = 0;
  (*pIn).size[2] = 1;
  (*pIn).size[3] = 0;
  (*pIn).size[4] = 1;
  (*pIn).size[5] = 0;
  (*pIn).size[6] = 1;
  
  (*pIn).F = ut_alloc_1d (3);	/* size of the domain to tesselate */

  (*pIn).F[0] = 1;
  (*pIn).F[1] = 1;
  (*pIn).F[2] = 1;

  (*pIn).ps2d = 0;
  (*pIn).vcoocalc = 0;
  (*pIn).verbosity = 0;

  (*pGermSet).N = 100;
  (*pGermSet).Id = -1;
  (*pGermSet).ttype = 0;

  (*pIn).print = 0;
  (*pIn).printfaces = ut_alloc_1d_int (7);
  (*pIn).printfaces[0] = 3;
  (*pIn).printfaces[1] = 2;
  (*pIn).printfaces[2] = 4;
  (*pIn).printfaces[3] = 6;

  (*pIn).input = -1;
  (*pIn).tess = NULL;
  (*pIn).gcoo = oneDCharDAlloc (1000);
  
  (*pIn).printpointpoly = 0;
  (*pIn).point = ut_alloc_1d_char (100);
  (*pIn).polyid = ut_alloc_1d_char (100);

  return;
}

void
SetOptions_t (struct IN *pIn, struct GERMSET *pGermSet, char **pbody,
	      int *pinibody, int argc, char **argv)
{
  int i, j, ArgQty, Res;
  char **ArgList = twoDCharDAlloc (101, 101);
  char *Arg = oneDCharDAlloc (101);

  /* This is the possible argument list. */
  ArgQty = 13;
  sprintf (ArgList[1], "-o");
  sprintf (ArgList[2], "-n");
  sprintf (ArgList[3], "-id");
  sprintf (ArgList[4], "-vcoo");
  sprintf (ArgList[5], "-ps2d");
  sprintf (ArgList[6], "-v");
  sprintf (ArgList[7], "-dsize");
  sprintf (ArgList[8], "-deformation");
  sprintf (ArgList[9], "-print");
  sprintf (ArgList[10], "-printfaces");
  sprintf (ArgList[11], "-gcoo");
  sprintf (ArgList[12], "-pointpoly");
  sprintf (ArgList[13], "-tesscentrepoly");

  for (i = 1; i < argc; i++)
    {
      if (argv[i][0] != '-')
	{
	  if ((*pIn).input == -1)
	    {
	      (*pIn).tess = oneDCharDAlloc (strlen (argv[i]));
	      (*pIn).tess = argv[i];
	      (*pIn).input = 0;
	    }
	  else
	    BadArguments ();
	}
      else
	{
	/* Searching option name (string completion stuff) */
	Res = StringComp (argv[i], ArgList, ArgQty, Arg);
	if (Res == 1)
	  {
	    LineHeader (2);
	    printf ("Several possibilities for option `%s'.\n", argv[i]);
	    BadArguments ();
	  }
	else if (Res == -1)
	  {
	    LineHeader (2);
	    printf ("Unknown option `%s'.\n", argv[i]);
	    BadArguments ();
	  }

	/* That's ok with the option; reading it. */
	if (strcmp (Arg, "-o") == 0 && i < argc - 1)
	  {
	    (*pbody) = NextArgAsChar (argv, &i, Arg);
	    (*pbody) = GiveBody ((*pbody));
	    (*pinibody) = 1;
	  }
	else if (strcmp (Arg, "-n") == 0 && i < argc - 1)
	  {
	    if ((*pIn).input == -1)
	    {
	      (*pIn).input = 1;
	      (*pGermSet).N = NextArgAsInt (argv, &i, Arg, 0, 1e7);
	    }
	    else
	      BadArguments ();
	  }
	else if (strcmp (Arg, "-id") == 0 && i < argc - 1)
	{
	  if ((*pGermSet).Id == 0)
	  {
	    ut_print_message (2, "Functions `-id' and `-gcoo' are mutually exclusive!");
	    abort();
	  }
	  (*pGermSet).Id = NextArgAsInt (argv, &i, Arg, 1, 1e7);
	}
	else if (strcmp (Arg, "-dsize") == 0 && i < argc - 1)
	  {
	    (*pIn).size[2] = NextArgAsReal (argv, &i, Arg, 0, 1e32);
	    (*pIn).size[4] = NextArgAsReal (argv, &i, Arg, 0, 1e32);
	    (*pIn).size[6] = NextArgAsReal (argv, &i, Arg, 0, 1e32);
	  }
	else if (strcmp (Arg, "-deformation") == 0 && i < argc - 1)
	  {
	    (*pIn).F[0] = NextArgAsReal (argv, &i, Arg, 0, 1e32);
	    (*pIn).F[1] = NextArgAsReal (argv, &i, Arg, 0, 1e32);
	    (*pIn).F[2] = NextArgAsReal (argv, &i, Arg, 0, 1e32);
	  }
	else if (strcmp (Arg, "-vcoo") == 0)
	  (*pIn).vcoocalc = NextArgAsInt (argv, &i, Arg, 0, 1);
	else if (strcmp (Arg, "-gcoo") == 0)
	{
	  if ((*pGermSet).Id > 0)
	  {
	    ut_print_message (2, "Functions `-id' and `-gcoo' are mutually exclusive!");
	    abort();
	  }
	  (*pGermSet).Id = 0;
	  (*pIn).gcoo = NextArgAsChar (argv, &i, Arg);
	}
	else if (strcmp (Arg, "-ps2d") == 0)
	  (*pIn).ps2d = NextArgAsInt (argv, &i, Arg, 0, 1);
	else if (strcmp (Arg, "-v") == 0 && i < argc - 1)
	  (*pIn).verbosity = NextArgAsInt (argv, &i, Arg, 0, 1);
	else if (strcmp (Arg, "-print") == 0 && i < argc - 1)
	  (*pIn).print = NextArgAsInt (argv, &i, Arg, 0, 2);
	else if (strcmp (Arg, "-printfaces") == 0 && i < argc - 1)
	{
	  (*pIn).printfaces[0] = NextArgAsInt (argv, &i, Arg, 1, 6);
	  for (j = 1; j <=(*pIn).printfaces[0]; j++)
	    (*pIn).printfaces[j] = NextArgAsInt (argv, &i, Arg, 1, 6);
	}
	else if (strcmp (Arg, "-pointpoly") == 0 && i < argc - 1)
	{
	  (*pIn).printpointpoly = 1;
	  (*pIn).point = ut_arg_nextaschar (argv, &i, Arg);
	}
	else if (strcmp (Arg, "-tesscentrepoly") == 0 && i < argc - 1)
	{
	  (*pIn).printpointpoly = 2;
	  (*pIn).point = ut_arg_nextaschar (argv, &i, Arg);
	}
	else
	  {
	    BadArguments ();
	    exit (0);
	  }
      }
    }

  oneDCharFree (Arg);
  twoDCharFree (ArgList, 101);

  return;
}
