/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"Tess.h"

/* Tessellation lies the polyhedra by suppressing vertices and faces
 * defined several times. That results in the Tess structure, which
 * contains all the information about the tessellation in Voronoi
 * polyhedra.
 */
void
Tessellation (struct GERMSET GermSet, struct POLY *Poly, struct TESS *pTess,
	     int verbosity)
{
/* TVNb and TFNb are boards containing the rellation between the
 * POLY-defined entities (ver & face) and the new entities
 * (TVer ver & TFace face).
 * SofTVNb ("Size of...") & SofTFNb contains the line lengths of TVNb &
 * TFNb: SofTVNb[i] = qty of vertices of poly i.
 */
  int **TVNb;
  int **TFNb;

  if (verbosity >= 1)
    MessageWNC (0, 72, "Building tessellation...");

  /* Recording of the tessellation generalities in Tess, i.e. number
   * of poly, vertices, faces and edges.
   */
  TessGenDet (GermSet, Poly, pTess);

  TVNb = oneDpIntDAlloc ((*pTess).PolyQty + 1);
  TFNb = oneDpIntDAlloc ((*pTess).PolyQty + 1);

  IniTxNb (*pTess, Poly, TFNb, TVNb);

  TessAlloc (pTess);

/* ************************************************************************
 * Recording of face properties
 */
  /* For each polyhedron, recording of FaceGerm, FaceEq & FaceVerQty
   */
  IniTessFaceOr (pTess, Poly, TFNb);

  /* For each polyhedron, recording of FaceVerNb
   */
  IniTessFaceBound (pTess, Poly, TFNb, TVNb);

/* ************************************************************************
 * Recording of vertex properties
 */
  IniTessVer (pTess, Poly, TVNb);

/* ************************************************************************
 * Recording of polyhedron properties
 */
  /* Recording of the center coordinates and the numbers of face and
   * vertex of the polyhedra
   */
  IniTessPolyCenNXQty (GermSet, pTess, Poly);

  /* Recording of the face and vertex numbers.
   */
  IniTessPolyXNb (pTess, Poly, TVNb, TFNb);

/* ************************************************************************
 * Recording of edge properties
 */
  /* Recording of the edges.
   */
  IniTessEdge (pTess);

  twoDIntFree (TVNb, (*pTess).PolyQty + 1);
  twoDIntFree (TFNb, (*pTess).PolyQty + 1);

  /* TessDes(pTess); */

/* ************************************************************************
 * Recording of the edge lengths from the edge properties, and of the
 * vertex parent edges.
 */
  IniTessVernEdge (pTess);

  return;
}
