/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "neper_t.h"

int
neper_t (int fargc, char **fargv, int argc, char **argv)
{
/* *********************************************************
 * STRUCTURE AND VARIABLE DEFINITIONS  */
  /* See the structure files for details. */
  FILE* file;
  struct INTEPARA IntePara;
  struct POLY Domain;
  struct GERMSET GermSet;
  struct POLYMOD Polymod;
  struct POLY *Poly;
  struct TESS Tess;
  struct IN In;

  char *mname = "-T";

  ModuleHeader (mname, fargc, fargv, argc, argv);

/* *********************************************************
 * PARAMETER, INPUT DATA AND VARIABLES INITIALIZATION  */
  /* InputData initializes all the parameters, input data and related
   * variables. */
  InputData_t (&In, &GermSet, fargc, fargv, argc, argv);

  if (In.input == 0)
  {
    file = ut_file_open (In.tess, "r") ;
    TessImport (&Tess, file);
    ut_file_close (file, In.tess, "r") ;
  }
  else
  {
    /* IniInternalPara initializes the internal parameters -those
     * of the IntePara structure. */
    InternalPara (&IntePara);

    ut_print_message (0, "Creating tessellation ...\n");

  /* *********************************************************
   * STUDY OF THE GERM DISTRIBUTION  */
    /* GermsNei analyses the neighbourhood of every germ: neighbour
     * numbers and distances. */
    DnGDistrib (In, &GermSet, &Domain, In.size, In.ps2d);

  /* *********************************************************
   * POLYEDRA DETERMINATION  */
    /* PolyComp decomposes the cube into the set of Voronoi polyhedra
     * and records them into Poly (which is allocated in the routine). */
    PolyComp (&IntePara, Domain, GermSet, &Polymod, &Poly, In.verbosity);

  /* *********************************************************
   * TESSELATION CONSTRUCTION  */
    /* Tessellation lies the polyhedra by suppressing vertices and faces
     * defined several times. That results in the Tess structure, which
     * contains all the information about the Voronoi tessellation. */
    Tessellation (GermSet, Poly, &Tess, In.verbosity);

    ut_print_message (0, "Created tessellation.\n");
  }

  TessDeform (&Tess, In.F);

  /* PolyFree(Poly,GermSet.N); */

/* *********************************************************
 * RESULTS WRITING  */
  Res_t (In, Tess);
  
  return 0;
}
