/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */
 
#include"ut_alloc.h"

int *
ut_alloc_1d_int (unsigned int line)
{
  int *res = (int *) calloc (line, sizeof (int));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

unsigned int *
ut_alloc_1d_uint (unsigned int line)
{
  unsigned int *res = (unsigned int *) calloc (line, sizeof (int));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

float *
ut_alloc_1d_float (unsigned int line)
{
  float *res = (float *) calloc (line, sizeof (float));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

double *
ut_alloc_1d_double (unsigned int line)
{
  double *res = (double *) calloc (line, sizeof (double));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

double *
ut_alloc_1d (unsigned int line)
{
  return ut_alloc_1d_double (line);
}

char *
ut_alloc_1d_char (unsigned int line)
{
  char *res = (char *) calloc (line, sizeof (char));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

FILE **
ut_alloc_1d_pfile (unsigned int line)
{
  FILE **res = (FILE **) calloc (line, sizeof (FILE *));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

size_t *
ut_alloc_1d_sizet (unsigned int line)
{
  size_t *res = (size_t *) calloc (line, sizeof (size_t));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

int **
ut_alloc_2d_int (unsigned int line, unsigned int column)
{
  unsigned int i;
  int **res = (int **) calloc (line, sizeof (int *));
  if (!res)
  {
    printf ("memory allocation failed! (%dx%d)\n", line, column);
    abort ();
  }

  for (i = 0; i < line; i++)
  {
    res[i] = (int *) calloc (column, sizeof (int));
    if (!res[i])
    {
      printf ("memory allocation failed! (%dx%d)\n", line, column);
      abort ();
    }
  }

  return res;
}

unsigned int **
ut_alloc_2d_uint (unsigned int line, unsigned int column)
{
  unsigned int i;
  unsigned int **res = (unsigned int **) calloc (line, sizeof (int *));
  if (!res)
  {
    printf ("memory allocation failed! (%dx%d)\n", line, column);
    abort ();
  }

  for (i = 0; i < line; i++)
  {
    res[i] = (unsigned int *) calloc (column, sizeof (int));
    if (!res[i])
    {
      printf ("memory allocation failed! (%dx%d)\n", line, column);
      abort ();
    }
  }

  return res;
}

float **
ut_alloc_2d_float (unsigned int line, unsigned int column)
{
  unsigned int i;
  float **res = (float **) calloc (line, sizeof (float *));
  if (!res)
  {
    printf ("memory allocation failed! (%dx%d)\n", line, column);
    abort ();
  }

  for (i = 0; i < line; i++)
  {
    res[i] = (float *) calloc (column, sizeof (float));
    if (!res[i])
    {
      printf ("memory allocation failed! (%dx%d)\n", line, column);
      abort ();
    }
  }

  return res;
}

double **
ut_alloc_2d_double (unsigned int line, unsigned int column)
{
  unsigned int i;
  double **res = (double **) calloc (line, sizeof (double *));
  if (!res)
  {
    printf ("memory allocation failed! (%dx%d)\n", line, column);
    abort ();
  }

  for (i = 0; i < line; i++)
  {
    res[i] = (double *) calloc (column, sizeof (double));
    if (!res[i])
    {
      printf ("memory allocation failed! (%dx%d)\n", line, column);
      abort ();
    }
  }

  return res;
}

double **
ut_alloc_2d (unsigned int line, unsigned int column)
{
  return ut_alloc_2d_double (line, column);
}

char **
ut_alloc_2d_char (unsigned int line, unsigned int column)
{
  unsigned int i;
  char **res = (char **) calloc (line, sizeof (char *));
  if (!res)
  {
    printf ("memory allocation failed! (%dx%d)\n", line, column);
    abort ();
  }

  for (i = 0; i < line; i++)
  {
    res[i] = (char *) calloc (column, sizeof (char));
    if (!res[i])
    {
      printf ("memory allocation failed! (%dx%d)\n", line, column);
      abort ();
    }
  }

  return res;
}

int ***
ut_alloc_3d_int (unsigned int line, unsigned int column, unsigned int depth)
{
  unsigned int i, j;
  int ***res = (int ***) calloc (line, sizeof (int **));
  if (!res)
  {
    printf ("memory allocation failed! (%dx%dx%d)\n", line, column, depth);
    abort ();
  }

  for (i = 0; i < line; i++)
  {
    res[i] = (int **) calloc (column, sizeof (int *));
    if (!res[i])
    {
      printf ("memory allocation failed! (%dx%dx%d)\n", line, column, depth);
      abort ();
    }

    for (j = 0; j < column; j++)
    {
      res[i][j] = (int *) calloc (depth, sizeof (int));
      if (!res[i][j])
      {
	printf ("memory allocation failed! (%dx%dx%d)\n", line, column,
		depth);
	abort ();
      }
    }
  }

  return res;
}

double ***
ut_alloc_3d_double (unsigned int line, unsigned int column,
		    unsigned int depth)
{
  unsigned int i, j;
  double ***res = (double ***) calloc (line, sizeof (double **));
  if (!res)
  {
    printf ("memory allocation failed! (%dx%dx%d)\n", line, column, depth);
    abort ();
  }

  for (i = 0; i < line; i++)
  {
    res[i] = (double **) calloc (column, sizeof (double *));
    if (!res[i])
    {
      printf ("memory allocation failed! (%dx%dx%d)\n", line, column, depth);
      abort ();
    }

    for (j = 0; j < column; j++)
    {
      res[i][j] = (double *) calloc (depth, sizeof (double));
      if (!res[i][j])
      {
	printf ("memory allocation failed! (%dx%dx%d)\n", line, column,
		depth);
	abort ();
      }
    }
  }

  return res;
}

double ***
ut_alloc_3d (unsigned int line, unsigned int column, unsigned int depth)
{
  return ut_alloc_3d_double (line, column, depth);
}

char ***
ut_alloc_3d_char (unsigned int line, unsigned int column, unsigned int depth)
{
  unsigned int i, j;
  char ***res = (char ***) calloc (line, sizeof (char **));
  if (!res)
  {
    printf ("memory allocation failed! (%dx%dx%d)\n", line, column, depth);
    abort ();
  }

  for (i = 0; i < line; i++)
  {
    res[i] = (char **) calloc (column, sizeof (char *));
    if (!res[i])
    {
      printf ("memory allocation failed! (%dx%dx%d)\n", line, column, depth);
      abort ();
    }
    for (j = 0; j < column; j++)
    {
      res[i][j] = (char *) calloc (depth, sizeof (char));
      if (!res[i])
      {
	printf ("memory allocation failed! (%dx%dx%d)\n", line, column,
		depth);
	abort ();
      }
    }
  }

  return res;
}

int **
ut_alloc_1d_pint (unsigned int line)
{
  int **res = (int **) calloc (line, sizeof (int *));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

char **
ut_alloc_1d_pchar (unsigned int line)
{
  char **res = (char **) calloc (line, sizeof (char *));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

double **
ut_alloc_1d_pdouble (unsigned int line)
{
  double **res = (double **) calloc (line, sizeof (double *));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

int ***
ut_alloc_1d_ppint (unsigned int line)
{
  int ***res = (int ***) calloc (line, sizeof (int **));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

double ***
ut_alloc_1d_ppdouble (unsigned int line)
{
  double ***res = (double ***) calloc (line, sizeof (double **));
  if (!res)
  {
    printf ("memory allocation failed! (%d)\n", line);
    abort ();
  }

  return res;
}

double ****
ut_alloc_4d_double (unsigned int dim1, unsigned int dim2,
		    unsigned int dim3, unsigned int dim4)
{
  unsigned int i, j, k;
  double ****res = (double ****) calloc (dim1, sizeof (double ***));
  if (!res)
  {
    printf ("memory allocation failed! (%dx%dx%dx%d)\n", dim1, dim2, dim3,
	    dim4);
    abort ();
  }

  for (i = 0; i < dim1; i++)
  {
    res[i] = (double ***) calloc (dim2, sizeof (double **));
    if (!res[i])
    {
      printf ("memory allocation failed!\n");
      abort ();
    }

    for (j = 0; j < dim2; j++)
    {
      res[i][j] = (double **) calloc (dim3, sizeof (double *));
      if (!res[i][j])
      {
	printf ("memory allocation failed!\n");
	abort ();
      }

      for (k = 0; k < dim2; k++)
      {
	res[i][j][k] = (double *) calloc (dim4, sizeof (double));
	if (!res[i][j][k])
	{
	  printf ("memory allocation failed!\n");
	  abort ();
	}
      }
    }
  }

  return res;
}

double ****
ut_alloc_4d (unsigned int dim1, unsigned int dim2,
	     unsigned int dim3, unsigned int dim4)
{
  return ut_alloc_4d_double (dim1, dim2, dim3, dim4);
}
