/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */
 
#include"ut_arg.h"

/* NextArgAsInt increments *pi and check if argv[*pi] is an integer. */
/* if yes, and if it bw min and max (strictly), it is returned; */
/* else an error message is generated. */
int
ut_arg_nextasint (char **argv, int *pi, char *option, int min, int max)
{
  int res;
  int length;
  int i;

  /* incrementing i */
  (*pi)++;

  /* reading the argument and testing if it is an int */
  if (sscanf (argv[*pi], "%d", &res) == 0)
    ut_arg_error (option, "not an integer value.");

  length = strlen (argv[*pi]);
  for (i = 0; i <= length - 1; i++)
    if (argv[*pi][i] == '.')
      ut_arg_error (option, "not an integer value.");

  /* checking bounds */
  if (res < min || res > max)
    ut_arg_error (option, "bad integer value.");

  return res;
}

/* Same than NextArgAsInt but for reals. */
double
ut_arg_nextasreal (char **argv, int *pi, char *option, double min, double max)
{
  double res;
  double eps = 1e-15;

  /* incrementing i */
  (*pi)++;

  /* reading the argument and testing if it is an int */
  if (sscanf (argv[*pi], "%lf", &res) == 0)
    ut_arg_error (option, "not a real value.");

  if (res < min - eps || res > max + eps)
    ut_arg_error (option, "bad real value.");

  return res;
}

/* Same than NextArgAsInt but for chars. */
char *
ut_arg_nextaschar (char **argv, int *pi, char *option)
{
  char *res = ut_alloc_1d_char (1000);

  /* incrementing i */
  (*pi)++;

  /* reading the argument and testing if it is an int */
  if (sscanf (argv[*pi], "%s", res) == 0)
    ut_arg_error (option, "not a char string value.");

  /*if(res<min-eps || res>max+eps) */
  /*ut_arg_error(option); */

  return res;
}


void
ut_arg_badarg (void)
{
  ut_print_lineheader (2);
  printf ("Bad arguments!\n\n");
  exit (1);

  return;
}

void
ut_arg_error (char *arg, char* explaination)
{
  ut_print_lineheader (2);
  printf ("argument \'%s\': %s\n", arg, explaination);
  exit (1);

  return;
}

int
ut_arg_listqty (char *arg)
{
  return 1 + ut_string_thischarqty (arg, ',');
}

char *
ut_arg_listmember (char *arg, int qty, int nb)
{
  int i, pos1, pos2, length;
  char *res;

  /* searching pos1, the character/comma before the arg */
  pos1 = ut_string_thischarpos (arg, ',', nb - 1);

  /* searching pos2, the character/comma after the arg */
  if (nb < qty)
    pos2 = ut_string_thischarpos (arg, ',', nb);
  else
    pos2 = strlen (arg);

  length = pos2 - pos1 - 1;
  res = ut_alloc_1d_char (length + 1);
  for (i = 0; i < length; i++)
    res[i] = arg[pos1 + i + 1];
  res[length] = '\0';

  return res;
}
