/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */
 
#include"ut_board.h"

void
ut_board_1d_scale (double *board, int qty, double scale)
{
  int i;

  for (i = 0; i < qty; i++)
    board[i] *= scale;

  return;
}

void
ut_board_1d_int_scale (int *board, int qty, int scale)
{
  int i;

  for (i = 0; i < qty; i++)
    board[i] *= scale;

  return;
}

extern void
ut_board_1d_add (double *a, double *b, int qty, double *c)
{
  int i;

  for (i = 0; i < qty; i++)
    c[i] = a[i] + b[i];

  return;
}

double
ut_board_1d_norm (double *a, int size)
{
  int i;
  double norm;

  norm = 0;
  for (i = 0; i < size; i++)
    norm += a[i] * a[i];

  return sqrt (norm);
}

double
ut_board_1d_norm_int (int *a, int size)
{
  int i;
  double norm;

  norm = 0;
  for (i = 0; i < size; i++)
    norm += (double) a[i] * (double) a[i];

  return sqrt ((double) norm);
}

void
ut_board_1d_zero (double *a, int size)
{
  int i;

  for (i = 0; i < size; i++)
    a[i] = 0;

  return;
}

void
ut_board_1d_int_zero (int *a, int size)
{
  int i;

  for (i = 0; i < size; i++)
    a[i] = 0;

  return;
}

void
ut_board_2d_zero (double **a, int size1, int size2)
{
  int i, j;

  for (i = 0; i < size1; i++)
    for (j = 0; j < size2; j++)
      a[i][j] = 0;

  return;
}

void
ut_board_2d_int_zero (int **a, int size1, int size2)
{
  int i, j;

  for (i = 0; i < size1; i++)
    for (j = 0; j < size2; j++)
      a[i][j] = 0;

  return;
}

int
ut_board_1d_fscanf (FILE * file, double *a, int size)
{
  int i, res = 0;

  for (i = 0; i < size; i++)
    res = fscanf (file, "%lf", &a[i]);

  return res;
}

int
ut_board_1d_int_fscanf (FILE * file, int *a, int size)
{
  int i, res = 0;

  for (i = 0; i < size; i++)
    res = fscanf (file, "%d", &a[i]);

  return res;
}

int
ut_board_1d_fprintf (FILE * file, double *a, int size, char *format)
{
  int i, res;

  for (i = 0; i < size; i++)
  {
    fprintf (file, format, a[i]);
    fprintf (file, " ");
  }
  res = fprintf (file, "\n");

  return res;
}

int
ut_board_2d_fscanf (FILE * file, double **a, int d1, int d2)
{
  int i, j, res = 0;

  for (i = 0; i < d1; i++)
    for (j = 0; j < d2; j++)
      res = fscanf (file, "%lf", &a[i][j]);

  return res;
}

int
ut_board_1d_int_fprintf (FILE * file, int *a, int size, char *format)
{
  int i, res;

  for (i = 0; i < size; i++)
  {
    fprintf (file, format, a[i]);
    fprintf (file, " ");
  }
  res = fprintf (file, "\n");

  return res;
}

int
ut_board_2d_int_fprintf (FILE * file, int **a, int size1, int size2,
			 char *format)
{
  int i, j, res;

  res = 0;
  for (i = 0; i < size1; i++)
  {
    for (j = 0; j < size2; j++)
    {
      res = fprintf (file, format, a[i][j]);
      res = fprintf (file, " ");
    }
    res = fprintf (file, "\n");
  }

  return res;
}

int
ut_board_2d_fprintf (FILE * file, double **a, int size1, int size2,
		     char *format)
{
  int i, j, res;

  res = 0;
  for (i = 0; i < size1; i++)
  {
    for (j = 0; j < size2; j++)
    {
      res = fprintf (file, format, a[i][j]);
      res = fprintf (file, " ");
    }
    res = fprintf (file, "\n");
  }

  return res;
}

int
ut_board_2d_int_fscanf (FILE * file, int **a, int d1, int d2)
{
  int i, j, res = 0;

  for (i = 0; i < d1; i++)
    for (j = 0; j < d2; j++)
      res = fscanf (file, "%d", &a[i][j]);

  return res;
}

void
ut_board_2d_scale (double **a, int d1, int d2, double scale)
{
  int i, j;

  for (i = 0; i < d1; i++)
    for (j = 0; j < d2; j++)
      a[i][j] *= scale;

  return;
}

double
ut_board_1d_min (double *a, int size)
{
  int i;
  double min;

  min = a[0];
  for (i = 0; i < size; i++)
    if (a[i] < min)
      min = a[i];

  return min;
}

double
ut_board_1d_mean (double *a, int size)
{
  int i;
  double mean = 0;

  for (i = 0; i < size; i++)
    mean += a[i];
  mean /= size;

  return mean;
}

double
ut_board_1d_wmean (double *a, double *w, int size)
{
  int i;
  double sum, mean = 0;

  sum = 0;
  for (i = 0; i < size; i++)
  {
    mean += a[i] * w[i];
    sum += w[i];
  }

  mean /= sum;

  return mean;
}

double
ut_board_1d_stddev (double *a, double mean, int size)
{
  int i;
  double stddev;

  stddev = 0;
  for (i = 0; i < size; i++)
    stddev += (a[i] - mean) * (a[i] - mean);
  stddev /= size;
  stddev = sqrt (stddev);

  return stddev;
}

int
ut_board_1d_min_index (double *a, int size)
{
  int i, pos;
  double min;

  pos = 0;
  min = a[0];
  for (i = 0; i < size; i++)
    if (a[i] < min)
    {
      min = a[i];
      pos = i;
    }

  return pos;
}

int
ut_board_1d_min2_index (double *a, int size)
{
  int i, pos;
  double min, min2;

  min = ut_board_1d_min (a, size);

  pos = 0;
  min2 = 1e32;
  for (i = 0; i < size; i++)
    if (a[i] < min2 && a[i] > min)
    {
      min2 = a[i];
      pos = i;
    }

  return pos;
}


double
ut_board_1d_max (double *a, int size)
{
  int i;
  double max;

  max = a[0];
  for (i = 0; i < size; i++)
    if (a[i] > max)
      max = a[i];

  return max;
}

double
ut_board_1d_absmax (double *a, int size)
{
  int i;
  double absmax;

  absmax = fabs (a[0]);
  for (i = 0; i < size; i++)
    if (fabs (a[i]) > absmax)
      absmax = fabs (a[i]);

  return absmax;
}

int
ut_board_1d_absmax_index (double *a, int size)
{
  int i, pos;
  double absmax;

  pos = 0;
  absmax = fabs (a[0]);
  for (i = 0; i < size; i++)
    if (fabs (a[i]) > absmax)
    {
      absmax = fabs (a[i]);
      pos = i;
    }

  return pos;
}

int
ut_board_1d_max_index (double *a, int size)
{
  int i, pos;
  double max;

  pos = 0;
  max = a[0];
  for (i = 0; i < size; i++)
    if (a[i] > max)
    {
      max = a[i];
      pos = i;
    }

  return pos;
}


int
ut_board_1d_int_min (int *a, int size)
{
  int i, min;

  min = a[0];
  for (i = 0; i < size; i++)
    if (a[i] < min)
      min = a[i];

  return min;
}

int
ut_board_1d_int_max (int *a, int size)
{
  int i, max;

  max = a[0];
  for (i = 0; i < size; i++)
    if (a[i] > max)
      max = a[i];

  return max;
}

int
ut_board_1d_max_int_index (int *a, int size)
{
  int i, pos, max;

  pos = 0;
  max = a[0];
  for (i = 0; i < size; i++)
    if (a[i] > max)
    {
      max = a[i];
      pos = i;
    }

  return pos;
}

void
ut_board_1d_memcpy (double *dest, int size, double *src)
{
  int i;

  for (i = 0; i < size; i++)
    dest[i] = src[i];

  return;
}

void
ut_board_1d_int_memcpy (int *dest, int size, int *src)
{
  int i;

  for (i = 0; i < size; i++)
    dest[i] = src[i];

  return;
}

void
ut_board_2d_memcpy (double **dest, int size1, int size2, double **src)
{
  int i, j;

  for (i = 0; i < size1; i++)
    for (j = 0; j < size2; j++)
      dest[i][j] = src[i][j];

  return;
}

void
ut_board_2d_int_memcpy (int **dest, int size1, int size2, int **src)
{
  int i, j;

  for (i = 0; i < size1; i++)
    for (j = 0; j < size2; j++)
      dest[i][j] = src[i][j];

  return;
}


/* The subroutine sorts the board from element beg */
/* to element end, using the bubble sorting algorithm. */
void
ut_board_1d_sort (double *board, int size)
{
  int i, j;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (board[j - 1] > board[j])
	ut_board_1d_exchange (board, j - 1, j);

  return;
}

void
ut_board_1d_sort_des (double *board, int size)
{
  int i, j;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (board[j - 1] < board[j])
	ut_board_1d_exchange (board, j - 1, j);

  return;
}

/* The subroutine indirectly sorts the board from element beg */
/* to element end, using the bubble sorting algorithm. */
/* the input board isn't modified */
void
ut_board_1d_sort_index (double *board, int size, int *pos)
{
  int i, j;
  double *boardcpy = ut_alloc_1d (size);

  ut_board_1d_memcpy (boardcpy, size, board);

  for (i = 0; i < size; i++)
    pos[i] = i;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (boardcpy[j - 1] > boardcpy[j])
      {
	ut_board_1d_exchange (boardcpy, j - 1, j);
	ut_board_1d_int_exchange (pos, j - 1, j);
      }

  ut_free_1d (boardcpy);

  return;
}

void
ut_board_1d_int_sort (int *board, int size)
{
  int i, j;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (board[j - 1] > board[j])
	ut_board_1d_int_exchange (board, j - 1, j);

  return;
}

void
ut_board_1d_int_dsort (int *board, int size)
{
  int i, j;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (board[j - 1] < board[j])
	ut_board_1d_int_exchange (board, j - 1, j);

  return;
}


void
ut_board_1d_int_sort_index (int *board, int size, int *pos)
{
  int i, j;
  int *boardcpy = ut_alloc_1d_int (size);

  ut_board_1d_int_memcpy (boardcpy, size, board);

  for (i = 0; i < size; i++)
    pos[i] = i;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (boardcpy[j - 1] > boardcpy[j])
      {
	ut_board_1d_int_exchange (boardcpy, j - 1, j);
	ut_board_1d_int_exchange (pos, j - 1, j);
      }

  ut_free_1d_int (boardcpy);

  return;
}

void
ut_board_1d_int_dsort_index (int *board, int size, int *pos)
{
  int i, j;
  int *boardcpy = ut_alloc_1d_int (size);

  ut_board_1d_int_memcpy (boardcpy, size, board);

  for (i = 0; i < size; i++)
    pos[i] = i;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (boardcpy[j - 1] < boardcpy[j])
      {
	ut_board_1d_int_exchange (boardcpy, j - 1, j);
	ut_board_1d_int_exchange (pos, j - 1, j);
      }

  ut_free_1d_int (boardcpy);

  return;
}


/* The subroutine sorts the board from element beg */
/* to element end, using the bubble sorting algorithm. */
void
ut_board_1d_abssort (double *board, int size)
{
  int i, j;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (fabs (board[j - 1]) > fabs (board[j]))
	ut_board_1d_exchange (board, j - 1, j);

  return;
}

/* The subroutine indirectly sorts the board from element beg */
/* to element end, using the bubble sorting algorithm. */
void
ut_board_1d_abssort_index (double *board, int size, int *pos)
{
  int i, j;
  double *boardcpy = ut_alloc_1d (size);

  ut_board_1d_memcpy (boardcpy, size, board);

  for (i = 0; i < size; i++)
    pos[i] = i;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (fabs (boardcpy[j - 1]) > fabs (boardcpy[j]))
      {
	ut_board_1d_exchange (boardcpy, j - 1, j);
	ut_board_1d_int_exchange (pos, j - 1, j);
      }

  ut_free_1d (boardcpy);

  return;
}

void
ut_board_1d_int_abssort_index (int *board, int size, int *pos)
{
  int i, j;
  int *boardcpy = ut_alloc_1d_int (size);

  ut_board_1d_int_memcpy (boardcpy, size, board);

  for (i = 0; i < size; i++)
    pos[i] = i;

  for (i = 0; i < size; i++)
    for (j = size - 1; j > i; j--)
      if (fabs (boardcpy[j - 1]) > fabs (boardcpy[j]))
      {
	ut_board_1d_int_exchange (boardcpy, j - 1, j);
	ut_board_1d_int_exchange (pos, j - 1, j);
      }

  ut_free_1d_int (boardcpy);

  return;
}


void
ut_board_1d_exchange (double *board, int n1, int n2)
{
  double tmp;

  tmp = board[n1];
  board[n1] = board[n2];
  board[n2] = tmp;

  return;
}

void
ut_board_1d_int_exchange (int *board, int n1, int n2)
{
  int tmp;

  tmp = board[n1];
  board[n1] = board[n2];
  board[n2] = tmp;

  return;
}

void
ut_board_1d_int_view (int *board, int n1)
{
  int i;

  for (i = 0; i < n1; i++)
    printf ("%d ", board[i]);

  printf ("\n");

  return;
}

void
ut_board_2d_int_view (int **board, int n1, int n2)
{
  int i, j;

  for (i = 0; i < n1; i++)
  {
    for (j = 0; j < n2; j++)
      printf ("%d ", board[i][j]);
    printf ("\n");
  }

  return;
}

void
ut_board_1d_view (double *board, int n1)
{
  int i;

  for (i = 0; i < n1; i++)
    printf ("%f ", board[i]);

  printf ("\n");

  return;
}

void
ut_board_2d_view (double **board, int n1, int n2)
{
  int i, j;

  for (i = 0; i < n1; i++)
  {
    for (j = 0; j < n2; j++)
      printf ("%f ", board[i][j]);
    printf ("\n");
  }

  return;
}

extern void
ut_board_2d_int_exchangelines (int **board, int n1, int n2,
			       int line1, int line2)
{
  int i, tmp;

  n1 += 0;
  for (i = 0; i < n2; i++)
  {
    tmp = board[line1][i];
    board[line1][i] = board[line2][i];
    board[line2][i] = tmp;
  }

  return;
}

void
ut_board_1d_int_rotate (int *board, int n1, int num)
{
  int i;
  int *copy;

  copy = (int *) ut_alloc_1d_int (n1);

  for (i = 0; i < n1; i++)
    copy[i] = board[i];

  for (i = 0; i < n1; i++)
    board[i] = copy[ut_num_rotpos (0, n1 - 1, i, num)];

  ut_free_1d_int (copy);

  printf ("ut_board_1d_int_rotate to be ckecked\n");
  abort ();

  return;
}

void
ut_board_1d_int_bubbleelt (int *board, int n1, int pos)
{
  int i, val;

  n1 += 0;

  val = board[pos];

  for (i = pos; i > 0; i--)
    board[i] = board[i - 1];

  board[0] = val;

  return;
}

int
ut_board_1d_int_find (int *board, int n1, int val)
{
  int i, res = -1;

  for (i = 0; i < n1; i++)
    if (board[i] == val)
    {
      res = i;
      break;
    }

  return res;
}

void
ut_board_1d_int_reverseelts (int *board, int n1)
{
  int i, tmp;

  for (i = 0; i < n1 / 2; i++)
  {
    tmp = board[i];
    board[i] = board[n1 - 1 - i];
    board[n1 - 1 - i] = tmp;
  }

  return;
}

void
ut_board_1d_permutation (double *board, int qty, int *permutation)
{
  int i;
  double *tmp = ut_alloc_1d (qty);


  for (i = 0; i < qty; i++)
    tmp[i] = board[i];

  for (i = 0; i < qty; i++)
    board[i] = tmp[permutation[i]];

  ut_free_1d_double (tmp);

  return;
}

void
ut_board_1d_permutation_int (int *board, int qty, int *permutation)
{
  int i;
  int *tmp = ut_alloc_1d_int (qty);


  for (i = 0; i < qty; i++)
    tmp[i] = board[i];

  for (i = 0; i < qty; i++)
    board[i] = tmp[permutation[i]];

  ut_free_1d_int (tmp);

  return;
}

void
ut_board_2d_permutation_int (int **board, int line, int col, int *permutation)
{
  int i, j;
  int **tmp = ut_alloc_2d_int (line, col);

  for (i = 0; i < line; i++)
    for (j = 0; j < col; j++)
      tmp[i][j] = board[i][j];

  for (i = 0; i < line; i++)
    for (j = 0; j < col; j++)
      board[i][j] = tmp[permutation[i]][j];

  ut_free_2d_int (tmp, line);

  return;
}

void
ut_board_distribparam (double *data, int n, double step, double *pfirst,
		       int *plength)
{
  double min, max, last;

  min = ut_board_1d_min (data, n);
  max = ut_board_1d_max (data, n);

  (*pfirst) = step * floor (min / step);
  last = step * floor (max / step);

  (*plength) = 1 + ut_num_d2ri ((last - (*pfirst)) / step);

  return;
}

void
ut_board_distrib (double *data, int n, double step, double first, int length,
		  double *distrib)
{
  int i, nb;

  for (i = 0; i < n; i++)
  {
    nb = (int) ((data[i] - first) / step);
    distrib[nb]++;
  }

  for (i = 0; i < length; i++)
    distrib[i] /= (step * (double) n);

  return;
}

double
ut_board_1d_sum (double *data, int n)
{
  int i;
  double res = 0;

  for (i = 0; i < n; i++)
    res += data[i];

  return res;
}

int
ut_board_1d_int_sum (int *data, int n)
{
  int i;
  int res = 0;

  for (i = 0; i < n; i++)
    res += data[i];
  return res;
}

double
ut_board_1d_abssum (double *data, int n)
{
  int i;
  double res = 0;

  for (i = 0; i < n; i++)
    res += fabs (data[i]);

  return res;
}

int
ut_board_1d_int_abssum (int *data, int n)
{
  int i;
  int res = 0;

  for (i = 0; i < n; i++)
    res += fabs (data[i]);
  return res;
}

void
ut_board_1d_abs (double *data, int n)
{
  int i;

  for (i = 0; i < n; i++)
    data[i] = fabs (data[i]);

  return;
}

void
ut_board_1d_int_abs (int *data, int n)
{
  int i;

  for (i = 0; i < n; i++)
    data[i] = ut_num_fabs_int (data[i]);

  return;
}

void
ut_board_1d_set (double *data, int n, double val)
{
  int i;

  for (i = 0; i < n; i++)
    data[i] = val;

  return;
}

void
ut_board_1d_int_set (int *data, int n, int val)
{
  int i;

  for (i = 0; i < n; i++)
    data[i] = val;

  return;
}

void
ut_board_2d_set (double **data, int n, int n2, double val)
{
  int i, j;

  for (i = 0; i < n; i++)
    for (j = 0; j < n2; j++)
      data[i][j] = val;

  return;
}

void
ut_board_2d_int_set (int **data, int n, int n2, int val)
{
  int i, j;

  for (i = 0; i < n; i++)
    for (j = 0; j < n2; j++)
      data[i][j] = val;

  return;
}

void
ut_board_3d_set (double ***data, int n, int n2, int n3, double val)
{
  int i, j, k;

  for (i = 0; i < n; i++)
    for (j = 0; j < n2; j++)
      for (k = 0; k < n3; k++)
	data[i][j][k] = val;

  return;
}

void
ut_board_3d_int_set (int ***data, int n, int n2, int n3, int val)
{
  int i, j, k;

  for (i = 0; i < n; i++)
    for (j = 0; j < n2; j++)
      for (k = 0; k < n3; k++)
	data[i][j][k] = val;

  return;
}

int
ut_board_1d_rotpos (int beg, int end, int pos, int near)
{
  pos += near;

  while (pos > end)
    pos -= (end - beg + 1);

  while (pos < beg)
    pos += (end - beg + 1);

  return pos;
}

int
ut_board_1d_int_eltpos (int *board, int size, int val)
{
  int i;

  for (i = 0; i < size; i++)
    if (board[i] == val)
      return i;

  return -1;
}

int
ut_board_1d_string_eltpos (char ** string, int size, char* val)
{
  int i;

  for (i = 0; i < size; i++)
    if (strcmp (string[i], val) == 0)
      return i;

  return -1;
}

int
ut_board_2d_int_eltpos (int **board, int size1, int size2, int val,
			int *px, int *py)
{
  int i, j;

  (*px) = -1;
  (*py) = -1;

  for (j = 0; j < size2; j++)
    for (i = 0; i < size1; i++)
      if (board[i][j] == val)
      {
	(*px) = i;
	(*py) = j;
	return 1;
      }

  return -1;
}
      
double
ut_board_1d_valuepos (double* board, int size, double val)
{
  int i;
  double pos = -1;

  for (i = 0; i < size - 1; i++)
    if ((val >= board[i] && val <= board[i + 1])
     || (val <= board[i] && val >= board[i + 1]))
    {
      pos = i +  (val - board[i]) / (board[i+1] - board[i]);
      break;
    }
  
  return pos;
}

void
ut_board_1d_inter (int* board1, int size1, int* board2, int size2,
                   int** pres, int* psizeres)
{
  int i, maxsize;
  int *tmp;
  int tmpsize;

  maxsize = ut_num_max_int (size1, size2);
  tmp = ut_alloc_1d_int (maxsize);

  tmpsize = 0;
  for (i = 0; i < size1; i++)
    if (ut_board_1d_int_eltpos (board2, size2, board1[i]) != -1)
      tmp[tmpsize++] = board1[i];

  *psizeres = tmpsize;
  (*pres) = ut_alloc_1d_int (tmpsize);
  ut_board_1d_int_memcpy (*pres, tmpsize, tmp);

  ut_free_1d_int (tmp);

  return;
}

void
ut_board_1d_int_compress (int *board, int* psize)
{
  int i, size2;

  size2 = 0;
  for (i = 0; i < *psize - 1; i++)
    if (board[i] != board[i + 1])
      board[++size2] = board[i + 1];

  *psize = size2 + 1;

  return;
}

void
ut_board_1d_int_sortncompress (int *board, int oldsize, int* psize)
{
  ut_board_1d_int_sort (board, oldsize);
  ut_board_1d_int_compress (board, psize);

  return;
}


void
ut_board_1d_sub (double* data1, double* data2, int size, double* data)
{
  int i;

  for (i = 0; i < size; i++)
    data[i] = data2[i] - data1[i];

  return;
}

void
ut_board_1d_sample (double* b1, int nbl, double factor,
                    double** pb2, int *pnbl2)
{
  int id;
  double factorinv = 1 / factor;
  double rid;

  (*pnbl2) = (unsigned int) ((double) nbl * factor) + 10;

  (*pb2) = ut_alloc_1d (*pnbl2);

  (*pnbl2) = 0;
  rid = 0;
  id = 0;
  while (id < nbl)
  {
    id = ut_num_d2ri (rid);

    (*pb2)[(*pnbl2)++] = b1[id];
    rid += factorinv;
  }

  return;
}

void
ut_board_2d_sample (double** b1, int nbl, int nbc, double factor,
                    double*** pb2, int *pnbl2)
{
  int id;
  double factorinv = 1 / factor;
  int j;
  double rid;

  (*pnbl2) = (int) (((double) nbl * factor) + 10);

  (*pb2) = ut_alloc_2d (*pnbl2, nbc);

  (*pnbl2) = 0;
  rid = 0;
  while (rid < nbl)
  {
    id = ut_num_d2ri (rid);

    for (j = 0; j < nbc; j++)
      (*pb2)[*pnbl2][j] = b1[id][j];

    (*pnbl2)++;

    rid += factorinv;
  }

  return;
}

int
ut_board_1d_fscanf_sample (FILE* file, int tsize, double factor, \
                           double** pb, int* psize)
{
  int res;
  double* data = ut_alloc_1d (tsize);

  res = ut_board_1d_fscanf (file, data, tsize);
  ut_board_1d_sample (data, tsize, factor, pb, psize);

  ut_free_1d (data);

  return res;
}

int
ut_board_2d_fscanf_sample (FILE* file, int tsize, int size2,
                           double factor, double*** pb, int* psize)
{
  int res;
  double** data = ut_alloc_2d (tsize, size2);

  res = ut_board_2d_fscanf (file, data, tsize, size2);
  ut_board_2d_sample (data, tsize, size2, factor, pb, psize);

  ut_free_2d (data, tsize);

  return res;
}

void
ut_board_1d_int_zero_be (int *board, int beg, int end)
{
  int i;

  for (i = beg; i <= end; i++)
    board[i] = 0;

  return;
}

void
ut_board_1d_float_zero_be (float *board, int beg, int end)
{
  int i;

  for (i = beg; i <= end; i++)
    board[i] = 0;

  return;
}

void
ut_board_1d_zero_be (double *board, int beg, int end)
{
  int i;

  for (i = beg; i <= end; i++)
    board[i] = 0;

  return;
}

void
ut_board_2d_int_zero_be (int **board, int begX, int endX, int begY, int endY)
{
  int i, j;

  for (i = begX; i <= endX; i++)
    for (j = begY; i <= endY; i++)
      board[i][j] = 0;

  return;
}

void
ut_board_2d_float_zero_be (float **board, int begX, int endX, int begY, int endY)
{
  int i, j;

  for (i = begX; i <= endX; i++)
    for (j = begY; i <= endY; i++)
      board[i][j] = 0;

  return;
}

void
ut_board_2d_zero_be (double **board, int begX, int endX, int begY, int endY)
{
  int i, j;

  for (i = begX; i <= endX; i++)
    for (j = begY; i <= endY; i++)
      board[i][j] = 0;

  return;
}


int
ut_board_1d_int_nbofthisval (int *board, int size, int val)
{
  int qty = 0;
  int i;

  for (i = 0; i < size; i++)
    if (board[i] == val)
      qty++;

  return qty;
}

void
ut_board_1d_commonelts (int *board1, int size1,
		        int *board2, int size2, int **pres, int* psize)
{
  int i;
  int tmpqty;
  int *tmp = ut_alloc_1d_int (size1);

  tmpqty = 0;
  for (i = 0; i < size1; i++)
  {
    if (ut_board_1d_int_eltpos (board2, size2, board1[i]) != -1)
    {
      tmp[tmpqty] = board1[i];
      tmpqty++;
    }
  }

  (*pres) = ut_alloc_1d_int (tmpqty);
  ut_board_1d_int_memcpy ((*pres), tmpqty, tmp);

  ut_free_1d_int (tmp);

  (*psize) = tmpqty;
  return;
}

int *
ut_board_1d_int_cat (int *a, int sizea, int *b, int sizeb)
{
  int i, elt;
  int *res = ut_alloc_1d_int (sizea + sizeb);

  elt = 0;
  for (i = 0; i < sizea; i++)
  {
    res[elt] = a[i];
    elt++;
  }
  for (i = 0; i < sizeb; i++)
  {
    res[elt] = b[i];
    elt++;
  }

  return res;
}



int
ut_board_1d_int_deletencompress (int *board, int size, int value, int loop)
{
  int l;
  int i;
  int qty = 0;
  int pos;

  for (l = 1; l <= loop; l++)
  {
    pos = ut_board_1d_int_eltpos (board, size, value);
    if (pos != -1)
    {
      for (i = pos; i < size - 1; i++)
	board[i] = board[i + 1];
      qty++;
    }
    else
      break;
  }

  return qty;
}


void
ut_board_1d_int_memcpy_be (int* b, int *a, int bega, int enda)
{
  int i;
  int qty = enda - bega + 1;

  for (i = 0; i < qty; i++)
    b[i] = a[bega + i];

  return;
}

