/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */
 
#include "ut_free.h"

void
ut_free_1d_int (int *res)
{
  if (!res)
  {
    return;
  }

  free (res);

  return;
}

void
ut_free_1d_uint (unsigned int *res)
{
  if (!res)
  {
    return;
  }

  free (res);

  return;
}

void
ut_free_1d_sizet (size_t * res)
{
  if (!res)
  {
    return;
  }

  free (res);

  return;
}

void
ut_free_1d_float (float *res)
{
  if (!res)
  {
    return;
  }

  free (res);

  return;
}

void
ut_free_1d_double (double *res)
{
  if (!res)
  {
    return;
  }

  free (res);

  return;
}

void
ut_free_1d (double *res)
{
  if (!res)
  {
    return;
  }

  ut_free_1d_double (res);

  return;
}

void
ut_free_2d_int (int **board, unsigned int line)
{
  int i;

  if (!board)
  {
    return;
  }

  for (i = line - 1; i >= 0; i--)
    free (board[i]);

  free (board);

  return;
}

void
ut_free_2d_uint (unsigned int **board, unsigned int line)
{
  int i;

  if (!board)
  {
    return;
  }

  for (i = line - 1; i >= 0; i--)
    free (board[i]);

  free (board);

  return;
}

void
ut_free_2d_float (float **board, unsigned int line)
{
  int i;

  if (!board)
  {
    return;
  }

  for (i = line - 1; i >= 0; i--)
    free (board[i]);

  free (board);

  return;
}

void
ut_free_2d_double (double **board, unsigned int line)
{
  int i;

  if (!board)
  {
    return;
  }

  for (i = line - 1; i >= 0; i--)
    free (board[i]);

  free (board);

  return;
}

void
ut_free_2d (double **board, unsigned int line)
{
  if (!board)
  {
    return;
  }

  ut_free_2d_double (board, line);

  return;
}

void
ut_free_3d_int (int ***board, unsigned int line, unsigned int column)
{
  int i, j;

  if (!board)
  {
    return;
  }

  for (i = line - 1; i >= 0; i--)
    for (j = column - 1; j >= 0; j--)
      free (board[i][j]);

  for (i = line - 1; i >= 0; i--)
    free (board[i]);

  free (board);

  return;
}

void
ut_free_3d (double ***board, unsigned int line, unsigned int column)
{
  int i, j;

  if (!board)
  {
    return;
  }

  for (i = line - 1; i >= 0; i--)
    for (j = column - 1; j >= 0; j--)
      free (board[i][j]);

  for (i = line - 1; i >= 0; i--)
    free (board[i]);

  free (board);

  return;
}


void
ut_free_1d_char (char *string)
{
  if (!string)
  {
    return;
  }

  free (string);

  return;
}

void
ut_free_2d_char (char **board, unsigned int line)
{
  int i;

  if (!board)
  {
    return;
  }

  for (i = line - 1; i >= 0; i--)
    free (board[i]);

  free (board);

  return;
}

extern void
ut_free_1d_pfile (FILE ** files)
{
  if (!files)
  {
    return;
  }

  free (files);

  return;
}
