/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */
 
#include"ut_space.h"

double
ut_space_dist2d (double *a, double *b)
{
  return sqrt ((b[0] - a[0]) * (b[0] - a[0]) + (b[1] - a[1]) * (b[1] - a[1]));
}

double
ut_space_dist (double *a, double *b)
{
  return sqrt ((b[0] - a[0]) * (b[0] - a[0])
	       + (b[1] - a[1]) * (b[1] - a[1])
	       + (b[2] - a[2]) * (b[2] - a[2]));
}

int
ut_space_quad_convex (double *A, double *B, double *C, double *D)
{
  int res;
  double *S1 = ut_alloc_1d (3);
  double *S2 = ut_alloc_1d (3);
  double *S3 = ut_alloc_1d (3);
  double *S4 = ut_alloc_1d (3);
  double sc1, sc2, sc3, sc4;

  S1[0] = B[0] - A[0];
  S1[1] = B[1] - A[1];

  S2[0] = C[0] - B[0];
  S2[1] = C[1] - B[1];

  S3[0] = D[0] - C[0];
  S3[1] = D[1] - C[1];

  S4[0] = A[0] - D[0];
  S4[1] = A[1] - D[1];

  sc1 = S1[0] * S2[1] - S1[1] * S2[0];
  sc2 = S2[0] * S3[1] - S2[1] * S3[0];
  sc3 = S3[0] * S4[1] - S3[1] * S4[0];
  sc4 = S4[0] * S1[1] - S4[1] * S1[0];

  if ((ut_num_sgn (sc1) == ut_num_sgn (sc2))
      && (ut_num_sgn (sc2) == ut_num_sgn (sc3))
      && (ut_num_sgn (sc3) == ut_num_sgn (sc4)))
    res = 1;
  else
    res = 0;

  ut_free_1d (S1);
  ut_free_1d (S2);
  ut_free_1d (S3);
  ut_free_1d (S4);

  return res;
}

int
ut_space_quad_convex_int (int *A, int *B, int *C, int *D)
{
  int res;
  int *S1;
  int *S2;
  int *S3;
  int *S4;
  int sc1, sc2, sc3, sc4;

  /*
     if ( (A[0] == B[0] && A[1] == B[1]) || (A[0] == C[0] && A[1] == C[1])
     || (A[0] == D[0] && A[1] == D[1]) || (B[0] == C[0] && B[1] == C[1])
     || (B[0] == D[0] && B[1] == D[1]) || (C[0] == D[0] && C[1] == D[1]))
     return 1;
     else if ( (A[0] == C[0] && A[1] == C[1])
     ||  (B[0] == D[0] && B[1] == D[1]) )
     return 0;
   */

  S1 = ut_alloc_1d_int (3);
  S2 = ut_alloc_1d_int (3);
  S3 = ut_alloc_1d_int (3);
  S4 = ut_alloc_1d_int (3);

  S1[0] = B[0] - A[0];
  S1[1] = B[1] - A[1];

  S2[0] = C[0] - B[0];
  S2[1] = C[1] - B[1];

  S3[0] = D[0] - C[0];
  S3[1] = D[1] - C[1];

  S4[0] = A[0] - D[0];
  S4[1] = A[1] - D[1];

  sc1 = S1[0] * S2[1] - S1[1] * S2[0];
  sc2 = S2[0] * S3[1] - S2[1] * S3[0];
  sc3 = S3[0] * S4[1] - S3[1] * S4[0];
  sc4 = S4[0] * S1[1] - S4[1] * S1[0];

  if ((ut_num_sgn_int (sc1) == ut_num_sgn_int (sc2))
      && (ut_num_sgn_int (sc2) == ut_num_sgn_int (sc3))
      && (ut_num_sgn_int (sc3) == ut_num_sgn_int (sc4)))
    res = 1;
  else
    res = 0;

  ut_free_1d_int (S1);
  ut_free_1d_int (S2);
  ut_free_1d_int (S3);
  ut_free_1d_int (S4);

  return res;
}

void
ut_space_bary2d (double **P, int qty, double *B)
{
  int i, j;
  double tmp;

  for (i = 0; i < 2; i++)
  {
    tmp = 0;
    for (j = 0; j < qty; j++)
      tmp += P[j][i];

    B[i] = tmp / (double) qty;
  }

  return;
}

void
ut_space_bary2d_int (int **P, int qty, int *B)
{
  int i, j;
  int tmp;

  for (i = 0; i < 2; i++)
  {
    tmp = 0;
    for (j = 0; j < qty; j++)
      tmp += P[j][i];

    B[i] = ut_num_d2ri ((double) tmp / (double) qty);
  }

  return;
}

int
ut_space_indomain2d (double **P, int qty, double x, double y)
{
  int i, interqty, res;
  double *bary = ut_alloc_1d (2);
  double *A = ut_alloc_1d (2);

  A[0] = x;
  A[1] = y;

  ut_space_bary2d (P, qty, bary);

  interqty = 0;
  for (i = 0; i < qty - 1; i++)
    interqty += ut_space_quad_convex (P[i], bary, P[i + 1], A);
  interqty += ut_space_quad_convex (P[qty - 1], bary, P[0], A);

  if (ut_num_even (interqty))
    res = 1;
  else
    res = 0;

  ut_free_1d (bary);
  ut_free_1d (A);

  return res;
}

int
ut_space_indomain2d_int (int **P, int qty, int *A)
{
  int i, interqty, res;
  int *bary = ut_alloc_1d_int (2);

  ut_space_bary2d_int (P, qty, bary);

  interqty = 0;
  for (i = 0; i < qty - 1; i++)
    interqty += ut_space_quad_convex_int (P[i], bary, P[i + 1], A);
  interqty += ut_space_quad_convex_int (P[qty - 1], bary, P[0], A);

  if (ut_num_even (interqty))
    res = 1;
  else
    res = 0;

  ut_free_1d_int (bary);

  return res;
}



/* indicates the half-space whose a specified point belongs to. */
int
ut_space_planeside (double *P, double *C)
{
  if (P[1] * C[1] + P[2] * C[2] + P[3] * C[3] - P[0] < 0)
    return -1;
  else
    return 1;
}

/* Side indicates the half-space whose a specified point belongs to. */
int
ut_space_planeside_tol (double *P, double *C, double eps)
{
  if (P[1] * C[1] + P[2] * C[2] + P[3] * C[3] - P[0] < eps)
    return -1;
  else
    return 1;
}

/* Side2d indicates the half-space whose a specified point belongs to (2d). */
int
ut_space_planeside_2d (double *P, double *C)
{
  if (P[1] * C[1] + P[2] * C[2] - P[0] < 0)
    return -1;
  else
    return 1;
}


/* ut_space_trianglearea area returns the area of the triangle
 * whose vertex coordinates are specified as input data:
 * vertex i: coo[i][1,2,3] (i=1,2,3)
 * The used formula is:
 * A = || (P2-P1) vect. (P3-P1) || /2
 */
double
ut_space_trianglearea (double *p1, double *p2, double *p3)
{
  int i;
  double res;
  double * tmp1 = ut_alloc_1d (3);
  double * tmp2 = ut_alloc_1d (3);
  double * tmp3 = ut_alloc_1d (3);

  printf ("In ut_space_trianglearea\n");
  printf ("p1: %f %f %f\n", p1[0], p1[1], p1[2]);
  printf ("p2: %f %f %f\n", p2[0], p2[1], p2[2]);
  printf ("p3: %f %f %f\n", p3[0], p3[1], p3[2]);

  for (i = 0; i <= 2; i++)
    {
      tmp1[i] = p2[i] - p1[i];
      tmp2[i] = p3[i] - p1[i];
    }
  ut_vector_vectprod (tmp1, tmp2, tmp3);

  res = 0.5 * ut_vector_norm (tmp3);

  ut_free_1d (tmp1);
  ut_free_1d (tmp2);
  ut_free_1d (tmp3);

  return res;
}

void ut_space_trianglenormal (double * p1, double * p2, double * p3 , double* n)
{
  int i;
  double * a = ut_alloc_1d (4);
  double * b = ut_alloc_1d (4);
  double norm;

  for (i = 0; i < 3; i++)
    {
      a[i] = p2[i] - p1[i];
      b[i] = p3[i] - p2[i];
    }
  ut_vector_vectprod (a, b, n);
  norm = ut_vector_norm (n);

  for (i = 0; i < 3; i++)
    n[i] /= norm;

  ut_free_1d (a);
  ut_free_1d (b);

  return;
}


int ut_space_pointaligned (double* p1, double* p2, double* p3, double eps)
{
  int i;
  double angle;
  double * v1 = ut_alloc_1d (3);
  double * v2 = ut_alloc_1d (3);
  
  for (i = 0; i < 3; i++)
    v1[i] = p2[i] - p1[i];

  for (i = 0; i < 3; i++)
    v2[i] = p3[i] - p1[i];
  
  angle = ut_vector_angle (v1, v2);
  
  if (angle < eps)
    return 1;
  else
    return 0;
}

