/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */
 
#include"ut_string.h"

int
ut_string_fnr (char *string, char find, char replace)
{
  int i, length, res;

  res = 0;
  length = strlen (string);
  for (i = 0; i <= length - 1; i++)
    if (string[i] == find)
    {
      string[i] = replace;
      res = 1;
    }

  return res;
}

int
ut_string_comp (char *in, char **list, int qty, char *out)
{
  int i;
  int res = 0;

  /* Checking if it matches exactly */
  for (i = 1; i <= qty; i++)	/* for every list parameter */
    if (strcmp (in, list[i]) == 0)
    {
      sprintf (out, "%s", list[i]);
      return 0;
    }

  for (i = 1; i <= qty; i++)	/* for every list parameter */
  {
    /* else, checking if one possibility could be ok */
    if (strncmp (in, list[i], strlen (in)) == 0)
    {
      if (res == 0)		/* this is a good possibility. */
	res = i;
      else			/* ie there are several possibilities: nok */
	return 1;
    }
  }

  if (res != 0)
  {
    sprintf (out, "%s", list[res]);	/* this is the goo possibility. */
    return 0;
  }
  else
    return -1;			/* there is no possibility. */
}

char *
ut_string_extension (char *in)
{
  char *ext;

  int i;
  int NoCin, NoCext;
  int DashPos = -1;

  NoCin = strlen (in);

  for (i = NoCin - 1; i >= 0; i--)
  {
    if (in[i] == '.')
    {
      DashPos = i;
      break;
    }
  }

  if (DashPos == -1)
    ext = NULL;
  else
  {
    NoCext = NoCin - DashPos;

    ext = ut_alloc_1d_char (NoCext + 1);

    for (i = 0; i <= NoCext - 1; i++)
      ext[i] = in[i + DashPos];
    ext[NoCext] = '\0';
  }

  return ext;
}

char *
ut_string_addextension (char *in, char *ext)
{
  char *out;

  unsigned int NoCin;		/* nb of characters of in */
  unsigned int NoCext;		/* nb of characters of extin */

  NoCin = strlen (in);
  NoCext = strlen (ext);

  out = ut_alloc_1d_char (NoCin + NoCext + 1);

  out = strncpy (out, in, NoCin);
  out = strcat (out, ext);
  out[NoCin + NoCext] = '\0';

  return out;
}



char *
ut_string_body (char *in)
{
  char *body;

  int i;
  int NoCin, NoCbody;
  int DashPos = -1;

  NoCin = strlen (in);

  for (i = NoCin - 1; i >= 0; i--)
  {
    if (in[i] == '.')
    {
      DashPos = i;
      break;
    }
  }

  if (DashPos == -1)
    NoCbody = NoCin;
  else
    NoCbody = DashPos;

  body = ut_alloc_1d_char (NoCbody + 1);

  for (i = 0; i <= NoCbody - 1; i++)
    body[i] = in[i];
  body[NoCbody] = '\0';

  return body;
}

int
ut_string_untilchar (char *string, char c, int *ppos, char *res)
{
  char *copy = ut_alloc_1d_char (strlen (string) + 1);
  /* char* copy ; */

  strcpy (copy, string + (*ppos));

  /* character `c' is replaced by `\0', which has for effect
   * to truncate the string -- it rests what we need.*/
  ut_string_fnr (copy, c, '\0');

  strcpy (res, copy);

  *ppos += strlen (copy);

  ut_free_1d_char (copy);

  if (strlen (res) > 0)
    return 1;
  else
    return 0;
  /*

     if ((*ppos) == (signed int)strlen (string))
     return 0;

     for (i = (*ppos); i < strlen (string) + 1; i++)
     if (string[i] != ',' && string[i] != '\0')
     {
     ext[j] = string[i];
     j++;
     }
     else
     break;

     (*ppos) = i + 1;
     res[j] = '\0';

     printf ("ext = %s\n", res);
   */
}


int
ut_string_thischarqty (char *string, char c)
{
  int i, res = 0;

  for (i = 0; i < (int) strlen (string); i++)
    if (string[i] == c)
      res++;

  return res;
}


int
ut_string_thischarpos (char *string, char c, int nb)
{
  int pos, tmp;

  /* by default, i.e. if nb == 0 or occurrence not found */
  pos = -1;

  if (nb != 0)
  {
    tmp = 0;
    for (pos = 0; pos < (int) strlen (string); pos++)
      if (string[pos] == c)
      {
	tmp++;
	if (tmp == nb)		/* the good occurence is found */
	  break;
      }
  }

  return pos;
}

void
ut_string_separate (char* string, char c, char*** parts, int* pqty)
{
  int i, pos;
  char** tmp = ut_alloc_2d_char (strlen(string), strlen(string));
  char*  val = ut_alloc_1d_char (strlen(string));
  
  (*pqty) = 0;
  pos = 0;
  while (ut_string_untilchar (string, c, &pos, val) == 1)
  {
    pos++;
    sprintf (tmp[(*pqty)++], "%s", val);
  }

  (*parts) = ut_alloc_1d_pchar (*pqty);
  for (i = 0; i < (*pqty); i++)
  {
    (*parts)[i] = ut_alloc_1d_char (strlen (tmp[i]));
    sprintf ((*parts)[i], "%s", tmp[i]);
  }

  ut_free_2d_char (tmp, strlen(string));
  ut_free_1d_char (val);

  return;
}
