/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */
 
#include"ut_vector.h"

double
ut_vector_norm (double *a)
{
  return sqrt (a[0] * a[0] + a[1] * a[1] + a[2] * a[2]);
}

double
ut_vector_int_norm (int *a)
{
  return sqrt ((double) a[0] * (double) a[0] + (double) a[1] * (double) a[1] +
	       (double) a[2] * (double) a[2]);
}

double
ut_vector_scalprod (double *a, double *b)
{
  return a[0] * b[0] + a[1] * b[1] + a[2] * b[2];
}

void
ut_vector_vectprod (double *a, double *b, double *c)
{
  c[0] = a[1] * b[2] - a[2] * b[1];
  c[1] = a[2] * b[0] - a[0] * b[2];
  c[2] = a[0] * b[1] - a[1] * b[0];

  return;
}

double
ut_vector_angle_rad (double *a, double *b)
{
  double angle;

  angle = ut_vector_scalprod (a, b);
  angle /= ut_vector_norm (a);
  angle /= ut_vector_norm (b);
  angle = acos (angle);
  if (isNaN (angle) == 1)
    angle = 0;

  return angle;
}

double
ut_vector_angle (double *a, double *b)
{
  return (180 / PI) * ut_vector_angle_rad (a, b);
}

double
ut_vector_int_angle_rad (int *a, int *b)
{
  int i;
  double angle;
  double *a2 = ut_alloc_1d (3);
  double *b2 = ut_alloc_1d (3);

  for (i = 0; i < 3; i++)
    a2[i] = (double) a[i];

  for (i = 0; i < 3; i++)
    b2[i] = (double) b[i];

  angle = ut_vector_scalprod (a2, b2);
  angle /= ut_vector_norm (a2);
  angle /= ut_vector_norm (b2);
  angle = acos (angle);
  if (isNaN (angle) == 1)
    angle = 0;

  ut_free_1d (a2);
  ut_free_1d (b2);

  return angle;
}

double
ut_vector_int_angle (int *a, int *b)
{
  return (180 / PI) * ut_vector_int_angle_rad (a, b);
}
